<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access
defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Sellacious\Cache\Reader\ProductsCacheReader;
use Sellacious\Event\EventHelper;
use Sellacious\Media\Image\ImageHelper;
use Sellacious\Media\Image\ResizeImage;
use Sellacious\StoreTimings\Slots;

/**
 * Sellacious seller helper
 *
 * @since   1.0.0
 */
class SellaciousHelperSeller extends SellaciousHelperBase
{
	/**
	 * List of days in a week
	 *
	 * @since  2.0.0
	 */
	const DAYS = array('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');
	const DAYS_LONG = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');

	/**
	 * Generate SQL query from the given filters and other clauses
	 *
	 * @param   array  $filters
	 *
	 * @return  JDatabaseQuery
	 *
	 * @since   1.0.0
	 */
	public function getListQuery($filters)
	{
		$db    = $this->db;
		$query = parent::getListQuery($filters);

		if (!isset($filters['list.from']))
		{
			$query->join('INNER', $db->qn('#__users', 'u') . ' ON a.user_id = u.id');
		}

		return $query;
	}

	/**
	 * Get the seller category for the given user
	 *
	 * @param   int   $userId
	 * @param   bool  $useDefault
	 * @param   bool  $full
	 *
	 * @return  int|stdClass
	 *
	 * @since   1.5.1
	 */
	public function getCategory($userId, $useDefault = false, $full = false)
	{
		$filter   = array(
			'list.select' => 'c.*',
			'list.join'   => array(array('inner', '#__sellacious_categories AS c ON c.id = a.category_id')),
			'user_id'     => $userId,
		);
		$category = $this->loadObject($filter);

		if (!$category && $useDefault)
		{
			$category = $this->helper->category->getDefault('seller');
		}

		return $category ? ($full ? $category : $category->id) : null;
	}

	/**
	 * Check whether the given user is an (active, optionally) seller or not
	 *
	 * @param   int   $user_id  User Id to query, default current user
	 * @param   bool  $active   Check enabled state
	 *
	 * @return  int
	 *
	 * @since   1.0.0
	 */
	public function is($user_id = null, $active = true)
	{
		$me     = JFactory::getUser($user_id);
		$filter = array('list.select' => 'a.category_id', 'user_id' => $me->id);

		if ($active)
		{
			$filter['state'] = 1;
		}

		return $this->loadResult($filter);
	}

	/**
	 * Set shippable locations for the selected seller
	 *
	 * @param   int    $seller_uid  Seller user id
	 * @param   int[]  $pks         Geo-location ids
	 *
	 * @since   1.0.0
	 */
	public function setShipLocations($seller_uid, $pks)
	{
		if ($seller_uid)
		{
			// If none selected, set to all
			if (count($pks) == 0)
			{
				// Calling method should handle this instead?
				$pks = array(0);
			}

			$current = $this->getShipLocations($seller_uid);

			$new    = array_filter(array_diff($pks, $current));
			$remove = array_filter(array_diff($current, $pks));

			if (count($remove))
			{
				$query = $this->db->getQuery(true);
				$query->delete('#__sellacious_seller_shippable')
					->where('seller_uid = ' . (int) $seller_uid)
					->where('gl_id IN (' . implode(',', $remove) . ')');
				$this->db->setQuery($query)->execute();
			}

			if (count($new))
			{
				$query = $this->db->getQuery(true);
				$query->insert('#__sellacious_seller_shippable')
					->columns(array('seller_uid', 'gl_id', 'state'));

				foreach ($new as $i)
				{
					$query->clear('values')->values(sprintf('%d, %d, %d', $seller_uid, $i, 1));
					$this->db->setQuery($query)->execute();
				}
			}
		}
	}

	/**
	 * Return the list of shippable geo locations for the selected seller
	 *
	 * @param   int   $seller_uid  Seller user id
	 * @param   bool  $grouped     Whether to group by geo location type
	 *
	 * @return  array
	 *
	 * @since   1.0.0
	 */
	public function getShipLocations($seller_uid, $grouped = false)
	{
		$query = $this->db->getQuery(true);

		$query->select('a.gl_id')
			->from('#__sellacious_seller_shippable AS a')
			->where('a.seller_uid = ' . (int) $seller_uid)
			->where('a.gl_id > 0');

		if ($grouped)
		{
			$query->select('g.type')
				->join('left', '#__sellacious_locations AS g ON (CASE WHEN g.type = \'zip\' THEN g.title = a.gl_id ELSE g.id = a.gl_id END)');

			$ids  = array();
			$rows = $this->db->setQuery($query)->loadObjectList();

			foreach ($rows as $row)
			{
				$ids[$row->type][] = (int) $row->gl_id;
			}
		}
		else
		{
			$ids = $this->db->setQuery($query)->loadColumn();
			$ids = array_map('intval', $ids);
		}

		return $ids;
	}

	/**
	 * Check an address whether it is allowed for the shipping by the selected seller
	 *
	 * @param   int|object  $address     The address object or address id
	 * @param   int         $seller_uid  The type/context of address. Acceptable values are: BT, ST
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   1.4.0
	 */
	public function isAddressShippable($address, $seller_uid)
	{
		// Should we use seller~address caching here? Not doing to prevent any undesirable effect until sure.
		if (is_numeric($address))
		{
			$address = $this->helper->user->getAddressById($address);
		}

		if (empty($address->id))
		{
			return false;
		}

		$allowed = $this->getShipLocations($seller_uid, true);

		if (empty($allowed))
		{
			return true;
		}

		if (is_bool($allow = $this->helper->location->isAllowed($address->country, $allowed)))
		{
			return $allow;
		}

		if (is_bool($allow = $this->helper->location->isAllowed($address->state_loc, $allowed)))
		{
			return $allow;
		}

		if (is_bool($allow = $this->helper->location->isAllowed($address->district, $allowed)))
		{
			return $allow;
		}

		/*
		 * If no preference is set at all it would be allowed already and we won't reach here.
		 * Therefore, if we are here it means either all upper level fields are blank or allowed by selected zip.
		 * Hence any region constraint in upper level due to zip must have already checked against.
		 * Also selected zip would be blank as this would not bring inherit up to here.
		 */
		if (!$address->zip)
		{
			return true;
		}

		$selected = ArrayHelper::getValue($allowed, 'zip', array(), 'array');
		$zipCodes = (array) $this->loadColumn(array('list.select' => 'a.title', 'id' => $selected));

		return count($zipCodes) == 0 || in_array($address->zip, $zipCodes);
	}

	/**
	 * Add seller commissions for the given seller for each product category maps
	 *
	 * @param   int    $sellerUid         Seller user id
	 * @param   array  $sellerCommission  Array [product_category_id => commission]
	 *
	 * @return  bool
	 *
	 * @throws  \Exception
	 *
	 * @since   1.5.0
	 */
	public function setCommissions($sellerUid, $sellerCommission)
	{
		$commissions = $this->getCommissions($sellerUid);

		foreach ($sellerCommission as $productCatid => $commission)
		{
			$old = ArrayHelper::getValue($commissions, $productCatid);

			$this->setCommission($productCatid, $sellerUid, $commission, $old);
		}

		return true;
	}

	/**
	 * Set the commission value for the given product category and seller category map
	 *
	 * @param   int     $productCatid  The product category id that would be affected
	 * @param   int     $sellerUid     The seller category id that would be affected
	 * @param   string  $commission    The new commission rate (float value or a string containing a float suffixed by % sign)
	 * @param   string  $old           The old commission rate (float value or a string containing a float suffixed by % sign)
	 *
	 * @return  void
	 * @throws  Exception
	 *
	 * @since   1.5.0
	 */
	protected function setCommission($productCatid, $sellerUid, $commission, $old)
	{
		$query = $this->db->getQuery(true);
		$zero  = trim($commission, '% ') == 0;

		// Insert if has value and not already exists
		if (!isset($old))
		{
			if (!$zero)
			{
				$query->insert('#__sellacious_seller_commissions')
					->columns('seller_uid, product_catid, commission')
					->values(implode(', ', $this->db->q(array($sellerUid, $productCatid, $commission))));

				$this->db->setQuery($query)->execute();
			}
		}
		else
		{
			// Delete if ZERO, and already exists
			if ($zero)
			{
				$query->delete('#__sellacious_seller_commissions')
					->where('seller_uid = ' . $this->db->q($sellerUid))
					->where('product_catid = ' . $this->db->q($productCatid));

				$this->db->setQuery($query)->execute();
			}
			// Update only if modified
			elseif ($commission != $old)
			{
				$query->update('#__sellacious_seller_commissions')
					->set('commission = ' . $this->db->q($commission))
					->where('seller_uid = ' . $this->db->q($sellerUid))
					->where('product_catid = ' . $this->db->q($productCatid));

				$this->db->setQuery($query)->execute();
			}
		}
	}

	/**
	 * Fetch seller commissions for the given seller for each product category maps
	 *
	 * @param   int  $sellerUid  Seller user id
	 *
	 * @return  array  Commissions for each product category
	 *
	 * @since   1.5.0
	 */
	public function getCommissions($sellerUid)
	{
		$query = $this->db->getQuery(true);

		$query->select('product_catid, commission')
			->from('#__sellacious_seller_commissions')
			->where('seller_uid = ' . $this->db->q($sellerUid));

		$items  = $this->db->setQuery($query)->loadObjectList();
		$result = ArrayHelper::getColumn((array) $items, 'commission', 'product_catid');

		return $result;
	}

	/**
	 * Fetch seller commission for the given product category
	 *
	 * @param   int   $sellerUid     Seller category id
	 * @param   int   $productCatid  Array [product_category_id => commission]
	 * @param   bool  $inherit       Whether to inherit value from parent category
	 *
	 * @return  mixed  The commission amount/rate
	 *
	 * @since   1.5.0
	 */
	public function getCommission($sellerUid, $productCatid, $inherit = false)
	{
		$query = $this->db->getQuery(true);

		$query->select('product_catid, commission')
			->from('#__sellacious_seller_commissions')
			->where('seller_uid = ' . $this->db->q($sellerUid))
			->where('product_catid = ' . $this->db->q($productCatid));

		$result = $this->db->setQuery($query)->loadResult();

		if (!$result && $inherit)
		{
			$filter   = array('list.select' => 'a.parent_id', 'id' => $productCatid);
			$parentId = $this->helper->category->loadResult($filter);
			$result   = $this->getCommission($sellerUid, $parentId, $inherit);
		}

		return $result;
	}

	/**
	 * Get Products count of a particular Seller
	 *
	 * @param   int  $seller_uid  Seller user id
	 *
	 * @return  int|mixed
	 *
	 * @since   1.6.0
	 */
	public function getSellerProductCount($seller_uid)
	{
		$multiVariant = (int) $this->helper->config->get('multi_variant');

		$cache = new ProductsCacheReader;

		$cache->filterValue('seller_uid', (int) $seller_uid, '=');
		$cache->filterValue('product_active', 1, '=');
		$cache->filterValue('seller_active', 1, '=');
		$cache->filterValue('listing_active', 1, '=');
		$cache->filterValue('is_selling', 1, '=');
		$cache->filterValue('is_visible', 0, '>');

		if ($multiVariant < 2)
		{
			$cache->filterValue('variant_id', 0, '=');
		}

		return $cache->getTotal();
	}

	/**
	 * Method to get seller stores for a module
	 *
	 * @param   Registry  $params  The module parameters
	 * @param   string    $type    The module type
	 *
	 * @return  \stdClass[]
	 *
	 * @since   1.6.0
	 *
	 * @deprecated   This will be moved to module's helper class
	 */
	public function getModStores($params, $type = 'stores')
	{
		$default_seller = $this->helper->config->get('default_seller', -1);
		$multi_seller   = $this->helper->config->get('multi_seller', 0);

		$limit       = $params->get('total_records', '50');
		$category_id = $params->get('category_id', '0');
		$ordering    = $params->get('ordering', '3');
		$orderBy     = $params->get('orderby', 'DESC');
		$filters     = array();

		$filters['list.select'][] = 'a.*, u.name, u.username, u.email';

		switch ($ordering)
		{
			case "1":
				$ord = 'a.title ' . $orderBy;
				break;
			case "2":
				$ord = 'rand() ';
				break;
			default:
				$ord = 'rand() ';
		}

		$filters['list.where'][] = 'a.state = 1';
		$filters['list.where'][] = 'u.block = 0';

		if ($category_id)
		{
			$filters['list.where'][] = 'a.category_id = ' . (int) $category_id;
		}

		if (!$multi_seller)
		{
			$filters['list.where'][] = 'a.user_id = ' . $default_seller;
		}

		$filters['list.order'][] = $ord;
		$filters['list.start']   = 0;
		$filters['list.limit']   = $limit;

		$arguments = array(
			'context' => 'mod_sellacious_stores',
			'query'   => $this->helper->seller->getListQuery($filters),
			'state'   => new JObject,
		);

		EventHelper::trigger('onProcessListQuery', $arguments);

		$stores = $this->helper->seller->loadObjectList($filters);

		foreach ($stores as $store)
		{
			$store->rating        = $this->helper->rating->getSellerRating($store->user_id);
			$store->product_count = $this->getSellerProductCount($store->user_id);
		}

		return $stores;
	}

	/**
	 * Get Reviews of a particular Seller
	 *
	 * @param   int  $sellerUid  Seller user id
	 * @param   int  $limit      Maximum number fo reviews to fetch
	 *
	 * @return  int|mixed
	 *
	 * @since   1.7.0
	 */
	public function getSellerReviews($sellerUid, $limit = 0)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select($db->qn(array('a.author_id', 'a.author_name', 'a.buyer', 'a.rating', 'a.title', 'a.comment', 'a.created')))
			->from('#__sellacious_ratings as a')
			->where('a.seller_uid = ' . (int) $sellerUid)
			->where('a.state = 1')
			->where('a.type = ' . $db->q('seller'));

		$db->setQuery($query, 0, $limit);

		$result = $db->loadObjectList();

		return $result;
	}

	/**
	 * Parse stores for rendering stores layout
	 *
	 * @param   array  $items  Stores
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	public function parseStoresForLayout($items)
	{
		$stores = array();

		foreach ($items as $item)
		{
			$store = new Registry($item);

			$filter    = array('list.select' => '*', 'user_id' => $item->user_id);
			$record_id = $this->helper->seller->loadResult($filter);

			$logo  = ImageHelper::getImage('sellers', $record_id, 'logo') ?: ImageHelper::getImage('user', $item->user_id, 'avatar');
			$image = $logo ? $logo->getUrl() ?: ImageHelper::getBlank('sellers', 'logo')->getUrl() : ImageHelper::getBlank('sellers', 'logo')
				->getUrl();

			$url = JRoute::_('index.php?option=com_sellacious&view=store&id=' . $item->user_id);

			$filter = array('list.select' => 'store_name', 'user_id' => $item->user_id);
			$name   = $this->helper->seller->loadResult($filter);
			if ($name)
			{
				$store->set('name', $name);
			}

			$store->set('logo', $image);
			$store->set('url', $url);

			$stores[] = $store;
		}

		return $stores;
	}

	/**
	 * Method to get seller Timings
	 *
	 * @param   int     $sellerUid   The seller user id
	 * @param   string  $type        The type of timing (store, delivery, etc.)
	 * @param   int     $activeOnly  Whether to get all timings or only published
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getTimings($sellerUid, $type, $activeOnly = 0)
	{
		$db   = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('a.id, a.week_day, a.full_day, a.from_time, a.to_time, a.state, a.params');
		$query->from($db->qn('#__sellacious_seller_timings', 'a'));
		$query->where('a.seller_uid = ' . (int) $sellerUid);
		$query->where('a.type = ' . $db->quote($type));

		if ($activeOnly)
		{
			$query->where('a.state = 1');
		}

		$db->setQuery($query);

		$timings = $db->loadAssocList();

		$seller   = JFactory::getUser($sellerUid);
		$timezone = $seller->getParam('timezone', 'UTC');

		foreach ($timings as &$timing)
		{
			// If UTC timings are not available in database, calculate them
			if (empty($timing['params']))
			{
				$params = new Registry();
				$slots = new Slots($timezone);
				$slots->addTime(self::DAYS[$timing['week_day']], JFactory::getDate($timing['from_time'])->format('H:i'), JFactory::getDate($timing['to_time'])->format('H:i'));

				// Convert To UTC
				$slotsUTC = $slots->convert('UTC');
				$params->set('timings_utc', $slotsUTC->toArray());

				$timing['params']    = $params->toString();
				$timing['slot_info'] = null;
			}
		}

		return $timings;
	}

	/**
	 * Method to get timing of particular week day
	 *
	 * @param   int     $sellerUid  The seller user id
	 * @param   int     $weekDay    The day number of the week
	 * @param   string  $type       Type of timing
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	public function getTiming($sellerUid, $weekDay = 0, $type = 'delivery_hours')
	{
		$db   = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('a.id, a.from_time, a.to_time, a.full_day');
		$query->from($db->qn('#__sellacious_seller_timings', 'a'));
		$query->where('a.seller_uid = ' . (int) $sellerUid . ' AND a.state = 1');
		$query->where('a.type = ' . $db->q($type) . ' AND a.week_day = ' . (int) $weekDay);

		$db->setQuery($query);

		return $db->loadObject();
	}

	/**
	 * Method to get store availability for the timing
	 *
	 * @param    array   $timings   The Seller timings data
	 * @param    array   $literals  Language literals for each availablity status
	 *
	 * @return   string
	 *
	 * @throws   Exception
	 *
	 * @since    2.0.0
	 */
	public function getStoreAvailability($timings, $literals = array())
	{
		$notAvailable = ArrayHelper::getValue($literals, 'not_available', 'COM_SELLACIOUS_SELLER_TIMINGS_NOT_AVAILABLE');
		$available    = ArrayHelper::getValue($literals, 'available', 'COM_SELLACIOUS_SELLER_TIMINGS_AVAILABLE');
		$availableIn  = ArrayHelper::getValue($literals, 'available_in', 'COM_SELLACIOUS_SELLER_TIMINGS_AVAILABLE_IN');

		if (empty($timings))
		{
			// If not timings available, assumed to be open
			return '<span class=\'timing_available\'>' . JText::_($available) . '</span>';
		}

		$now = JFactory::getDate();

		foreach ($timings as $timing)
		{
			$day    = $now->format('D');
			$params = new Registry($timing['params']);
			$utc    = $params->get('timings_utc', array());

			if (!empty($utc))
			{
				$timingUtc = array_values(array_filter($utc, function ($item) use ($day) {
					return $item->day == $day;
				}));

				if (!empty($timingUtc))
				{
					$timingUtc           = array_shift($timingUtc);
					$timing['from_time'] = $timingUtc->start;
					$timing['to_time']   = $timingUtc->end;
				}
			}

			if ($timing['week_day'] == ($now->format('N') - 1))
			{
				// If Timings is 24 hours, then always available
				if ($timing['full_day'] == 1)
				{
					return '<span class=\'timing_available\'>' . JText::_($available) . '</span>';
				}

				$from = JFactory::getDate($timing['from_time']);
				$to   = JFactory::getDate($timing['to_time']);

				if ($from <= $now && $to >= $now)
				{
					return '<span class=\'timing_available\'>' . JText::_($available) . '</span>';
				}
				elseif ($now < $from)
				{
					$diff       = $now->diff($from);
					$diffFormat = $diff->format('%i') . ' Minute(s) ';

					if (!empty($diff->format('%h')))
					{
						$diffFormat = $diff->format('%h') . ' Hour(s) ' . $diffFormat;
					}

					return '<span class=\'timing_available_in\'>' . JText::sprintf($availableIn, $diffFormat) . '</span>';
				}
			}
			elseif ($timing['week_day'] > ($now->format('N') - 1) && $timing['state'])
			{
				$diffDays = $timing['week_day'] - ($now->format('N') - 1);
				$date2    = JFactory::getDate($timing['from_time'])->modify('+' . $diffDays . ' day');
				$diff     = $now->diff($date2);

				$diffFormat = $diff->format('%i') . ' Minute(s) ';

				if (!empty($diff->format('%h')))
				{
					$diffFormat = $diff->format('%h') . ' Hour(s) ' . $diffFormat;
				}

				if (!empty($diff->format('%d')))
				{
					$diffFormat = $diff->format('%d') . ' Day(s) ' . $diffFormat;
				}

				return '<span class=\'timing_available_in\'>' . JText::sprintf($availableIn, $diffFormat) . '</span>';
			}
		}

		return '<span class=\'timing_unavailable\'>' . JText::_($notAvailable) . '</span>';
	}

	/**
	 * Method to save timings
	 *
	 * @param   array   $data       The form data
	 * @param   string  $type       Type of timing (store, delivery, etc.)
	 * @param   int     $sellerUid  The seller user id
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function saveTimings($data, $type, $sellerUid)
	{
		$user     = JFactory::getUser();
		$seller   = JFactory::getUser($sellerUid);
		$timezone = $seller->getParam('timezone', $user->getParam('timezone', 'UTC'));

		foreach ($data as $weekday => $timing)
		{
			$table = SellaciousTable::getInstance('SellerTimings');
			$table->load(array('seller_uid' => $sellerUid, 'type' => $type, 'week_day' => $weekday));

			$params = new Registry($table->get('params', array()));

			if (isset($timing['full_day']) && $timing['full_day'] == 1)
			{
				$timing['from_time'] = '00:00';
				$timing['to_time']   = '23:59';
			}

			$timing['type']       = $type;
			$timing['seller_uid'] = $sellerUid;
			$timing['full_day']   = isset($timing['full_day']) ? $timing['full_day'] : 0;
			$timing['from_time']  = JFactory::getDate($timing['from_time'])->format('H:i:s');
			$timing['to_time']    = JFactory::getDate($timing['to_time'])->format('H:i:s');

			$slots = new Slots($timezone);
			$slots->addTime(self::DAYS[$weekday], JFactory::getDate($timing['from_time'])->format('H:i'), JFactory::getDate($timing['to_time'])->format('H:i'));

			// Convert To UTC
			$slotsUTC = $slots->convert('UTC');
			$params->set('timings_utc', $slotsUTC->toArray());

			$timing['params'] = $params->toArray();

			if (!isset($timing['week_day']))
			{
				$timing['state']    = 0;
				$timing['week_day'] = $weekday;
			}
			else
			{
				$timing['state'] = 1;
			}

			$table->bind($timing);
			$table->check();
			$table->store();
		}
	}

	/**
	 * Fetch Sellers suggestion list
	 *
	 * @param   string  $key             Searched key
	 * @param   array   $userIds         The preselected user Ids
	 * @param   array   $offset          Search result list offset and Max number of result items to return
	 * @param   array   $defaultOptions  Default options
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public function getSuggestions($key, $userIds = null, $offset = null, $defaultOptions = null)
	{
		$multi_seller = $this->helper->config->get('multi_seller', 0);

		if (!$multi_seller)
		{
			// Load only default seller when multi-seller is off.
			$default_seller = $this->helper->config->get('default_seller');
			$filter         = array(
				'list.select' => 'u.id, IF (a.title != \'\', a.title, u.name) as title',
				'list.where'  => array('a.state = 1', 'u.block = 0', 'u.id = ' . (int) $default_seller),
			);
		}
		else
		{
			$filter = array(
				'list.select' => 'u.id, IF (a.title != \'\', a.title, u.name) as title',
				'list.where'  => array('a.state = 1', 'u.block = 0'),
				'list.group'  => 'u.id, u.username, u.email',
				'list.order'  => 'u.name ASC'
			);
		}

		if (strlen($key))
		{
			$match = $this->db->q('%' . $this->db->escape($key) . '%', false);

			$filter['list.where'][] = '(a.title LIKE ' . $match . ' OR u.name LIKE ' . $match . ')';
		}

		if ($userIds)
		{
			$filter['user_id'] = $userIds;
		}

		if ($offset && is_array($offset) && isset($offset[1]))
		{
			$filter['list.start'] = (int) $offset[0];
			$filter['list.limit'] = (int) $offset[1];
		}

		$sellers = $this->loadObjectList($filter);

		if ($defaultOptions)
		{
			foreach ($defaultOptions as $defaultOption)
			{
				$defaultOption = (object) $defaultOption;

				if ((!$userIds || in_array($defaultOption->value, $userIds)))
				{
					$seller           = new stdClass();
					$seller->id       = isset($defaultOption->value) ? $defaultOption->value : '';
					$seller->disabled = isset($defaultOption->disabled) ? $defaultOption->disabled : '';
					$seller->title    = '<span' . (isset($defaultOption->class) ? ' class="' . $defaultOption->class . '">' : '') . $defaultOption->text . '</span>';

					array_unshift($sellers, $seller);
				}
			}
		}

		return $sellers;
	}
}
