<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Aditya Chakraborty <info@bhartiy.com> - http://www.bhartiy.com
 */

namespace Sellacious\Form\Handler;

// No direct access
defined('_JEXEC') or die;

use Exception;
use JFactory;
use JFormHelper;
use JHtml;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use JTable;
use JText;
use Sellacious\Cart;
use Sellacious\Cart\Item\Internal;
use Sellacious\Form\FormData;
use Sellacious\StoreTimings\StoreTimingsHelper;
use stdClass;

/**
 * Class for Delivery
 *
 * @since   2.0.0
 */
class DeliveryHandler extends AbstractCheckoutQuestionsFormHandler
{
	/**
	 * Constructor
	 *
	 * @param   string  $name  The handler name
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct($name)
	{
		parent::__construct($name);

		$this->contexts->set('products', true);
		$this->contexts->set('product', true);
		$this->contexts->set('cart_summary', true);
		$this->setItemised(true);
	}

	/**
	 * Method to get the name of this checkout questions form object. Must be unique for each context
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getName()
	{
		return 'delivery';
	}

	/**
	 * Method to prepare checkout questions form
	 *
	 * @param  string  $itemUid  Product code
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function prepareForm($itemUid = null)
	{
		if (!$itemUid || !$this->helper->product->parseCode($itemUid, $productId, $variantId, $sellerUid))
		{
			return;
		}

		// If Product seller attributes not found, then psx will give empty values in its Registry object
		$psx              = $this->getPsxAttributes($productId, $sellerUid);
		$disable_checkout = $this->helper->config->get('disable_checkout');
		$stock            = $psx->get('stock', 0);
		$minQty           = $psx->get('quantity_min', 0);
		$disable_stock    = $psx->get('disable_stock', 1);
		$checkStock       = $stock > 0 && (!$minQty || $stock >= $minQty);
		$priceType        = $psx->get('pricing_type');

		if (!($disable_checkout == 0 && ($checkStock || $disable_stock == 1)))
		{
			return;
		}

		// Allowed price types
		if (!in_array($priceType, array('basic', 'dynamic', 'flat', 'free')))
		{
			return;
		}

		$sellerTimeZone = StoreTimingsHelper::getUserTimezone($sellerUid);
		$now            = JFactory::getDate('now', $sellerTimeZone);
		$weekDay        = $this->helper->core->getWeekDay($now);

		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		$seller       = $this->helper->seller->getItem(array('user_id' => $sellerUid));
		$sellerParams = new Registry($seller->params);
		$slotLimit    = $sellerParams->get('slot_limit', 0);
		$prepTime     = $sellerParams->get('delivery_prep_time', null);
		$slotWindow   = $sellerParams->get( 'slot_window', null);

		if ($timeSelection == 2 && (!isset($slotWindow->m) || $slotWindow->m <= 0))
		{
			return;
		}

		if ($timeSelection >= 1 && ($timeSelection == 3 || $slotLimit >= 0) && $scope == 'product' && $this->isDeliverable($productId))
		{
			JHtml::_('stylesheet', 'plg_system_sellaciousdelivery/form.delivery.css', array('version' => S_VERSION_CORE, 'relative' => true));

			JFormHelper::addFieldPath(JPATH_PLUGINS . '/system/sellaciousdelivery/forms/fields');
			$this->form->loadFile(JPATH_PLUGINS . '/system/sellaciousdelivery/forms/delivery.xml');

			if ($timeSelection != 2)
			{
				$this->form->removeField('delivery_slot', 'delivery');
			}

			$disabledDates = array();
			$todayUntil    = '';

			// Dates to select
			$today      = $now->format('Y-m-d', true);
			$todaySlots = StoreTimingsHelper::getAvailableSlots($productId, $sellerUid, $today, $now->format('H:i:s', true));

			if ($timeSelection == 2 && empty($todaySlots))
			{
				$disabledDates[] = $today;
			}

			$dateNow = JFactory::getDate('now', $sellerTimeZone);

			if ($prepTime && ($timeSelection == 2 || $timeSelection == 3))
			{
				$dateNow->modify($prepTime->m . ' ' . $prepTime->u);
			}

			$selectedDate = $dateNow->format($timeSelection == 3 ? 'Y-m-d H:i:s' : 'Y-m-d 00:00:00', true);
			$deliveryHour = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');

			if (!empty($deliveryHour))
			{
				$to_time            = JFactory::getDate($today . ' ' . $deliveryHour->to_time, $sellerTimeZone);
				$today_availability = $sellerParams->get('today_availability', '');
				$today_availability = $today_availability ? JFactory::getDate($today . ' ' . $today_availability, $sellerTimeZone) : null;
				$nxtAvailableDate   = StoreTimingsHelper::getNextAvailableDate($sellerUid);

				if ($today_availability && $today_availability <= $to_time)
				{
					if ($now <= $today_availability)
					{
						$until_time   = $today_availability->diff($now);
						$order_within = $until_time->i . ' mins';

						if ($hr = $until_time->h)
						{
							$order_within = $hr . ' hrs ' . $order_within;
						}

						$todayUntil = JText::sprintf('PLG_SYSTEM_SELLACIOUSDELIVERY_ORDER_WITHIN', $order_within);

						if ($timeSelection == 3)
						{
							$this->form->setValue('delivery_date', 'delivery', $to_time->format('Y-m-d H:i:s', true));
						}
					}
					elseif ($nxtAvailableDate)
					{
						$selectedDate = $nxtAvailableDate->format('Y-m-d H:i:s', true);
					}
				}
				elseif ($timeSelection == 3)
				{
					$defaultDate = ($to_time > $now) ? $to_time->format('Y-m-d H:i:s', true) :  ($nxtAvailableDate ? $nxtAvailableDate->format('Y-m-d H:i:s', true) : null);

					$this->form->setValue('delivery_date', 'delivery', $defaultDate);
				}
			}

			$unavailableDates = StoreTimingsHelper::getUnavailableDates($productId, $sellerUid);
			$disabledDates    = array_unique(array_merge($disabledDates, $unavailableDates));

			// Check for enabled delivery week days
			$deliveryTimings  = $this->helper->seller->getTimings($sellerUid, 'delivery_hours', 1);
			$deliveryWeekDays = ArrayHelper::getColumn($deliveryTimings, 'week_day');

			foreach ($deliveryWeekDays as &$deliveryWeekDay)
			{
				if ($deliveryWeekDay == 6)
				{
					$deliveryWeekDay = 0;
				}
				else
				{
					$deliveryWeekDay += 1;
				}
			}

			$disabledDays = array_values(array_diff(array(0, 1, 2, 3, 4, 5, 6), $deliveryWeekDays));

			$this->form->setFieldAttribute('delivery_date', 'seller_uid', $sellerUid, 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'time_selection', $timeSelection, 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'selected_date', $selectedDate, 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'disabled_days', implode(',', $disabledDays), 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'disabled_dates', json_encode($disabledDates), 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'product_uid', $itemUid, 'delivery');
			$this->form->setFieldAttribute('delivery_date', 'slot_picker_id', 'jform_delivery_delivery_slot_' . $itemUid, 'delivery');
			$this->form->setFieldAttribute('delivery_slot', 'name', 'delivery_slot_' . $itemUid, 'delivery');

			if ($todayUntil)
			{
				$this->form->setFieldAttribute('today_until_note', 'label', $todayUntil, 'delivery');
			}
		}
	}

	/**
	 * Method to prepare checkout questions form data
	 *
	 * @param   Cart    $cart     The cart object
	 * @param   string  $itemUid  Product code
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	public function getData($cart, $itemUid = null)
	{
		if ($itemUid)
		{
			$params = $cart->getItemParam($itemUid, 'delivery_params');
		}
		else
		{
			$params = $cart->getParam('delivery_params');
		}

		return (object) $params;
	}

	/**
	 * Method to prepare checkoutform data for rendering
	 *
	 * @param   array  $data  The data
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public function buildData($data)
	{
		$itemsList = null;

		if ($data)
		{
			$delivery = ArrayHelper::getValue($data, 'delivery', array());

			if (is_array($delivery))
			{
				$formData = new FormData($delivery);
				$formData->buildItems();
				$items = $formData->getItems();

				foreach ($items as $item)
				{
					$name = $item->getName();

					if ($name == 'delivery_date')
					{
						$item->setLabel(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_FORM_DATA_DELIVERY_DATE'));

						$deliveryDate = $item->getValue();
						$deliveryDate = str_replace(array('AM', 'PM'), '', $deliveryDate);
						$deliveryDate = JFactory::getDate($deliveryDate);
						$value        = $deliveryDate->format('F d, Y');

						if ($deliveryDate->format('H:i:s') != '00:00:00')
						{
							$value .= ' ' . $deliveryDate->format('g:i A');
						}

						$item->setValue($value);
					}
					elseif (strpos($name, 'delivery_slot') !== false)
					{
						$item->setLabel(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_FORM_DATA_DELIVERY_SLOT'));

						$slots = explode('-', $item->getValue());
						$slots = array_map('trim', $slots);

						if (isset($slots[1]))
						{
							$slotStart = JFactory::getDate($slots[0]);
							$slotEnd   = JFactory::getDate($slots[1]);
							$slots[0]  = $slotStart->format('g:i A');
							$slots[1]  = $slotEnd->format('g:i A');

							$item->setValue(implode(' - ', $slots));
						}
					}
				}

				$itemsList = $formData->getItemsList();
			}
		}

		return $itemsList;
	}

	/**
	 * Method to render checkout questions form data
	 *
	 * @param   Cart    $cart       The cart object
	 * @param   string  $itemUid    Product code
	 * @param   bool    $buildData  Build form data
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function render(Cart $cart, $itemUid, $buildData)
	{
		$html = parent::render($cart, $itemUid, $buildData);
		$data = (array) $this->getData($cart, $itemUid);

		if ($buildData)
		{
			$data = $this->buildData($data);
		}

		if ($data)
		{
			$values = ArrayHelper::getColumn($data, 'value');
			$html   .= JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_ON') . implode(' ', $values);
		}

		return $html;
	}

	/**
	 * Method to validate checkout questions form
	 *
	 * @param   array  $data  Checkout form data
	 * @param   string  $uid  Product code (optional)
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function validateForm($data, $uid = null)
	{
		if (!parent::validateForm($data))
		{
			return false;
		}

		if ($data && array_key_exists('delivery', $data) && $uid)
		{
			$parsed         = $this->helper->product->parseCode($uid, $productId, $variantId, $sellerUid);
			$sellerTimeZone = StoreTimingsHelper::getUserTimezone($sellerUid);
			$seller         = $this->helper->seller->getItem(array('user_id' => $sellerUid));
			$sellerParams   = new Registry($seller->params);
			$slotLimit      = $sellerParams->get('slot_limit', 0);
			$timeSelection  = $this->helper->config->get('delivery_time_selection', 0);

			$now  = JFactory::getDate('now', $sellerTimeZone);
			$date = ArrayHelper::getValue($data['delivery'], 'delivery_date');
			$date = str_replace(array('AM', 'PM'), '', $date);
			$dt   = JFactory::getDate($date, $sellerTimeZone);

			$weekDay = $this->helper->core->getWeekDay($dt);
			$timing  = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');

			if (!$timing)
			{
				$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOT_DATE_UNAVAILABLE'));
				return false;
			}

			if ($timeSelection == 2)
			{
				$time = '';
				$slot = ArrayHelper::getValue($data['delivery'], 'delivery_slot_' . $uid);

				if ($slot)
				{
					$parts = explode('-', $slot);

					if (isset($parts[1]))
					{
						$time = trim($parts[1]);

						$fromTime  = $date . ' ' . $parts[0];
						$toTime    = $date . ' ' . $parts[1];
						$slotCount = StoreTimingsHelper::getRemainingSlots($sellerUid, $productId, $fromTime, $toTime);

						if ($slotLimit > 0 && $slotCount <= 0)
						{
							$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOTS_FULL'));
							return false;
						}
					}
				}

				if ($time)
				{
					$dt = JFactory::getDate($date . ' ' . $time, $sellerTimeZone);
				}
				else
				{
					$dt = $dt->setTime(0, 0, 0);
				}
			}
			elseif ($timeSelection != 3)
			{
				$fullDayFromTime = $date . ' ' . $timing->from_time;
				$fullDayToTime   = $date . ' ' . $timing->to_time;
				$fullDaySlots    = StoreTimingsHelper::getRemainingSlots($sellerUid, $productId, $fullDayFromTime, $fullDayToTime, 1);

				if ($slotLimit > 0 && $fullDaySlots <= 0)
				{
					$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOTS_FULL'));
					return false;
				}

				$now->setTime(0, 0, 0);
				$dt = $dt->setTime(0, 0, 0);
			}

			if ($dt < $now)
			{
				$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_DATE_PASSED'));
				return false;
			}

			if ($parsed)
			{
				$now      = JFactory::getDate('now', $sellerTimeZone);
				$date     = JFactory::getDate($date);
				$day      = $date->format('Y-m-d');
				$weekDay  = $this->helper->core->getWeekDay($date);
				$timing   = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');
				$fromTime = JFactory::getDate($day . ' ' . $timing->from_time);
				$toTime   = JFactory::getDate($day . ' ' . $timing->to_time);

				if ($timeSelection == 3 && ($date < $fromTime || $date > $toTime))
				{
					$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOT_DATE_UNAVAILABLE'));
					return false;
				}

				$today_availability = $sellerParams->get('today_availability', '');
				$today_availability = $today_availability ? JFactory::getDate($now->format('Y-m-d', true) . ' ' . $today_availability) : null;

				if ($today_availability && $today_availability <= $toTime && $day == $now->format('Y-m-d') && $now > $today_availability)
				{
					$this->setError(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOT_TODAY_UNAVAILABLE'));
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Method to save form data in cart
	 *
	 * @param   array     $data  The valid form data
	 * @param   Cart      $cart  The cart object
	 * @param   Internal  $item  The cart item object
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function saveFormInCart($data, $cart, $item = null)
	{
		if ($item)
		{
			$cart->setItemParam($item->getUid(), 'delivery_params', $data);
		}
		else
		{
			$cart->setParam('delivery_params', $data);
		}
	}

	/**
	 * Method to save checkout questions in order
	 *
	 * @param   Cart  $cart     The cart object
	 * @param   int   $orderId  The order id
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function saveFormInOrder($cart, $orderId)
	{
		JTable::addIncludePath(JPATH_SELLACIOUS . '/components/com_sellaciousdelivery/tables');

		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);

		foreach ($cart->getItems() as $item)
		{
			$data     = (array)$this->getData($cart, $item->getUid());
			$delivery = ArrayHelper::getValue($data, 'delivery');

			if ($delivery && $this->helper->product->parseCode($item->getUid(), $productId, $variantId, $sellerUid))
			{
				$delivery  = (array)$delivery;
				$orderItem = $this->helper->order->getOrderItem($orderId, $item->getUid());

				$deliveryDate = ArrayHelper::getValue($delivery, 'delivery_date');
				$deliveryDate = str_replace(array('AM', 'PM'), '', $deliveryDate);
				$weekDay      = JFactory::getDate($deliveryDate)->format('w');
				$weekDay      = $weekDay == 0 ? 6 : ($weekDay - 1);
				$deliveryHour = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');
				$fromTime     = $deliveryHour->from_time;
				$toTime       = $deliveryHour->to_time;

				if ($timeSelection == 3)
				{
					$fromDate = JFactory::getDate($deliveryDate)->format('Y-m-d H:i:s');
					$toDate   = $fromDate;
					$fullDay  = 0;
				}
				else
				{
					$fromDate = $deliveryDate;
					$toDate   = $deliveryDate;
					$fullDay  = 0;

					if (isset($delivery['delivery_slot_' . $item->getUid()]))
					{
						$deliverySlot = $delivery['delivery_slot_' . $item->getUid()];
						$deliverySlot = array_filter(explode(' - ', $deliverySlot));

						if (!empty($deliverySlot))
						{
							$fromTSlot = JFactory::getDate($deliverySlot[0])->format('H:i:s');
							$toTSlot   = JFactory::getDate($deliverySlot[1])->format('H:i:s');
							$fromTime  = $fromTSlot != '00:00:00' ? $fromTSlot : $fromTime;
							$toTime    = $toTSlot != '00:00:00' ? $toTSlot : $toTime;
						}
					}
					else
					{
						$fullDay = 1;
					}

					$fromDate .= ' ' . $fromTime;
					$toDate   .= ' ' . $toTime;
				}

				$data = array(
					'order_item_id'  => $orderItem->id,
					'slot_from_time' => $fromDate,
					'slot_to_time'   => $toDate,
					'full_day'       => $fullDay,
				);

				StoreTimingsHelper::saveOrderDeliverySlot($data);
			}
		}
	}

	/**
	 * Method to check if product is deliverable
	 *
	 * @param   int  $productId  Product id
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	protected function isDeliverable($productId)
	{
		$product = $this->helper->product->getItem($productId);

		if ($product->type == 'electronic')
		{
			return false;
		}

		if ($product->type == 'package')
		{
			$products = $this->helper->package->getProducts($productId);

			foreach ($products as $product)
			{
				if (isset($product->type) && $product->type == 'physical')
				{
					return true;
				}
			}
		}

		return true;
	}

	/**
	 * Get product-seller attributes
	 *
	 * @param   int  $productId  The product id
	 * @param   int  $sellerUid  The seller user id
	 *
	 * @return  Registry
	 *
	 * @since   2.0.0
	 */
	protected function getPsxAttributes($productId, $sellerUid)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select('a.pricing_type, a.stock, a.quantity_min, a.disable_stock')
			->from($db->qn('#__sellacious_product_sellers', 'a'))
			->join('inner', $db->qn('#__sellacious_products', 'b') . ' ON b.id = a.product_id')
			->where('product_id = ' . (int) $productId)
			->where('seller_uid = ' . (int) $sellerUid)
			->where('b.state = 1');

		$result = $db->setQuery($query)->loadObject();
		$result = new Registry($result);

		return $result;
	}
}
