<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Import\Processor\Products;

defined('_JEXEC') or die;

use Joomla\String\StringHelper;
use Sellacious\Import\AbstractImporter;
use Sellacious\Import\Processor\AbstractProcessor;
use Sellacious\Media\MediaHelper;

class AttachmentProcessor extends AbstractProcessor
{
	protected $helper;

	/**
	 * Constructor
	 *
	 * @param   AbstractImporter  $importer  The parent importer instance object
	 *
	 * @since   2.0.0
	 */
	public function __construct(AbstractImporter $importer)
	{
		parent::__construct($importer);

		try
		{
			$this->helper = \SellaciousHelper::getInstance();
		}
		catch (\Exception $e)
		{
		}
	}

	/**
	 * The columns that will be the part of import CSV
	 *
	 * @return  string[]
	 *
	 * @see     getcolumns()
	 *
	 * @since   2.0.0
	 */
	protected function getCsvColumns()
	{
		return array(
			'attachment_url',
			'attachment_folder',
			'attachment_filename',
		);
	}

	/**
	 * The columns that will NOT be the part of import CSV,
	 * but they are needed to be evaluated first by any other processors.
	 * Without these keys evaluated this processor cannot process.
	 *
	 * @return  string[]
	 *
	 * @see     getDependencies()
	 *
	 * @since   2.0.0
	 */
	protected function getRequiredColumns()
	{
		return array(
			'x__product_id',
		);
	}

	/**
	 * The columns that will NOT be the part of import CSV,
	 * but they will be evaluated by this processors and are available to be used by any other processor.
	 *
	 * @return  string[]
	 *
	 * @see     getDependables()
	 *
	 * @since   2.0.0
	 */
	protected function getGeneratedColumns()
	{
		return array();
	}

	/**
	 * Method to perform the actual import tasks for individual record.
	 * Any write actions can be performed at this stage relevant to the passed record.
	 * If this is called then all dependency must've been already fulfilled by some other processors.
	 *
	 * @param   \stdClass  $obj  The record obtained from CSV, was pre-processed in <var>preProcessRecord()</var>
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function processRecord($obj)
	{
		if (!$obj->x__product_id)
		{
			return;
		}

		if ($obj->attachment_filename)
		{
			$this->saveFile($obj);
		}

		if ($obj->attachment_url)
		{
			$this->saveUrl($obj);
		}
	}

	/**
	 * Load the file as specified in the record
	 *
	 * @param   \stdClass  $obj  The entire row from import
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	protected function saveFile($obj)
	{
		$url = rtrim($obj->attachment_folder . '/' . $obj->attachment_filename, '/ ');
		$url = $this->parseCode($obj, $url);

		jimport('joomla.filesystem.path');
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');

		if (!is_file(\JPath::clean(JPATH_SITE . '/' . $url)))
		{
			return true;
		}

		// Use this file for product: id, table_name, record_id, context, original_name, type, path, size, state
		$filename  = basename($url);
		$directory = $this->helper->media->getBaseDir(sprintf('%s/attachments/%d', 'products', $obj->x__product_id));
		$directory = ltrim($directory, '/');

		// If file already exists, then create a copy
		$filename = $this->generateFileName($filename, JPATH_SITE . '/' . $directory . '/');

		if (\JFolder::create(JPATH_SITE . '/' . $directory) && \JFile::copy($url, $directory . '/' . $filename, JPATH_SITE))
		{
			// Todo: check for uniqueness
			$rec = new \stdClass;

			$rec->table_name    = 'products';
			$rec->context       = 'attachments';
			$rec->record_id     = $obj->x__product_id;
			$rec->path          = $directory . '/' . $filename;
			$rec->original_name = $filename;
			$rec->type          = MediaHelper::getMimeType(JPATH_SITE . '/' . $directory . '/' . $filename);
			$rec->size          = filesize(JPATH_SITE . '/' . $directory . '/' . $filename);
			$rec->state         = 1;
			$rec->created       = \JFactory::getDate()->toSql();

			$db = $this->importer->getDb();

			return $db->insertObject('#__sellacious_media', $rec, 'id');
		}

		return false;
	}

	/**
	 * Load the file as specified in the record from a remote URL
	 *
	 * @param   \stdClass  $obj  The entire row from import
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	protected function saveUrl($obj)
	{
		$url = $obj->attachment_url;
		$url = $this->parseCode($obj, $url);

		jimport('joomla.filesystem.path');
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');

		// People often forget http(s), we can at most detect a 'www.'
		if (substr($url, 0, 4) == 'www.')
		{
			$url = 'http://' . $url;
		}

		if (substr($url, 0, 7) != 'http://' && substr($url, 0, 8) != 'https://')
		{
			// This may be a local file, no B/C
			return false;
		}

		// Use this file for product: id, table_name, record_id, context, original_name, type, path, size, state
		$filename  = basename($url);
		$directory = $this->helper->media->getBaseDir(sprintf('%s/attachments/%d', 'products', $obj->x__product_id));
		$directory = ltrim($directory, '/');

		// We'll download this file later in a separate batch. Put a placeholder for now
		$placeholder = \JHtml::_('image', 'com_importer/coming-soon-placeholder.png', '', null, true, 1);
		$placeholder = $placeholder ? substr($placeholder, strlen(rtrim(\JUri::root(true), '\\/'))) : null;

		if (is_file(JPATH_SITE . $placeholder) &&
			\JFolder::create(JPATH_SITE . '/' . $directory) &&
			\JFile::copy($placeholder, $directory . '/' . $filename, JPATH_SITE))
		{
			$params = array(
				'remote_download' => true,
				'download_url'    => $url,
			);

			// Todo: check for uniqueness
			$rec = new \stdClass;

			$rec->table_name    = 'products';
			$rec->context       = 'attachments';
			$rec->record_id     = $obj->x__product_id;
			$rec->path          = $directory . '/' . $filename;
			$rec->original_name = $filename;
			$rec->type          = 'application/octet-stream';
			$rec->size          = 0;
			$rec->state         = -1;
			$rec->params        = json_encode($params);
			$rec->created       = \JFactory::getDate()->toSql();

			$db = $this->importer->getDb();

			return $db->insertObject('#__sellacious_media', $rec, 'id');
		}

		return false;
	}

	/**
	 * Method to generate a new filename if it already exists
	 *
	 * @param   string  $filename  Name of the file
	 * @param   string  $dirPath   Directory Path where the file is located
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected function generateFileName($filename, $dirPath = '')
	{
		while (\JFile::exists($dirPath . $filename))
		{
			$fname = \JFile::stripExt($filename);
			$ext   = \JFile::getExt($filename);
			$fname = StringHelper::increment($fname, 'dash');

			$filename = $fname . '.' . $ext;
		}

		return $filename;
	}

	/**
	 * Process the embedded short codes in the file path/url
	 *
	 * @param   \stdClass  $obj  The import record
	 * @param   string     $url  The path/url to process
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected function parseCode($obj, $url)
	{
		static $pattern;

		if (strpos($url, '%') !== false)
		{
			// Optimize! Build pattern only once (We use the headers as short-code)
			if (!$pattern)
			{
				$headers    = array();
				$headersAll = $this->importer->getHeaders();

				foreach ($headersAll as $header)
				{
					$headers[] = '%' . preg_quote($header, '/') . '%';
				}

				$pattern = '/(' . implode('|', $headers) . ')/i';
			}

			$matches = array();

			preg_match_all($pattern, $url, $matches, PREG_SET_ORDER);

			foreach ($matches as $match)
			{
				$key = strtolower($match[1]);

				$url = str_replace($match[0], isset($obj->$key) ? $obj->$key : '', $url);
			}
		}

		// If there is no file, do not proceed here
		return strlen($url) >= 5 ? $url : null;
	}
}
