<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Aditya Chakraborty <info@bhartiy.com> - http://www.bhartiy.com
 */

namespace Sellacious\StoreTimings;

// no direct access
use DateTimeZone;
use Exception;
use JFactory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Language\Wrapper\JTextWrapper;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use JTable;
use SellaciousHelper;
use stdClass;

defined('_JEXEC') or die;

/**
 * Store Timings Helper
 *
 * @package Sellacious\StoreTimings
 *
 * @since   2.0.0
 */
class StoreTimingsHelper
{
	/**
	 * List of days in a week
	 *
	 * @var    array
	 * @since  2.0.0
	 */
	protected static $days = array('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');

	/**
	 * Method to get remaining slots
	 *
	 * @param   int     $sellerUid  The seller user id
	 * @param   int     $productId  The product id
	 * @param   string  $fromTime   From time
	 * @param   string  $toTime     To Time
	 * @param   int     $fullDay    Is it a full day slot?
	 *
	 * @return  int
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRemainingSlots($sellerUid, $productId, $fromTime = null, $toTime = null, $fullDay = 0)
	{
		JTable::addIncludePath(JPATH_SITE . '/components/com_sellacious/tables');

		$helper    = SellaciousHelper::getInstance();
		$seller    = $helper->seller->getItem(array('user_id' => $sellerUid));
		$params    = new Registry($seller->params);
		$slotLimit = $params->get('slot_limit', 0);

		$now  = JFactory::getDate()->format('Y-m-d 00:00:00');
		$data = array(
			'seller_uid'     => $sellerUid,
			'full_day'       => $fullDay,
			'slot_from_time' => $fromTime ?: $now,
			'slot_to_time'   => $toTime ?: $now
		);

		if ($productId)
		{
			$data['product_id'] = $productId;
		}

		$slLimit = JTable::getInstance('ProductSellerSlotLimit', 'SellaciousTable');
		$slLimit->load($data);

		if ($slLimit->get('id'))
		{
			$slotCount = $slLimit->get('slot_count', 0);
		}
		else
		{
			$slotCount = $slotLimit;
		}

		return $slotCount;
	}

	/**
	 * Method to get available slots
	 *
	 * @param   int     $productId   The product id
	 * @param   int     $sellerUid   The seller user id
	 * @param   string  $date        The date for which available slots are required
	 * @param   string  $fromTime    The time from which slots are required
	 * @param   bool    $all         Whether to get all slots or only slots that are available at current time
	 * @param   string  $type        Type of slots
	 * @param   int     $shipRuleId  Shipping rule id (optional)
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getAvailableSlots($productId, $sellerUid, $date = '', $fromTime = '', $all = false, $type = 'delivery', $shipRuleId = null)
	{
		$shipRuleId     = $type != 'delivery' ? null : $shipRuleId;
		$sellerTimeZone = StoreTimingsHelper::getUserTimezone($sellerUid);
		$now            = JFactory::getDate('now', $sellerTimeZone);
		$helper         = SellaciousHelper::getInstance();
		$timeSelection  = $helper->config->get('delivery_time_selection', 0);

		$slotDate = $date != '' ? JFactory::getDate($date) : $now;
		$weekDay  = $helper->core->getWeekDay($slotDate);

		$prefix = $type == 'delivery' ? '' : $type . '_';

		$seller            = $helper->seller->getItem(array('user_id' => $sellerUid));
		$params            = new Registry($seller->params);
		$slotLimit         = $params->get('slot_limit', 0);
		$slotWindow        = $params->get($prefix . 'slot_window', null);
		$prepTime          = $params->get('delivery_prep_time', null);
		$todayAvailability = $params->get($prefix . 'today_availability', '');
		$todayAvailability = $todayAvailability ? JFactory::getDate($now->format('Y-m-d', true) . ' ' . $todayAvailability) : null;

		if ($timeSelection != 2)
		{
			$prepTime = null;
		}

		// Get Delivery Timing of selected day
		$timings     = $helper->seller->getTiming($sellerUid, $weekDay, $type . '_hours');
		$slotOptions = array();

		// Check if full day slots are remaining or not
		$fullDayFromTime = isset($timings->from_time) ? $slotDate->format('Y-m-d') . ' ' . $timings->from_time : null;
		$fullDayToTime   = isset($timings->from_time) ? $slotDate->format('Y-m-d') . ' ' . $timings->to_time : null;
		$fullDaySlots    = $type == 'delivery' ? static::getRemainingSlots($sellerUid, $productId, $fullDayFromTime, $fullDayToTime, 1) : 1;

		if ($timeSelection == 1 && !empty($timings))
		{
			$slotWindow    = new stdClass();
			$daySlot       = JFactory::getDate($timings->to_time)->diff(JFactory::getDate($timings->from_time));
			$slotWindow->m = $daySlot->h;
			$slotWindow->u = 'hours';
		}

		if (($timeSelection != 1 || ($slotLimit == 0 || $fullDaySlots > 0)) && !empty($timings))
		{
			if (isset($slotWindow->m) && $slotWindow->m > 0)
			{
				$fromTime = $timings->from_time ? $timings->from_time : $fromTime;
				$toTime   = $timings->to_time;
				$fromTime = JFactory::getDate($slotDate->format('Y-m-d', true) . ' ' . $fromTime);
				$toTime   = JFactory::getDate($slotDate->format('Y-m-d', true) . ' ' . $toTime);

				// If the slot date is current date
				if (!$all && $slotDate->format('Y-m-d', true) == $now->format('Y-m-d', true))
				{
					$nextSlotAvailable = JFactory::getDate($now->format('Y-m-d H:i:s', true));

					// If current date slot has exceeded Same day availability
					if ($todayAvailability && $nextSlotAvailable > $todayAvailability)
					{
						return $slotOptions;
					}
				}

				if ($slotOptions = static::getSlotsForTime($timings->id, $sellerUid, $fromTime, $toTime, !$all, $shipRuleId, false, $prepTime))
				{
					foreach ($slotOptions as $slotRange => $slotOption)
					{
						$slot        = explode('-', $slotRange);
						$slotFromVal = trim($slot[0]);
						$slotToVal   = trim($slot[1]);

						$slotCount = static::getRemainingSlots($sellerUid, $productId, $slotDate->format('Y-m-d') . ' ' . $slotFromVal . ':00', $slotDate->format('Y-m-d') . ' ' . $slotToVal . ':00');

						if ($slotLimit > 0 && $slotCount <= 0)
						{
							unset($slotOptions[$slotRange]);
						}
					}
				}
			}
			elseif ($timeSelection == 3)
			{
				$fromTime                              = JFactory::getDate($slotDate->format('Y-m-d', true) . ' ' . $timings->from_time);
				$slotOptions[$fromTime->format('H:i')] = $fromTime->format('H:i');
			}
		}

		return $slotOptions;
	}

	/**
	 * Method to calculate and store slots
	 *
	 * @param   array   $timings    Array of timings
	 * @param   int     $sellerUid  User id of the seller for whom slots are being stored
	 * @param   string  $type       Type of slots
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function saveSlots($timings, $sellerUid, $type = 'delivery')
	{
		$db  = JFactory::getDbo();
		$day = JFactory::getDate()->format('Y-m-d');

		$slotsToInsert = array();

		foreach ($timings as $timing)
		{
			$weekDay = $timing['week_day'];

			$table = \SellaciousTable::getInstance('SellerTimings');
			$table->load(array('seller_uid' => $sellerUid, 'type' => $type . '_hours', 'week_day' => $weekDay));

			$timingId = $table->get('id');

			if (!$timingId)
			{
				continue;
			}

			$params = new Registry($table->get('params'));
			$oldKey = $params->get('timing_setting_key', '');
			$newKey = static::generateSettingKey($table->getProperties(1), $sellerUid, $type);

			$params->set('timing_setting_key', $newKey);

			$table->set('params', $params->toString());
			$table->check();
			$table->store();

			// If setting hasn't changed, no need to delete and insert slots
			if ($newKey == $oldKey)
			{
				continue;
			}

			$fromTime = JFactory::getDate($day . ' ' . $table->get('from_time'));
			$toTime   = JFactory::getDate($day . ' ' . $table->get('to_time'));

			// Remove all slots first
			static::removeSlots($timingId);

			// Calculate slots
			$slots = static::getCalculatedSlots($sellerUid, $fromTime, $toTime, false, $type);

			if ($slots)
			{
				$slotsToInsert[$timingId] = $slots;
			}
		}

		if ($slotsToInsert)
		{
			$query = $db->getQuery(true);
			$query->insert($db->qn('#__sellacious_seller_time_slots'))
				->columns($db->qn(array('timing_id', 'from_time', 'to_time')));

			foreach ($slotsToInsert as $timingId => $slots)
			{
				foreach ($slots as $slotRange => $slotOption)
				{
					$slot        = explode('-', $slotRange);
					$slotFromVal = trim($slot[0]);
					$slotToVal   = trim($slot[1]);

					$values = array($timingId, $slotFromVal, $slotToVal);
					$query->values(implode(', ', $db->q($values)));
				}
			}

			$db->setQuery($query)->execute();
		}
	}

	/**
	 * Method to generate a setting key for timing
	 *
	 * @param   array  $timing     Timing data
	 * @param   int    $sellerUid  Seller user id
	 * @param   string $type       Type of timing slots
	 *
	 * @return  string
	 *
	 * @throws  Exception
	 * 
	 * @since   2.0.0
	 */
	protected static function generateSettingKey($timing, $sellerUid, $type = 'delivery')
	{
		$array  = array();
		$helper = SellaciousHelper::getInstance();
		$seller = $helper->seller->getItem(array('user_id' => $sellerUid));
		$params = new Registry($seller->params);
		$prefix = $type == 'delivery' ? '' : $type . '_';

		$array['slot_window'] = $params->get($prefix . 'slot_window', null);
		$array['slot_pad']    = $params->get($prefix . 'slot_pad', null);
		$array['from_time']   = $timing['from_time'];
		$array['to_time']     = $timing['to_time'];

		$key = md5(serialize($array));

		return $key;
	}

	/**
	 * Method to get slots for timing
	 *
	 * @param   int  $timingId  Id of the timing
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public static function getSlots($timingId)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select('*')
			->from('#__sellacious_seller_time_slots')
			->where('timing_id = ' . (int) $timingId);

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	/**
	 * Method to get time slots for a seller timing
	 *
	 * @param   int       $timingId    Timing id
	 * @param   int       $sellerUid   Seller user id
	 * @param   Date      $fromTime    From time
	 * @param   Date      $toTime      To Time
	 * @param   bool      $current     Whether to get slots for current time
	 * @param   int       $shipRuleId  Shipping rule id (optional)
	 * @param   bool      $amountOnly  Show amount only (applicable only if shipping rule id is given)
	 * @param   stdClass  $prepTime    Preparation Time
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getSlotsForTime($timingId, $sellerUid, $fromTime, $toTime, $current = false, $shipRuleId = null, $amountOnly = false, $prepTime = null)
	{
		$slotOptions = array();

		$helper     = SellaciousHelper::getInstance();
		$c_currency = $helper->currency->current('code_3');
		$g_currency = $helper->currency->getGlobal('code_3');

		$tz    = StoreTimingsHelper::getUserTimezone($sellerUid);
		$now   = JFactory::getDate('now', $tz);
		$db    = JFactory::getDbo();
		$day   = $fromTime->format('Y-m-d');
		$query = $db->getQuery(true);

		if ($prepTime)
		{
			$now->modify($prepTime->m . ' ' . $prepTime->u);
		}

		if ($now > $toTime)
		{
			$toTime = $fromTime;
		}

		$query->select('a.from_time, a.to_time')
			->from($db->qn('#__sellacious_seller_time_slots', 'a'))
			->where('timing_id = ' . (int) $timingId);

		if ($shipRuleId > 0)
		{
			$query->select('b.amount');
			$query->join('left', $db->qn('#__sellacious_shiprule_time_slot_rates', 'b') .' ON b.slot_id = a.id');
			$query->where('b.shippingrule_id = ' . (int) $shipRuleId);
			$query->group('a.id');
		}

		$query->where('a.from_time >= ' . $db->q($fromTime->format('H:i')));
		$query->where('a.to_time <= ' . $db->q($toTime->format('H:i')));

		if ($current && $day == $now->format('Y-m-d'))
		{
			$query->where('a.from_time > ' . $db->q($now->format('H:i',true)));
		}

		$db->setQuery($query);

		$slots = $db->loadObjectList();

		if (!$slots)
		{
			return $slotOptions;
		}

		foreach ($slots as $slot)
		{
			$slotFrom = JFactory::getDate($day . ' ' . $slot->from_time);
			$slotTo   = JFactory::getDate($day . ' ' . $slot->to_time);

			$slotFromVal  = $slotFrom->format('H:i');
			$slotToVal    = $slotTo->format('H:i');
			$slotFromText = $slotFrom->format('g:i A');
			$slotToText   = $slotTo->format('g:i A');

			$text = $slotFromText . ' - ' . $slotToText;

			if ($shipRuleId && isset($slot->amount))
			{
				$slotTo  = JFactory::getDate($day . ' ' . $slot->to_time, $tz);
				$amount  = $slot->amount;
				$amount2 = static::getAdditionalRate($shipRuleId, $sellerUid, $slotTo);
				$amount  = $amount + $amount2;

				if ($amountOnly)
				{
					$text = $amount;
				}
				else
				{
					$text .= ' (' . $helper->currency->display($amount, $g_currency, $c_currency) . ')';
				}
			}

			$slotOptions[$slotFromVal . ' - ' . $slotToVal] = $text;
		}

		return $slotOptions;
	}

	/**
	 * Method to remove slots for a timing
	 *
	 * @param  int  $timingId  Timing id
	 *
	 * @since  2.0.0
	 */
	public static function removeSlots($timingId)
	{
		if (!$timingId)
		{
			return;
		}

		// First remove all rates associated with the timing
		static::removeTimingSlotRates($timingId);

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->delete('#__sellacious_seller_time_slots')
			->where('timing_id = ' . (int) $timingId);

		$db->setQuery($query)->execute();
	}

	/**
	 * Method to delete slot rates for a timing
	 *
	 * @param  int  $timingId  Timing id
	 *
	 * @since  2.0.0
	 */
	public static function removeTimingSlotRates($timingId)
	{
		$db = JFactory::getDbo();

		$query = 'DELETE a FROM ' . $db->qn('#__sellacious_shiprule_time_slot_rates', 'a');
		$query .= ' LEFT JOIN ' . $db->qn('#__sellacious_seller_time_slots', 'b') . ' ON b.id = a.slot_id';
		$query .= ' WHERE b.timing_id = ' . (int) $timingId;

		$db->setQuery($query)->execute();
	}

	/**
	 * Method to delete all slot rates for shipping rule
	 *
	 * @param  int  $shipRuleId  Shipping rule id
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	public static function removeShippingRuleRates($shipRuleId)
	{
		if (!$shipRuleId)
		{
			return false;
		}

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->delete('#__sellacious_shiprule_time_slot_rates')
			->where('shippingrule_id = ' . (int) $shipRuleId);

		$db->setQuery($query)->execute();

		return true;
	}

	/**
	 * Method to get calculated timing slots set by seller
	 *
	 * @param   int     $sellerUid  Seller user id
	 * @param   Date    $fromTime   From time
	 * @param   Date    $toTime     To Time
	 * @param   bool    $current    Whether to get slots for current time
	 * @param   string  $type       Type of slots
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getCalculatedSlots($sellerUid, $fromTime, $toTime, $current = false, $type = 'delivery')
	{
		$helper = SellaciousHelper::getInstance();
		$seller = $helper->seller->getItem(array('user_id' => $sellerUid));
		$params = new Registry($seller->params);
		$tz     = StoreTimingsHelper::getUserTimezone($sellerUid);
		$now    = JFactory::getDate('now', $tz);
		$prefix = $type == 'delivery' ? '' : $type . '_';

		$slotOptions = array();
		$slotWindow  = $params->get($prefix . 'slot_window', null);
		$slotPad     = $params->get($prefix . 'slot_pad', null);

		if (!isset($slotWindow->m) || !$slotWindow->m)
		{
			return $slotOptions;
		}

		$slotWin     = $slotWindow->m;
		$slotWinUnit = $slotWindow->u;

		for ($i = $fromTime; $i < $toTime; $i->modify('+' . $slotWin . ' ' . $slotWinUnit))
		{
			$slot         = JFactory::getDate($i->format('Y-m-d H:i'));
			$slotFrom     = JFactory::getDate($i->format('Y-m-d H:i'), $tz);
			$slotFromVal  = $slot->format('H:i');
			$slotFromText = $slot->format('g:i A');

			$slot->modify('+' . $slotWin . ' ' . $slotWinUnit);

			if (strtotime($slot) > strtotime($toTime))
			{
				$slot = $toTime;
			}

			$slotToVal  = $slot->format('H:i');
			$slotToText = $slot->format('g:i A');

			if (!$current || $now < $slotFrom)
			{
				$slotOptions[$slotFromVal . ' - ' . $slotToVal] = $slotFromText . ' - ' . $slotToText;
			}

			if (isset($slotPad->m) && $slotPad->m > 0)
			{
				$i->modify('+' . $slotPad->m . ' ' . $slotPad->u);
			}
		}

		return $slotOptions;
	}

	/**
	 * Method to get slot dates which are unavailable
	 *
	 * @param   int  $productId  The product id
	 * @param   int  $sellerUid  The seller user id
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getUnavailableDates($productId, $sellerUid)
	{
		// Get Full day slot dates that are unavailable
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('DATE_FORMAT(a.slot_from_time, \'%Y-%m-%d\') as slot_date');
		$query->from($db->qn('#__sellacious_product_seller_slot_limits', 'a'));
		$query->where('a.full_day = 1');
		$query->where('a.slot_count = 0');
		$query->where('a.product_id = ' . (int)$productId . ' AND a.seller_uid = ' . (int)$sellerUid);

		$db->setQuery($query);

		$unavailableDates = $db->loadColumn();

		// Get dates which are unavailable due to all slots are used
		$query = $db->getQuery(true);
		$query->select('DISTINCT DATE_FORMAT(a.slot_from_time, \'%Y-%m-%d\') as slot_date');
		$query->from($db->qn('#__sellacious_product_seller_slot_limits', 'a'));
		$query->where('DATE_FORMAT(a.slot_from_time, \'%Y-%m-%d\') <> \'0000-00-00\' AND slot_count = 0');

		$db->setQuery($query);

		$slotDates = $db->loadColumn();

		foreach ($slotDates as $slotDate)
		{
			$availableSlots = static::getAvailableSlots($productId, $sellerUid, $slotDate);

			if (empty($availableSlots))
			{
				$unavailableDates[] = $slotDate;
			}
		}

		return array_unique($unavailableDates);
	}

	/**
	 * Method to save flat rate for all slots of shipping rule
	 *
	 * @param   int     $shipRuleId  Id of shipping rule
	 * @param   int     $sellerUid   User id of seller
	 * @param   float   $amount      Rate amount
	 * @param   string  $type        Type of slots
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function saveSlotsFlatRate($shipRuleId, $sellerUid, $amount = 0.00, $type = 'delivery')
	{
		if (!$shipRuleId || !$sellerUid)
		{
			return false;
		}

		$helper = SellaciousHelper::getInstance();

		// Get Timings
		$timings = $helper->seller->getTimings($sellerUid, $type . '_hours');

		if (!$timings)
		{
			return false;
		}

		foreach ($timings as $timing)
		{
			$slots = static::getSlots($timing['id']);

			foreach ($slots as $slot)
			{
				static::saveSlotRate($shipRuleId, $slot->id, $amount);
			}
		}

		return true;
	}

	/**
	 * Method to save slot rate for a shipping rule
	 *
	 * @param   int    $shipRuleId  Id of shipping rule
	 * @param   int    $slotId      Id of the timing slot
	 * @param   float  $amount      Rate Amount
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	public static function saveSlotRate($shipRuleId, $slotId, $amount = 0.00)
	{
		if (!$shipRuleId || !$slotId)
		{
			return false;
		}

		$table = \SellaciousTable::getInstance('TimeSlotRates');
		$data  = array('shippingrule_id' => $shipRuleId, 'slot_id' => $slotId);
		$table->load($data);

		$data['amount'] = $amount;

		$table->bind($data);
		$table->check();
		$table->store();

		return true;
	}

	/**
	 * Method to get time slot rate
	 *
	 * @param   array  $filters
	 *
	 * @return  object
	 *
	 * @since   2.0.0
	 */
	public static function getTimeSlotRate($filters = array())
	{
		$table = \SellaciousTable::getInstance('TimeSlotRates');
		$table->load($filters);

		return (object) $table->getProperties(1);
	}

	/**
	 * Method to get slot rates for the date and shipping rule
	 *
	 * @param   int     $shipRuleId
	 * @param   int     $sellerUid
	 * @param   string  $slotDate
	 * @param   string  $type
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getDaySlotRates($shipRuleId, $sellerUid, $slotDate, $type = 'delivery')
	{
		$helper = SellaciousHelper::getInstance();

		$daySlotRates = array();
		$slotDate     = JFactory::getDate($slotDate);
		$weekDay      = $helper->core->getWeekDay($slotDate);

		$timing = $helper->seller->getTiming($sellerUid, $weekDay, $type . '_hours');

		if (!$timing->id)
		{
			return $daySlotRates;
		}

		$fromTime = $timing->from_time;
		$toTime   = $timing->to_time;
		$fromTime = JFactory::getDate($slotDate->format('Y-m-d') . ' ' . $fromTime);
		$toTime   = JFactory::getDate($slotDate->format('Y-m-d') . ' ' . $toTime);

		$daySlotRates = static::getSlotsForTime($timing->id, $sellerUid, $fromTime, $toTime, false, $shipRuleId, true);

		return $daySlotRates;
	}

	/**
	 * Method to get next X time delivery rates for a shipping rule
	 *
	 * @param   int  $shipRuleId  Shipping rule id
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public static function getNextTimeRates($shipRuleId)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select('a.*')
			->from($db->qn('#__sellacious_shiprule_next_time_rates', 'a'))
			->where('a.shippingrule_id = ' . (int) $shipRuleId);

		$db->setQuery($query);

		$rates = $db->loadObjectList();

		if ($rates)
		{
			foreach ($rates as $rate)
			{
				$rate->formatted_time = '';
				$rate->next_x_time    = json_decode($rate->next_x_time, true);

				$unit  = $rate->next_x_time['u'];
				$value = $rate->next_x_time['m'];

				if ($unit == 'tomorrow' || $unit == 'today')
				{
					$rate->formatted_time = \JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_NEXT_HOUR_RATES_TIME_' . strtoupper($unit));
				}
				else
				{
					$rate->formatted_time = \JText::sprintf('PLG_SYSTEM_SELLACIOUSDELIVERY_NEXT_HOUR_RATES_TIME_NEXT_X_TIME', $value, $unit);
				}
			}
		}

		return $rates;
	}

	/**
	 * Method to get additional rates for a shipping rule (based on next x time rates)
	 *
	 * @param   int   $shipRuleId  Shippiing rule id
	 * @param   int   $sellerUid   Seller user id
	 * @param   Date  $slotTo      Date object of slot To-Time
	 *
	 * @return  float
	 *
	 * @see    getNextTimeRates
	 *
	 * @since  2.0.0
	 */
	public static function getAdditionalRate($shipRuleId, $sellerUid, $slotTo)
	{
		$sellerTimeZone = StoreTimingsHelper::getUserTimezone($sellerUid);
		$nextXRates     = static::getNextTimeRates($shipRuleId);
		$amount         = 0.0;

		if ($nextXRates)
		{
			foreach ($nextXRates as $nextXRate)
			{
				if (!$nextXRate->add_to_slot_rates)
				{
					continue;
				}

				$time = $nextXRate->next_x_time;
				$u    = $time['u'];
				$m    = $time['m'];

				if ($u == 'today')
				{
					$date = JFactory::getDate('now', $sellerTimeZone);

					if ($slotTo->format('Y-m-d', true) == $date->format('Y-m-d', true))
					{
						$amount += $nextXRate->amount;
					}
				}
				else
				{
					$unit = ($u == 'tomorrow' || $u == 'days') ? 'day' : $u;
					$m    = $u == 'tomorrow' ? 1 : $m;

					$date = JFactory::getDate('now', $sellerTimeZone)->modify('+' . $m . ' ' . $unit);

					if ($unit == 'day')
					{
						$date->setTime(23, 59, 59);
					}

					if (($u == 'tomorrow' && $slotTo->format('Y-m-d', true) == $date->format('Y-m-d', true)) || ($u != 'tomorrow' && $slotTo <= $date))
					{
						$amount += $nextXRate->amount;
					}
				}
			}
		}

		return $amount;
	}

	/**
	 * Method to get process dates for a shipping rule, based on next X rates
	 *
	 * @param   int  $shipRuleId  Shipping rule id
	 *
	 * @return  array
	 *
	 * @see     getNextTimeRates
	 *
	 * @since   2.0.0
	 */
	public static function processDatesForShipRule($shipRuleId)
	{
		$date          = JFactory::getDate();
		$disabledDates = array();
		$nextXRates    = static::getNextTimeRates($shipRuleId);

		if ($nextXRates)
		{
			$days = 0;

			foreach ($nextXRates as $nextXRate)
			{
				if ($nextXRate->add_to_slot_rates)
				{
					continue;
				}

				$time = $nextXRate->next_x_time;
				$u    = $time['u'];
				$m    = $time['m'];

				if ($u == 'hours' && $m > 0)
				{
					$d = round($m / 24);

					if ($d > $days)
					{
						$days = $d;
					}
				}
				elseif ($u == 'days' && $m > 0)
				{
					if ($m > $days)
					{
						$days = $m;
					}
				}
				elseif ($u == 'today')
				{
					$disabledDates[] = $date->format('Y-m-d');
				}
				else if ($u == 'tomorrow')
				{
					$dt              = JFactory::getDate()->modify('+1 day');
					$disabledDates[] = $date->format('Y-m-d');
					$disabledDates[] = $dt->format('Y-m-d');
				}
			}

			if ($days > 0)
			{
				$date->modify($days . ' days');
			}
		}

		return array($date, $disabledDates);
	}

	/**
	 * Method to save next X time rates
	 *
	 * @param   array  $rates       Array of rates to save
	 * @param   int    $shipRuleId  Shipping rule id for which rates are being saved
	 *
	 * @since   2.0.0
	 */
	public static function saveNextTimeRates($rates, $shipRuleId)
	{
		if (!$rates)
		{
			return;
		}

		$helper        = SellaciousHelper::getInstance();
		$timeSelection = $helper->config->get('delivery_time_selection', 0);

		// Validate first
		self::validateNextTimeRates($rates);

		foreach ($rates as $rate)
		{
			$table = \SellaciousTable::getInstance('NextTimeRates');

			if ($id = $rate['id'])
			{
				$table->load($id);
			}

			if ($timeSelection != 2)
			{
				$rate['add_to_slot_rates'] = 0;
			}

			$rate['shippingrule_id'] = $shipRuleId;

			$table->bind($rate);
			$table->check();
			$table->store();
		}
	}

	/**
	 * Method to validate next time rates
	 *
	 * @param   array  $rates  Array of rates to validate
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function validateNextTimeRates(array $rates)
	{
		foreach ($rates as $rate)
		{
			$nextXTime = $rate['next_x_time'];
			$m         = $nextXTime['m'];
			$u         = $nextXTime['u'];

			if ($u == 'days' && $m < 2)
			{
				throw new Exception(\JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_NEXT_X_TIME_DAYS_INVALID'));
			}

			if ($u == 'hours' && $m < 1)
			{
				throw new Exception(\JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_NEXT_X_TIME_HOURS_INVALID'));
			}
		}
	}

	/**
	 * Method to delete next x time rates
	 *
	 * @param  int[]  $ids  Array of ids to delete
	 *
	 * @since  2.0.0
	 */
	public static function deleteNextTimeRates($ids)
	{
		if (!$ids)
		{
			return;
		}

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->delete('#__sellacious_shiprule_next_time_rates')
			->where('id IN (' . implode(',', $ids) . ')');

		$db->setQuery($query)->execute();
	}

	/**
	 * Method to delete all next x time rates for shipping rule
	 * 
	 * @param   int  $shipRuleId  Shipping rule id
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	public static function deleteShipRuleNextRates($shipRuleId)
	{
		if (!$shipRuleId)
		{
			return false;
		}

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->delete('#__sellacious_shiprule_next_time_rates')
			->where('shippingrule_id = ' . (int) $shipRuleId);

		$db->setQuery($query)->execute();

		return true;
	}

	/**
	 * Method to get user timezone
	 *
	 * @param   $userId
	 *
	 * @return  DateTimeZone
	 *
	 * @since   2.0.0
	 */
	public static function getUserTimezone($userId)
	{
		$user = JFactory::getUser($userId);
		$tz   = $user->getParam('timezone', 'UTC');
		$tz   = new DateTimeZone($tz);

		return $tz;
	}

	/**
	 * Method too get disabled days from timings
	 *
	 * @param   array  $timings
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	public static function getDisabledDays(array $timings)
	{
		$weekdays = ArrayHelper::getColumn($timings, 'week_day');

		foreach ($weekdays as &$weekday)
		{
			if ($weekday == 6)
			{
				$weekday = 0;
			}
			else
			{
				$weekday += 1;
			}
		}

		$disabledDays = array_values(array_diff(array(0, 1, 2, 3, 4, 5, 6), $weekdays));

		return $disabledDays;
	}

	/**
	 * Method to next available date which has timing set by the seller
	 *
	 * @param   int     $sellerUid
	 * @param   string  $type
	 * @param   bool    $includeToday
	 *
	 * @return  Date
	 *
	 * @since   2.0.0
	 */
	public static function getNextAvailableDate($sellerUid, $type = 'delivery_hours', $includeToday = false)
	{
		$helper     = SellaciousHelper::getInstance();
		$sellerTz   = static::getUserTimezone($sellerUid);
		$now        = JFactory::getDate('now', $sellerTz);
		$nextDay    = JFactory::getDate('now', $sellerTz)->modify('+1 day');
		$weekDay    = $helper->core->getWeekDay($now);
		$weekDayNxt = $helper->core->getWeekDay($nextDay);

		if ($includeToday)
		{
			$timing = $helper->seller->getTiming($sellerUid, $weekDay, $type);

			if ($timing)
			{
				$nxtDate = JFactory::getDate($now->format('Y-m-d') . ' ' . $timing->from_time, $sellerTz);

				return $nxtDate;
			}
		}

		while ($weekDayNxt != $weekDay)
		{
			$timing = $helper->seller->getTiming($sellerUid, $weekDayNxt, $type);

			if ($timing)
			{
				$nxtDate = JFactory::getDate($nextDay->format('Y-m-d') . ' ' . $timing->from_time, $sellerTz);

				return $nxtDate;
			}

			$nextDay->modify('+1 day');
			$weekDayNxt = $helper->core->getWeekDay($nextDay);
		}

		$timing = $helper->seller->getTiming($sellerUid, $weekDayNxt, $type);

		if ($timing)
		{
			$nxtDate = JFactory::getDate($nextDay->format('Y-m-d') . ' ' . $timing->from_time, $sellerTz);

			return $nxtDate;
		}

		return null;
	}

	/**
	 * Method to save order delivery slot
	 *
	 * @param   array  $data
	 *
	 * @since   2.0.0
	 */
	public static function saveOrderDeliverySlot($data)
	{
		$table = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');

		if (!isset($data['full_day']))
		{
			$data['full_day'] = 0;
		}

		$table->bind($data);
		$table->check();
		$table->store();
	}
}
