<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Sellacious\Cache\CacheHelper;
use Sellacious\Cache\Storage\AbstractCacheStorageBuild;
use Sellacious\Database\Table\Filter\AbstractFilter;

/**
 * Class AbstractCacheBuilder
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
abstract class AbstractCacheBuilder
{
	/**
	 * Cache object name
	 *
	 * @var   string
	 *
	 * @since   2.0.0
	 */
	protected $name;

	/**
	 * Names of the columns names to be used in the cache table
	 *
	 * @var   string[]
	 *
	 * @since   2.0.0
	 */
	protected $columns = array();

	/**
	 * Record index to track how many records are currently appended. Not using <var>count($records)</var> intentionally
	 *
	 * @var   int
	 *
	 * @since   2.0.0
	 */
	protected $cursor = 0;

	/**
	 * Records queued to be added to the cache table
	 *
	 * @var   Registry[]
	 *
	 * @since   2.0.0
	 */
	protected $records = array();

	/**
	 * Number of records to hold before writing out data to the cache table. Zero value means no auto writing.
	 *
	 * @var   int
	 *
	 * @since   2.0.0
	 */
	protected $implicitFlush = 0;

	/**
	 * The storage handler
	 *
	 * @var   AbstractCacheStorageBuild
	 *
	 * @since   2.0.0
	 */
	protected $store;

	/**
	 * CacheBuilder constructor
	 *
	 * @param   string  $name  Cache object name
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct($name)
	{
		$this->name = $name;
	}

	/**
	 * The instance name
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * The insert queue
	 *
	 * @return  Registry[]
	 *
	 * @since   2.0.0
	 */
	public function getRecords()
	{
		return $this->records;
	}

	/**
	 * The insert queue size
	 *
	 * @return  int
	 *
	 * @since   2.0.0
	 */
	public function getCursor()
	{
		return $this->cursor;
	}

	/**
	 * Build the complete cache
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	abstract public function build();

	/**
	 * Build cache for given specific items
	 *
	 * @param   int|int[]  $pks
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function rebuild($pks);

	/**
	 * Delete specific cache records
	 *
	 * @param   AbstractFilter  $filter
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function remove(AbstractFilter $filter);

	/**
	 * Update specific cache records
	 *
	 * @param   array  $batch  A list of updates each containing [values, conditions]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function update(array $batch);

	/**
	 * Get a list of columns relevant to the given table
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	abstract public function getColumns();

	/**
	 * Enable implicit flush to the output database
	 *
	 * @param   int  $value  Flush automatically after {$value} records are appended. Set zero value to disable.
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function implicitFlush($value = 0)
	{
		$this->implicitFlush = (int) $value;
	}

	/**
	 * Append a new record to the cache,
	 * It will be written to the storage automatically if implicit flush is enabled
	 * Otherwise you need to call <var>flush()</var> when needed
	 *
	 * @param   Registry  $record  The record to insert
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function append(Registry $record)
	{
		$this->records[] = $record;

		$this->cursor++;

		if ($this->implicitFlush > 0 && $this->cursor >= $this->implicitFlush)
		{
			$this->flush();
		}
	}

	/**
	 * Write the records stored in write queue to the storage
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function flush()
	{
		$flush     = false;
		$isRunning = CacheHelper::isRunning();

		if ($isRunning)
		{
			$my_pid   = getmypid();
			$tmp      = Factory::getConfig()->get('tmp_path');
			$curr_pid = trim(@file_get_contents($tmp . '/.s-cache-lock/pid'));

			if ($curr_pid == $my_pid)
			{
				$flush = true;
			}
		}

		if (!$flush)
		{
			throw new Exception('Aborting cache flush as lock file removed externally.');
		}

		$app = Factory::getApplication();
		$app->enqueueMessage('Flush: ' . $this->cursor . ' at ' . microtime(true) . ' sec');

		$this->store->write($this->getName(), $this->getRecords(), $this->getCursor());

		$this->reset();
	}

	/**
	 * Reset the write queue
	 * This should be called after writing the cache or when a write is not desired
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function reset()
	{
		$this->records = array();
		$this->cursor  = 0;
	}
}
