<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use JDatabaseQuery;
use Joomla\CMS\Factory;
use Sellacious\Config\ConfigHelper;
use stdClass;

class ListingAdapter
{
	/**
	 * The flag to indicate whether preloaded
	 *
	 * @var   bool
	 *
	 * @since   2.0.0
	 */
	protected static $preloaded;

	/**
	 * The preloaded data storage
	 *
	 * @var   stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected static $cache;

	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		static $columns = array(
			'listing_active'    => 'int',
			'listing_purchased' => 'date',
			'listing_start'     => 'date',
			'listing_end'       => 'date',
		);

		return $columns;
	}

	/**
	 * Get the cache record properties
	 *
	 * @param   int  $productId
	 * @param   int  $sellerUid
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($productId, $sellerUid = null)
	{
		$config = ConfigHelper::getInstance('com_sellacious');

		if ($config->get('free_listing'))
		{
			$obj = static::getFreeExtension();
		}
		elseif (static::$preloaded)
		{
			$productId = (int) $productId;
			$sellerUid = (int) $sellerUid;

			return isset(static::$cache[$productId][$sellerUid]) ? static::$cache[$productId][$sellerUid] : null;
		}
		else
		{
			$db    = Factory::getDbo();
			$query = static::getQuery();

			$query->where('l.product_id = ' . (int) $productId);
			$query->where('l.seller_uid = ' . (int) $sellerUid);

			$obj = $db->setQuery($query)->loadObject();

			if ($obj)
			{
				unset($obj->product_id, $obj->seller_uid);

				$obj->listing_active = (int) $obj->listing_active;
			}
		}

		return $obj;
	}

	/**
	 * Preload all possible values from the database for quick batch processing
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function preload()
	{
		$config = ConfigHelper::getInstance('com_sellacious');

		if ($config->get('free_listing'))
		{
			return;
		}

		$db    = Factory::getDbo();
		$query = static::getQuery();
		$cache = array();

		$iterator = $db->setQuery($query)->getIterator();

		foreach ($iterator as $i => $obj)
		{
			$productId = (int) $obj->product_id;
			$sellerUid = (int) $obj->seller_uid;

			unset($obj->product_id, $obj->seller_uid);

			$obj->listing_active = (int) $obj->listing_active;

			$cache[$productId][$sellerUid] = $obj;
		}

		static::$cache     = $cache;
		static::$preloaded = true;
	}

	/**
	 * Get the database query for this adapter
	 *
	 * @return  JDatabaseQuery
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getQuery()
	{
		static $query;

		if (!$query)
		{
			$cols = array(
				'product_id'        => 'l.product_id',
				'seller_uid'        => 'l.seller_uid',
				'listing_active'    => 'l.state',
				'listing_purchased' => 'l.subscription_date',
				'listing_start'     => 'l.publish_up',
				'listing_end'       => 'l.publish_down',
			);

			$now  = Factory::getDate()->toSql();
			$db   = Factory::getDbo();
			$nDt  = $db->getNullDate();
			$cond = array(
				'l.publish_up != ' . $db->q($nDt),
				'l.publish_down != ' . $db->q($nDt),
				'l.publish_up <= ' . $db->q($now),
				'l.publish_down > ' . $db->q($now),
				'l.category_id = 0',
				'l.state = 1',
			);

			$query = $db->getQuery(true);

			$query->select($db->quoteName(array_values($cols), array_keys($cols)))
				->from($db->quoteName('#__sellacious_seller_listing', 'l'))
				->where($cond);
		}

		return clone $query;
	}

	/**
	 * Get the properties to set the free listing
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	protected static function getFreeExtension()
	{
		static $obj;

		if (!$obj)
		{
			$obj = new stdClass;

			$obj->listing_active = 1;
			$obj->listing_end    = Factory::getDate()->modify('+1 year')->format('Y-12-31 23:59:59');
		}

		return $obj;
	}
}
