<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use JDatabaseQuery;
use Joomla\CMS\Factory;
use stdClass;

class ManufacturerAdapter
{
	/**
	 * The flag to indicate whether preloaded
	 *
	 * @var   bool
	 *
	 * @since   2.0.0
	 */
	protected static $preloaded = false;

	/**
	 * The preloaded data storage
	 *
	 * @var   stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected static $cache;

	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		static $columns;

		if (!$columns)
		{
			$columns = array(
				'manufacturer_name'    => 'string',
				'manufacturer_company' => 'string',
				'manufacturer_catid'   => 'int',
				'manufacturer_code'    => 'string',
			);
		}

		return $columns;
	}

	/**
	 * Get the manufacturer information for the given manufacturer id
	 *
	 * @param   int  $mId  Manufacturer uid
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($mId)
	{
		if (static::$preloaded)
		{
			return isset(static::$cache[$mId]) ? static::$cache[$mId] : null;
		}
		else
		{
			$db    = Factory::getDbo();
			$query = static::getQuery();

			$query->where('u.id = ' . (int) $mId);

			$item = $db->setQuery($query)->loadObject();

			if ($item)
			{
				static::fixType($item);
			}

			return $item;
		}
	}

	/**
	 * Preload all possible values from the database for quick batch processing
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function preload()
	{
		$db    = Factory::getDbo();
		$query = static::getQuery();
		$cache = array();

		$iterator = $db->setQuery($query)->getIterator();

		foreach ($iterator as $i => $obj)
		{
			$id = $obj->manufacturer_id;

			unset($obj->manufacturer_id);

			$cache[$id] = static::fixType($obj);
		}

		static::$cache     = $cache;
		static::$preloaded = true;
	}

	/**
	 * Get the database query for this adapter
	 *
	 * @return  JDatabaseQuery
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getQuery()
	{
		static $query;

		if (!$query)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select('u.id manufacturer_id, u.name manufacturer_name')
				->from($db->quoteName('#__users', 'u'));

			$query->select('m.title manufacturer_company, m.category_id manufacturer_catid, m.code manufacturer_code')
				->join('inner', $db->quoteName('#__sellacious_manufacturers', 'm') . ' ON m.user_id = u.id');
		}

		return clone $query;
	}

	/**
	 * Convert the data type of the columns which are not explicitly definitive
	 *
	 * @param   stdClass  $obj
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	protected static function fixType($obj)
	{
		$obj->manufacturer_catid  = (int) $obj->manufacturer_catid;

		return $obj;
	}
}
