<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use JDatabaseQuery;
use Joomla\CMS\Factory;
use Sellacious\Config\ConfigHelper;
use stdClass;

class ProductAdapter
{
	/**
	 * The flag to indicate whether preloaded
	 *
	 * @var   bool
	 *
	 * @since   2.0.0
	 */
	protected static $preloaded = false;

	/**
	 * The preloaded data storage
	 *
	 * @var   stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected static $cache;

	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		static $columns;

		if (!$columns)
		{
			$columns = array(
				'product_id'          => 'int',
				'product_title'       => 'string',
				'product_alias'       => 'string',
				'product_type'        => 'string',
				'local_sku'           => 'string',
				'manufacturer_sku'    => 'string',
				'manufacturer_id'     => 'int',
				'product_features'    => 'string',
				'product_introtext'   => 'string',
				'product_description' => 'string',
				'product_active'      => 'int',
				'product_ordering'    => 'int',
				'metakey'             => 'string',
				'metadesc'            => 'string',
				'primary_video_url'   => 'string',
				'tags'                => 'string',
				'owner_uid'           => 'int',
				'language'            => 'string',
				'listing_purchased'   => 'date',
				'listing_start'       => 'date',
			);

			$config = ConfigHelper::getInstance('com_sellacious');

			if (!$config->get('multi_seller'))
			{
				$columns['product_sku'] = 'string';
			}
		}

		return $columns;
	}

	/**
	 * Get the cache record properties
	 *
	 * @param   int  $productId
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($productId)
	{
		if (static::$preloaded)
		{
			return isset(static::$cache[$productId]) ? static::$cache[$productId] : null;
		}
		else
		{
			$db    = Factory::getDbo();
			$query = static::getQuery();

			$query->where('a.id = ' . (int) $productId);

			$item = $db->setQuery($query)->loadObject();

			if ($item)
			{
				static::fixType($item);
			}

			return $item;
		}
	}

	/**
	 * Preload all possible values from the database for quick batch processing
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function preload()
	{
		$db    = Factory::getDbo();
		$query = static::getQuery();
		$cache = array();

		$iterator = $db->setQuery($query)->getIterator();

		foreach ($iterator as $i => $obj)
		{
			static::fixType($obj);

			$cache[$obj->product_id] = $obj;
		}

		static::$cache     = $cache;
		static::$preloaded = true;
	}

	/**
	 * Get the database query for this adapter
	 *
	 * @return  JDatabaseQuery
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getQuery()
	{
		static $query;

		if (!$query)
		{
			$columns = array(
				'product_id'          => 'a.id',
				'product_title'       => 'a.title',
				'product_alias'       => 'a.alias',
				'product_type'        => 'a.type',
				'local_sku'           => 'a.local_sku',
				'manufacturer_sku'    => 'a.manufacturer_sku',
				'manufacturer_id'     => 'a.manufacturer_id',
				'product_features'    => 'a.features',
				'product_introtext'   => 'a.introtext',
				'product_description' => 'a.description',
				'product_active'      => 'a.state',
				'product_ordering'    => 'a.ordering',
				'metakey'             => 'a.metakey',
				'metadesc'            => 'a.metadesc',
				'primary_video_url'   => 'a.primary_video_url',
				'tags'                => 'a.tags',
				'owner_uid'           => 'a.owned_by',
				'language'            => 'a.language',
				'listing_purchased'   => 'a.created',
				'listing_start'       => 'a.created',
			);

			$config = ConfigHelper::getInstance('com_sellacious');

			if (!$config->get('multi_seller'))
			{
				$columns['product_sku'] = 'a.local_sku';
			}

			$allowed = $config->get('allowed_product_type');
			$allowed = $allowed == 'both' ? array('physical', 'electronic') : array($allowed);

			if ($config->get('allowed_product_package'))
			{
				$allowed[] = 'package';
			}

			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select($db->quoteName(array_values($columns), array_keys($columns)));

			$query->from($db->quoteName('#__sellacious_products', 'a'))
				->where('(a.state = 0 OR a.state = 1)')
				->where('(a.type = ' . implode(' OR a.type = ', $db->quote($allowed)) . ')');
		}

		return clone $query;
	}

	/**
	 * Convert the data type of the columns which are not explicitly definitive
	 *
	 * @param   stdClass  $obj
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	protected static function fixType($obj)
	{
		$obj->product_id       = (int) $obj->product_id;
		$obj->manufacturer_id  = (int) $obj->manufacturer_id;
		$obj->product_active   = (int) $obj->product_active;
		$obj->product_ordering = (int) $obj->product_ordering;
		$obj->owner_uid        = (int) $obj->owner_uid;
		$obj->product_features = json_decode($obj->product_features);

		return $obj;
	}
}
