<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use JDatabaseQuery;
use Joomla\CMS\Factory;
use Sellacious\Cache\Builder\Adapter\AbstractCacheAdapter;
use Sellacious\Config\ConfigHelper;
use stdClass;

class PsxAdapter extends AbstractCacheAdapter
{
	/**
	 * The flag to indicate whether preloaded
	 *
	 * @var   bool
	 *
	 * @since   2.0.0
	 */
	protected static $preloaded;

	/**
	 * The preloaded data storage
	 *
	 * @var   stdClass[][]
	 *
	 * @since   2.0.0
	 */
	protected static $cache;

	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		static $columns;

		if (!$columns)
		{
			$columns = array(
				'psx_id'          => 'int',
				'seller_uid'      => 'int',
				'seller_sku'      => 'string',
				'is_selling'      => 'int',
				'pricing_type'    => 'string',
				'quantity_min'    => 'int',
				'quantity_max'    => 'int',
				'disable_stock'   => 'int',
				'stock'           => 'int',
				'over_stock'      => 'int',
				'stock_reserved'  => 'int',
				'stock_sold'      => 'int',
				'stock_capacity'  => 'int',
				'no_stock'        => 'int',
				'product_lat'     => 'float',
				'product_lng'     => 'float',
				'psx_country'     => null,
				'psx_state'       => null,
				'psx_district'    => null,
				'psx_city'        => null,
				'psx_locality'    => null,
				'psx_sublocality' => null,
				'psx_zip'         => null,
			);

			$config = ConfigHelper::getInstance('com_sellacious');

			if ($config->get('multi_seller'))
			{
				$columns['product_sku'] = 'string';
			}
		}

		return $columns;
	}

	/**
	 * Get the cache record properties
	 *
	 * @param   int  $productId
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($productId)
	{
		if (static::$preloaded)
		{
			return isset(static::$cache[$productId]) ? static::$cache[$productId] : array();
		}
		else
		{
			$db    = Factory::getDbo();
			$query = static::getQuery();

			$query->where('psx.product_id = ' . (int) $productId);

			$rows = (array) $db->setQuery($query)->loadObjectList();

			foreach ($rows as $row)
			{
				list($latP, $lngP) = explode(',', $row->product_location . ',');

				unset($row->product_id, $row->product_location);

				$row->product_lat = $latP;
				$row->product_lng = $lngP;

				static::fixType($row);
			}

			return $rows;
		}
	}

	/**
	 * Preload all possible values from the database for quick batch processing
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function preload()
	{
		$db    = Factory::getDbo();
		$query = static::getQuery();
		$cache = array();

		$iterator = $db->setQuery($query)->getIterator();

		foreach ($iterator as $i => $obj)
		{
			$productId = $obj->product_id;

			list($latP, $lngP) = explode(',', $obj->product_location . ',');

			unset($obj->product_id, $obj->product_location);

			$obj->product_lat = $latP;
			$obj->product_lng = $lngP;

			$cache[$productId][$obj->seller_uid] = static::fixType($obj);
		}

		static::$cache     = $cache;
		static::$preloaded = true;
	}

	/**
	 * Get the database query for this adapter
	 *
	 * @return  JDatabaseQuery
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getQuery()
	{
		static $query;

		if (!$query)
		{
			$columns = array(
				'psx_id'           => 'psx.id',
				'product_id'       => 'psx.product_id',
				'seller_uid'       => 'psx.seller_uid',
				'seller_sku'       => 'psx.seller_sku',
				'is_selling'       => 'psx.state',
				'pricing_type'     => 'psx.pricing_type',
				'quantity_min'     => 'psx.quantity_min',
				'quantity_max'     => 'psx.quantity_max',
				'disable_stock'    => 'psx.disable_stock',
				'stock'            => 'psx.stock',
				'over_stock'       => 'psx.over_stock',
				'stock_reserved'   => 'psx.stock_reserved',
				'stock_sold'       => 'psx.stock_sold',
				'product_location' => 'psx.product_location',
				'psx_country'      => 'psx.loc_country',
				'psx_state'        => 'psx.loc_state',
				'psx_district'     => 'psx.loc_district',
				'psx_city'         => 'psx.loc_city',
				'psx_locality'     => 'psx.loc_locality',
				'psx_sublocality'  => 'psx.loc_sublocality',
				'psx_zip'          => 'psx.loc_zip',
			);

			$db     = Factory::getDbo();
			$config = ConfigHelper::getInstance('com_sellacious');

			if ($config->get('multi_seller'))
			{
				$columns['product_sku'] = 'psx.seller_sku';
			}

			$query = $db->getQuery(true);

			$query->select($db->quoteName(array_values($columns), array_keys($columns)));
			$query->select('psx.stock + psx.over_stock AS stock_capacity');
			$query->select('psx.stock + psx.over_stock <= 0 AS no_stock');
			$query->from($db->quoteName('#__sellacious_product_sellers', 'psx'));

			if (!$config->get('multi_seller'))
			{
				$default_seller = $config->get('default_seller');

				$query->where('(psx.seller_uid = ' . (int) $default_seller . ' OR COALESCE(psx.seller_uid, 0) = 0)');
			}
		}

		return clone $query;
	}

	/**
	 * Convert the data type of the columns which are not explicitly definitive
	 *
	 * @param   stdClass  $obj
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	protected static function fixType($obj)
	{
		$obj->psx_id         = (int) $obj->psx_id;
		$obj->seller_uid     = (int) $obj->seller_uid;
		$obj->is_selling     = (int) $obj->is_selling;
		$obj->quantity_min   = (int) $obj->quantity_min;
		$obj->quantity_max   = (int) $obj->quantity_max;
		$obj->disable_stock  = (int) $obj->disable_stock;
		$obj->stock          = (int) $obj->stock;
		$obj->over_stock     = (int) $obj->over_stock;
		$obj->stock_reserved = (int) $obj->stock_reserved;
		$obj->stock_sold     = (int) $obj->stock_sold;
		$obj->stock_capacity = (int) $obj->stock_capacity;
		$obj->no_stock       = (int) $obj->no_stock;
		$obj->product_lat    = (float) $obj->product_lat;
		$obj->product_lng    = (float) $obj->product_lng;

		return $obj;
	}
}
