<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use JDatabaseQuery;
use Joomla\CMS\Factory;
use Sellacious\Cache\Builder\Adapter\AbstractCacheAdapter;
use Sellacious\Config\ConfigHelper;
use SellaciousHelper;
use stdClass;

class SellerAdapter extends AbstractCacheAdapter
{
	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		return array(
			'seller_name'       => 'string',
			'seller_username'   => 'string',
			'seller_email'      => 'string',
			'seller_block'      => 'int',
			'seller_company'    => 'string',
			'seller_catid'      => 'int',
			'seller_code'       => 'string',
			'seller_store'      => 'string',
			'store_address'     => 'string',
			'store_lat'         => 'float',
			'store_lng'         => 'float',
			'store_country'     => null,
			'store_state'       => null,
			'store_district'    => null,
			'store_city'        => null,
			'store_locality'    => null,
			'store_sublocality' => null,
			'store_zip'         => null,
			'seller_commission' => null,
			'seller_active'     => 'int',
			'seller_mobile'     => 'string',
			'seller_website'    => 'string',
			'seller_currency'   => 'string',
			'seller_gl_id'      => 'string',
			'forex_rate'        => 'float',
			'seller_rating'     => 'string',
		);
	}

	/**
	 * Get the seller information for the given seller uid
	 *
	 * @param   int  $uid  Seller uid
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($uid)
	{
		static $sellers = array();

		$uid = (int) $uid;

		if (!array_key_exists($uid, $sellers))
		{
			$sellers[$uid] = static::getSeller($uid);
		}

		return $sellers[$uid];
	}

	/**
	 * Get the database query for this adapter
	 *
	 * @return  JDatabaseQuery
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getQuery()
	{
		static $query;

		if (!$query)
		{
			$columns = array(
				'seller_name'       => 'u.name',
				'seller_username'   => 'u.username',
				'seller_email'      => 'u.email',
				'seller_block'      => 'u.block',
				'seller_company'    => 's.title',
				'seller_catid'      => 's.category_id',
				'seller_code'       => 's.code',
				'seller_store'      => 's.store_name',
				'store_address'     => 's.store_address',
				'store_location'    => 's.store_location',
				'store_country'     => 's.loc_country',
				'store_state'       => 's.loc_state',
				'store_district'    => 's.loc_district',
				'store_city'        => 's.loc_city',
				'store_locality'    => 's.loc_locality',
				'store_sublocality' => 's.loc_sublocality',
				'store_zip'         => 's.loc_zip',
				'seller_commission' => 's.commission',
				'seller_active'     => 's.state',
				'seller_mobile'     => 'p.mobile',
				'seller_website'    => 'p.website',
			);

			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select($db->quoteName(array_values($columns), array_keys($columns)));
			$query->from($db->quoteName('#__users', 'u'));
			$query->join('left', $db->quoteName('#__sellacious_sellers', 's') . ' ON s.user_id = u.id');
			$query->join('left', $db->quoteName('#__sellacious_profiles', 'p') . ' ON p.user_id = u.id');
		}

		return clone $query;
	}

	/**
	 * Get the seller record
	 *
	 * @param   int  $uid
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function getSeller($uid)
	{
		$query = static::getQuery();

		$query->where('u.id = ' . (int) $uid);

		$db  = Factory::getDbo();
		$obj = $db->setQuery($query)->loadObject();

		if ($obj)
		{
			$config     = ConfigHelper::getInstance('com_sellacious');
			$helper     = SellaciousHelper::getInstance();
			$g_currency = $helper->currency->getGlobal('code_3');

			$obj->seller_currency = $helper->currency->forSeller($uid, 'code_3');
			$obj->seller_active   = $obj->seller_active && !$obj->seller_block;

			$values = $helper->seller->getShipLocations($uid, false);

			// If no filter, add root node
			$obj->seller_gl_id = array('all' => $values ? array_map('intval', $values) : array(1));

			list($latP, $lngP) = explode(',', $obj->store_location . ',');

			unset($obj->store_location);

			$obj->store_lat = $latP;
			$obj->store_lng = $lngP;

			if ($config->get('seller_rating'))
			{
				$obj->seller_rating = $helper->rating->getSellerRating($uid);
			}

			try
			{
				$obj->forex_rate = $helper->currency->getRate($obj->seller_currency, $g_currency);
			}
			catch (Exception $e)
			{
				$obj->forex_rate = null;
			}

			static::fixType($obj);
		}

		return $obj;
	}

	/**
	 * Convert the data type of the columns which are not explicitly definitive
	 *
	 * @param   stdClass  $obj
	 *
	 * @return  stdClass
	 *
	 * @since   2.0.0
	 */
	protected static function fixType($obj)
	{
		$obj->seller_block  = (int) $obj->seller_block;
		$obj->seller_catid  = (int) $obj->seller_catid;
		$obj->seller_active = (int) $obj->seller_active;
		$obj->store_lat     = (float) $obj->store_lat;
		$obj->store_lng     = (float) $obj->store_lng;
		$obj->forex_rate    = (float) $obj->forex_rate;

		return $obj;
	}
}
