<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder\Adapter\Product;

use Exception;
use Joomla\CMS\Factory;
use stdClass;

class SpecificationAdapter
{
	/**
	 * The flag to indicate whether preloaded
	 *
	 * @var   bool
	 *
	 * @since   2.0.0
	 */
	protected static $preloaded;

	/**
	 * The preloaded data storage
	 *
	 * @var   stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected static $cache;

	/**
	 * Get the list of cache columns
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		static $columns = null;

		if ($columns === null)
		{
			$columns = array();
			$pks     = static::getFilterableFields();

			foreach ($pks as $id)
			{
				$key = sprintf('f%d', $id);

				$columns[$key] = null;
			}
		}

		return $columns;
	}

	/**
	 * Get the cache record properties
	 *
	 * @param   int  $productId
	 * @param   int  $variantId
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	public static function getRecord($productId, $variantId = 0)
	{
		if (!static::getFilterableFields())
		{
			return null;
		}

		if (static::$preloaded)
		{
			$values = static::getSpecifications($productId, $variantId);
		}
		else
		{
			$values = static::fetchSpecifications($productId, $variantId);
		}

		return $values;
	}

	/**
	 * Preload all possible values from the database for quick batch processing
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function preload()
	{
		$pks   = static::getFilterableFields();
		$cache = array();

		if ($pks)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select('a.table_name, a.record_id, a.field_id, a.field_value, a.is_json')
				->from($db->qn('#__sellacious_field_values', 'a'))
				->where("(a.table_name = 'products' OR a.table_name = 'variants')")
				->where('a.field_id IN (' . implode(',' , $pks) . ')');

			$iterator = $db->setQuery($query)->getIterator();

			foreach ($iterator as $i => $obj)
			{
				$value = $obj->is_json ? json_decode($obj->field_value) : $obj->field_value;

				$cache[$obj->table_name][$obj->record_id]['f' . $obj->field_id] = $value;
			}
		}

		static::$cache     = $cache;
		static::$preloaded = true;
	}

	/**
	 * Method to get the specifications for the given product/variant from preloaded data
	 *
	 * @param   int  $productId  Product id to query for
	 * @param   int  $variantId  Variant id to query for
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	protected static function getSpecifications($productId, $variantId = 0)
	{
		if ($variantId)
		{
			$values = isset(static::$cache['variants'][$variantId]) ? static::$cache['variants'][$variantId] : array();
		}
		else
		{
			$values = isset(static::$cache['products'][$productId]) ? static::$cache['products'][$productId] : array();
		}

		return $values;
	}

	/**
	 * Method to get the specifications for the given product/variant from database
	 *
	 * @param   int  $productId  Product id to query for
	 * @param   int  $variantId  Variant id to query for
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	protected static function fetchSpecifications($productId, $variantId = 0)
	{
		$pks   = static::getFilterableFields();
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select("CONCAT('f', a.field_id) AS fid, a.field_value AS val, a.is_json")
			->from($db->qn('#__sellacious_field_values', 'a'))
			->where('a.field_id IN (' . implode(',' , $pks) . ')');

		if (!$variantId)
		{
			$query->where('table_name = ' . $db->q('products'));
			$query->where('record_id = ' . (int) $productId);
		}
		else
		{
			$query->where('table_name = ' . $db->q('variants'));
			$query->where('record_id = ' . (int) $variantId);
		}

		$iterator = $db->setQuery($query)->getIterator();
		$values   = array();

		foreach ($iterator as $obj)
		{
			$values[$obj->fid] = $obj->is_json ? @json_decode($obj->val) : $obj->val;
		}

		return $values;
	}

	/**
	 * Method to get the list of filterable specification fields
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	protected static function getFilterableFields()
	{
		static $pks;

		if (!isset($pks))
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->select('id')
				->from($db->qn('#__sellacious_fields'))
				->where('context = ' . $db->q('product'))
				->where('type != ' . $db->q('fieldgroup'))
				->where('filterable = 1')
				->where('state = 1');

			$pks = (array) $db->setQuery($query)->loadColumn();
		}

		return $pks;
	}
}
