<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Builder;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Factory;
use Sellacious\Cache\Builder\Adapter\Product\ManufacturerAdapter;
use Sellacious\Cache\Builder\Adapter\Product\ProductAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PsxAdapter;
use Sellacious\Cache\Builder\Adapter\Product\SpecificationAdapter;
use Sellacious\Cache\CacheHelper;
use Sellacious\Cache\Record\ProductCacheRecord;
use Sellacious\Config\ConfigHelper;
use Sellacious\Database\Table\Filter\AbstractFilter;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;
use Sellacious\Price\PriceHelper;

/**
 * Class ProductsCacheBuilder
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
class ProductsCacheBuilder extends AbstractCacheBuilder
{
	/**
	 * ProductsCacheBuilder constructor
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct()
	{
		parent::__construct('products');
	}

	/**
	 * Build the complete cache
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function build()
	{
		$config = ConfigHelper::getInstance('com_sellacious');
		$db     = Factory::getDbo();
		$query  = $db->getQuery(true);

		$query->select('id')->from('#__sellacious_products');

		$iterator    = $db->setQuery($query)->getIterator();
		$this->store = CacheHelper::getBuildStorage();

		$this->store->initialise($this->getName(), $this->getColumns());

		$this->reset();

		$this->implicitFlush(1000);

		if ($config->get('cache_preload_table'))
		{
			$this->preload();
		}

		foreach ($iterator as $o)
		{
			$processor = new ProductCacheRecord($o->id);
			$records   = $processor->getRecords();

			foreach ($records as $record)
			{
				$this->append($record);
			}
		}

		$this->flush();

		$this->store->finalise($this->getName());
	}

	/**
	 * Build specific product cache
	 *
	 * @param   int|int[]  $pks
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function rebuild($pks)
	{
		$cursor = 0;
		$queue  = array();
		$pks    = array_map('intval', (array) $pks);
		$dks    = array();

		foreach ($pks as $productId)
		{
			$processor = new ProductCacheRecord($productId);
			$records   = $processor->getRecords();

			foreach ($records as $record)
			{
				$queue[] = $record;
				$dks[]   = $record->get('code');

				$cursor++;
			}
		}

		$and = new FilterSet('and');

		$and->addElement(new FilterElement('in', 'product_id', $pks));
		$and->addElement(new FilterElement('except', 'code', $dks));

		CacheHelper::getBuildStorage()->replace('products', $queue, $cursor, $and);
	}

	/**
	 * Delete specific product cache records
	 *
	 * @param   AbstractFilter  $filter
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function remove(AbstractFilter $filter)
	{
		CacheHelper::getBuildStorage()->remove('products', $filter);
	}

	/**
	 * Update specific product cache records
	 *
	 * @param   array  $batch  A list of updates each containing [values, conditions]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function update(array $batch)
	{
		CacheHelper::getBuildStorage()->update('products', $batch);
	}

	/**
	 * Get a list of columns relevant to the given table
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getColumns()
	{
		return ProductCacheRecord::getColumns();
	}

	/**
	 * Preload all adapter data before initiating cache rebuild
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function preload()
	{
		$start = microtime(true);
		$app   = Factory::getApplication();

		$app->enqueueMessage('Starting preload at ' . $start . ' sec as 0.00 sec');

		ProductAdapter::preload();

		$app->enqueueMessage('Products preloaded at ' . (microtime(true) - $start) . ' sec');

		ManufacturerAdapter::preload();

		$app->enqueueMessage('Manufacturers preloaded at ' . (microtime(true) - $start) . ' sec');

		SpecificationAdapter::preload();

		$app->enqueueMessage('Specifications preloaded at ' . (microtime(true) - $start) . ' sec');

		PsxAdapter::preload();

		$app->enqueueMessage('Psx preloaded at ' . (microtime(true) - $start) . ' sec');

		PriceHelper::preload();

		$app->enqueueMessage('Prices preloaded at ' . (microtime(true) - $start) . ' sec');

		$app->enqueueMessage('Preloading completed in ' . (microtime(true) - $start) . ' sec');
	}
}
