<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Reader;

defined('_JEXEC') or die;

use ArrayIterator;
use Exception;
use Iterator;
use JText;
use Sellacious\Cache\CacheHelper;
use Sellacious\Cache\Storage\AbstractCacheStorageRead;
use Sellacious\Database\Query\QueryBuilderTrait;
use Sellacious\Database\Table\Filter\FilterSet;
use stdClass;

/**
 * Class AbstractCacheReader
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
abstract class AbstractCacheReader
{
	use QueryBuilderTrait;

	/**
	 * Cache object name
	 *
	 * @var   string
	 *
	 * @since   2.0.0
	 */
	protected $name;

	/**
	 * The storage handler
	 *
	 * @var   AbstractCacheStorageRead
	 *
	 * @since   2.0.0
	 */
	protected $store;

	/**
	 * AbstractCacheReader constructor
	 *
	 * @param   string  $name  Cache object name
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct($name = null)
	{
		if ($name)
		{
			$this->name = $name;
		}

		$this->store   = CacheHelper::getReadStorage($this);
		$this->filters = new FilterSet('and');

		$version = CacheHelper::getSchemaVersion();
		$matched = $this->store->matchVersion($version);

		if (!$matched)
		{
			throw new Exception(JText::sprintf('COM_SELLACIOUS_ERROR_CACHE_OUTDATED_VERSION', $name));
		}
	}

	/**
	 * The instance name
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * Method to load a single record found with matching filters
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItem()
	{
		if ($this->fallacy)
		{
			return null;
		}

		return $this->store->getItem();
	}

	/**
	 * Method to load a list of records found with matching filters
	 *
	 * @param   int  $offset  The called method
	 * @param   int  $limit   The array of arguments passed to the method
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItems($offset = 0, $limit = 0)
	{
		if ($this->fallacy)
		{
			return array();
		}

		return $this->store->getItems($offset, $limit);
	}

	/**
	 * Method to load an iterator for records found with matching filters
	 *
	 * @return  Iterator
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getIterator()
	{
		if ($this->fallacy)
		{
			return new ArrayIterator;
		}

		return $this->store->getIterator();
	}

	/**
	 * Method to find number of matching records
	 *
	 * @return  int
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getTotal()
	{
		if ($this->fallacy)
		{
			return 0;
		}

		return $this->store->getTotal();
	}
}
