<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
namespace Sellacious\Cache\Record;

defined('_JEXEC') or die;

use Exception;
use Joomla\Registry\Registry;
use Sellacious\Cache\Builder\Adapter\Product\CategoryAdapter;
use Sellacious\Cache\Builder\Adapter\Product\ListingAdapter;
use Sellacious\Cache\Builder\Adapter\Product\ManufacturerAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PhysicalAdapter;
use Sellacious\Cache\Builder\Adapter\Product\ProductAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PsxAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PsxElectronicAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PsxPackageAdapter;
use Sellacious\Cache\Builder\Adapter\Product\PsxPhysicalAdapter;
use Sellacious\Cache\Builder\Adapter\Product\SellerAdapter;
use Sellacious\Cache\Builder\Adapter\Product\SpecificationAdapter;
use Sellacious\Cache\Builder\Adapter\Product\SplCategoryAdapter;
use Sellacious\Cache\Builder\Adapter\Product\VariantAdapter;
use Sellacious\Cache\Builder\Adapter\Product\VsxAdapter;
use Sellacious\Config\Config;
use Sellacious\Config\ConfigHelper;
use Sellacious\Event\EventHelper;
use Sellacious\Price\PriceHelper;
use SellaciousHelper;
use stdClass;

/**
 * @package   Sellacious\Cache
 *
 * @since   2.0.0
 */
class ProductCacheRecord
{
	/**
	 * The table columns for this record type
	 *
	 * @var   string[]
	 *
	 * @since   2.0.0
	 */
	protected static $columns = array();

	/**
	 * Sellacious application helper
	 *
	 * @var    SellaciousHelper
	 *
	 * @since   2.0.0
	 */
	protected $helper;

	/**
	 * Sellacious configuration object
	 *
	 * @var  Config
	 *
	 * @since   2.0.0
	 */
	protected $config;

	/**
	 * @var  int
	 *
	 * @since   2.0.0
	 */
	protected $productId;

	/**
	 * @var  Registry
	 *
	 * @since   2.0.0
	 */
	protected $registry;

	/**
	 * @var  Registry[]
	 *
	 * @since   2.0.0
	 */
	protected $sellers = array();

	/**
	 * @var  Registry[]
	 *
	 * @since   2.0.0
	 */
	protected $variants = array();

	/**
	 * Prices records for products / sellers
	 *
	 * @var  stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected $prices;

	/**
	 * @var  Registry[]
	 *
	 * @since   2.0.0
	 */
	protected $items = array();

	/**
	 * Constructor
	 *
	 * @param   int  $productId
	 *
	 * @since   2.0.0
	 */
	public function __construct($productId)
	{
		try
		{
			$this->registry  = null;
			$this->productId = (int) $productId;
			$this->helper    = SellaciousHelper::getInstance();
			$this->config    = ConfigHelper::getInstance('com_sellacious');
		}
		catch (Exception $e)
		{
			// Not happening
		}
	}

	/**
	 * Get the column names for the cache table
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getColumns()
	{
		if (static::$columns)
		{
			return static::$columns;
		}

		$cols = array();

		$cols[] = ProductAdapter::getColumns();
		$cols[] = ManufacturerAdapter::getColumns();
		$cols[] = CategoryAdapter::getColumns();
		$cols[] = PhysicalAdapter::getColumns();
		$cols[] = PsxAdapter::getColumns();
		$cols[] = SellerAdapter::getColumns();
		$cols[] = ListingAdapter::getColumns();
		$cols[] = PsxPhysicalAdapter::getColumns();
		$cols[] = PsxElectronicAdapter::getColumns();
		$cols[] = PsxPackageAdapter::getColumns();
		$cols[] = SplCategoryAdapter::getColumns();
		$cols[] = SpecificationAdapter::getColumns();
		$cols[] = VariantAdapter::getColumns();
		$cols[] = VsxAdapter::getColumns();

		$cols[] = array(
			'code'             => 'string',
			'seller_count'     => 'int',
			'variant_count'    => 'int',
			'product_rating'   => 'string',
			'order_count'      => 'int',
			'order_units'      => 'int',
			'related_products' => 'string',
			'product_uid'      => 'string',
			'listings_by_type' => 'string',
		);

		$cols[] = array(
			'product_price'  => 'float',
			'variant_price'  => 'float',
			'list_price'     => 'float',
			'basic_price'    => 'float',
			'sales_price'    => 'float',
			'advance_prices' => 'string',
			'sales_price_fx' => 'float',
			'price_valid'    => 'int',
		);

		$cols[] = array(
			'batch_id'   => 'string',
			'is_visible' => 'int',
		);

		$columns = array_reduce($cols, 'array_merge', array());

		// Allow plugins to extend
		$arguments = array('context' => 'com_sellacious.cache', 'name' => 'products', 'columns' => &$columns);

		EventHelper::trigger('onFetchCacheColumns', $arguments);

		$items = array();

		// B/C when datatype was not implemented
		foreach ($columns as $col => $t)
		{
			if (is_numeric($col))
			{
				$col = $t;
				$t   = null;
			}

			$items[$col] = $t;
		}

		return static::$columns = $items;
	}

	/**
	 * Get the column names for the cache table indexes
	 *
	 * @return  array
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function getIndexes()
	{
		$indexes = array(
			array('product_id'),
			array('seller_uid'),
			array('variant_id'),
			array('manufacturer_id'),
			array('product_id', 'seller_uid'),
			array(true, 'product_id', 'variant_id', 'seller_uid'),
			array('product_title', 'local_sku', 'manufacturer_sku'),
			array('product_title'),
			array('local_sku'),
			array('product_sku'),
			array('manufacturer_sku'),
			array('is_visible', 'no_stock'),
			array('batch_id'),
			array('is_visible'),
			array('batch_id', 'is_visible'),
			array('no_stock'),
			array('sales_price'),
			array('product_lat', 'product_lng'),
			array('store_lat', 'store_lng'),
			array('sales_price_fx'),
		);

		return $indexes;
	}

	/**
	 * Load the data for all variants and sellers for this product to be stored in the cache
	 *
	 * @return  Registry[]  List of cache records for this product
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getRecords()
	{
		$this->loadProduct();

		if ($this->registry)
		{
			$this->loadCategories();

			$this->loadSellerList();

			$this->loadVariantList();

			$this->loadTypeAttributes();

			$this->loadTypeAttributesSeller();

			$this->loadListing();

			$this->loadSpecialCategories();

			$this->loadRelatedProducts();

			$this->batchProcess();
		}

		return $this->items;
	}

	/**
	 * Method to load the product record for the active product
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function loadProduct()
	{
		$item = ProductAdapter::getRecord($this->productId);

		if ($item)
		{
			$this->registry = new Registry($item);

			if ($item->manufacturer_id)
			{
				$user = ManufacturerAdapter::getRecord($item->manufacturer_id);

				if ($user)
				{
					$this->registry->loadObject($user);
				}
			}
		}
	}

	/**
	 * Method to load the list of sellers for the active product
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function loadSellerList()
	{
		$records = PsxAdapter::getRecord($this->productId);
		$count   = 0;

		foreach ($records as $psx)
		{
			$uid    = (int) $psx->seller_uid;
			$seller = SellerAdapter::getRecord($uid);

			if ($seller)
			{
				if ($seller->seller_active)
				{
					$count++;
				}

				if (!$psx->psx_country && !$psx->psx_state && !$psx->psx_district &&
					!$psx->psx_city && !$psx->psx_locality && !$psx->psx_sublocality && !$psx->psx_zip)
				{
					$psx->psx_country     = $seller->store_country;
					$psx->psx_state       = $seller->store_state;
					$psx->psx_district    = $seller->store_district;
					$psx->psx_city        = $seller->store_city;
					$psx->psx_locality    = $seller->store_locality;
					$psx->psx_sublocality = $seller->store_sublocality;
					$psx->psx_zip         = $seller->store_zip;
				}

				if (!$psx->product_lat && !$psx->product_lng)
				{
					$psx->product_lat = $seller->store_lat;
					$psx->product_lng = $seller->store_lng;
				}
			}

			$registry = new Registry;

			$registry->loadObject($psx);
			$registry->loadObject($seller);

			$this->sellers[$uid] = $registry;
		}

		$this->registry->set('seller_count', $count);
	}

	/**
	 * Method to load the list of variants for the active product
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function loadVariantList()
	{
		if ($this->config->get('multi_variant'))
		{
			$adapter  = new VariantAdapter;
			$variants = $adapter->getRecord($this->productId);

			foreach ($variants as $vid => $record)
			{
				$this->variants[$vid] = new Registry($record);
			}

			$this->registry->set('variant_count', count($this->variants));
		}
	}

	/**
	 * Method to load the list of categories for the active product
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function loadCategories()
	{
		$adapter = new CategoryAdapter;
		$obj     = $adapter->getRecord($this->productId);

		$this->registry->loadObject($obj);
	}

	/**
	 * Method to load the product type specific attributes for the active product
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function loadTypeAttributes()
	{
		if ($this->registry->get('product_type') === 'physical')
		{
			$adapter = new PhysicalAdapter;

			$obj = $adapter->getRecord($this->productId);

			$this->registry->loadObject($obj);
		}
	}

	/**
	 * Method to load the product type specific attributes for the active product for each seller
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function loadTypeAttributesSeller()
	{
		$type = $this->registry->get('product_type');

		switch ($type)
		{
			case 'electronic':
				$adapter = new PsxElectronicAdapter;
				break;
			case 'physical':
				$adapter = new PsxPhysicalAdapter;
				break;
			case 'package':
				$adapter = new PsxPackageAdapter;
				break;
			default:
				return;
		}

		foreach ($this->sellers as $sKey => $seller)
		{
			$obj = $adapter->getRecord($seller->get('psx_id'));

			$this->sellers[$sKey]->loadObject($obj);
		}
	}

	/**
	 * Method to load the listing attributes for the active product and each of its sellers
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function loadListing()
	{
		foreach ($this->sellers as $seller)
		{
			$obj = ListingAdapter::getRecord($this->productId, $seller->get('seller_uid'));

			if ($obj)
			{
				$seller->loadObject($obj);
			}
		}
	}

	/**
	 * Method to load the special categories associated with the active product and each of its sellers
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function loadSpecialCategories()
	{
		$adapter = new SplCategoryAdapter;

		foreach ($this->sellers as $seller)
		{
			$obj = $adapter->getRecord($this->productId, $seller->get('seller_uid'));

			$seller->loadObject($obj);
		}
	}

	protected function loadRelatedProducts()
	{
		$related = $this->helper->relatedProduct->getByProduct($this->productId);

		$this->registry->set('related_products', array_map('intval', $related));
	}

	/**
	 * Process the records for caching
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function batchProcess()
	{
		foreach ($this->sellers as $seller)
		{
			$registry = new Registry;

			$registry->merge($this->registry);
			$registry->merge($seller);

			$specifications = SpecificationAdapter::getRecord($this->productId);

			$registry->loadArray($specifications);

			$this->append($this->makeRecord($registry, $specifications));

			if ($this->config->get('multi_variant'))
			{
				$adapter = new VsxAdapter;

				foreach ($this->variants as $variant)
				{
					$vsx = $adapter->getRecord($variant->get('variant_id'), $seller->get('seller_uid'));

					if ($vsx)
					{
						$this->append($this->makeRecord($registry, $specifications, $variant, $vsx));
					}
				}
			}
		}

		$this->addPrices();

		foreach ($this->items as $item)
		{
			$item->set('sales_price_fx', round($item->get('sales_price') * $item->get('forex_rate'), 2));
		}

		EventHelper::trigger('onProcessCacheRecord', array('context' => 'com_sellacious.product', 'items' => &$this->items));

		$this->checkListable();

		$byType = $this->getListingsByType();

		foreach ($this->items as $item)
		{
			$item->set('listings_by_type', $byType);
		}
	}

	/**
	 * Method to populate variant level record from base product record
	 *
	 * @param   Registry  $registry        The base record object with just the common values for all variants
	 * @param   array     $specifications  The main product specifications, @see   getSpecifications(), unused for now, for later
	 * @param   Registry  $variant         The variant record, @see  loadVariantList()
	 * @param   stdClass  $vsx             The variant-seller attributes, @see   getVsx()
	 *
	 * @return  Registry
	 *
	 * @since   2.0.0
	 */
	protected function makeRecord(Registry $registry, $specifications = array(), $variant = null, $vsx = null)
	{
		$item = new Registry;

		$item->merge($registry);
		$item->merge($variant);

		$item->loadObject($vsx);

		$variantId = $item->get('variant_id');
		$sellerUid = $item->get('seller_uid');

		$code       = $this->helper->product->getCode($this->productId, $variantId, $sellerUid);
		$orderCount = $this->helper->order->getOrderCount($this->productId, $variantId, $sellerUid);
		$orderUnits = $this->helper->order->getOrderCount($this->productId, $variantId, $sellerUid, true);

		if ($this->config->get('product_rating'))
		{
			$rating = $this->helper->rating->getProductRating($this->productId, $variantId);

			$item->set('product_rating', $rating);
		}

		if ($variantId)
		{
			$vSpecifications = SpecificationAdapter::getRecord($this->productId, $variantId);

			$item->loadArray($vSpecifications);
		}

		$item->set('code', $code);
		$item->set('variant_id', (int) $variantId);
		$item->set('order_count', (int) $orderCount);
		$item->set('order_units', (int) $orderUnits);
		$item->set('sales_price_fx', null);
		$item->set('product_uid', $item->get('seller_count') > 1 ? $code : (int) $this->productId);

		return $item;
	}

	/**
	 * Method to get items by listing type
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	protected function getListingsByType()
	{
		$show  = $this->config->get('show_listing_type');
		$pType = $this->registry->get('product_type');
		$types = (array) $this->config->get('allowed_listing_type');

		if (!$show || $pType == 'electronic')
		{
			return null;
		}

		$results = array();

		foreach ($types as $type)
		{
			$tmp   = null;
			$sids  = array();
			$codes = array();

			foreach ($this->items as $item)
			{
				if ($item->get('is_visible') &&
					$item->get('listing_type', 1) == $type &&
					in_array($item->get('pricing_type'), array('basic', 'dynamic', 'flat', 'free')))
				{
					if ($tmp === null || $item->get('sales_price_fx') < $tmp->get('sales_price_fx'))
					{
						$tmp = $item;
					}

					$uid = $item->get('code');

					$sids[$uid] = 1;

					$codes[] = $item->get('code');
				}
			}

			if ($tmp)
			{
				$results[$type] = array(
					'listing_type' => $type,
					'count'        => count($sids),
					'from_price'   => $tmp->get('sales_price'),
					'currency'     => $tmp->get('seller_currency'),
					'item_uids'    => $codes,
				);
			}
		}

		return $results;
	}

	/**
	 * Get seller-specific prices for this product. Please note that all amounts are in seller's currency.
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function addPrices()
	{
		$pTypes = array();

		// Collate by pricing type
		foreach ($this->items as $item)
		{
			$pt = $item->get('pricing_type');

			$pTypes[$pt] = true;
		}

		$pTypes = array_keys($pTypes);

		// Only allowed pricing types to be called
		$allowed = $this->config->get('pricing_types', array());
		$pTypes  = $allowed ? array_intersect($pTypes, $allowed) : $pTypes;

		foreach ($pTypes as $pt)
		{
			try
			{
				$handler = PriceHelper::getHandler($pt);
				$handler->setPricesForCache($this->productId, $this->items);
			}
			catch (Exception $e)
			{
				// Ignore for now
			}
		}
	}

	/**
	 * Method to check items that can be display in the list view on account of
	 * special listing or best price and grouped display of sellers and variants of a product
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function checkListable()
	{
		$hzp   = $this->config->get('hide_zero_priced');
		$hns   = $this->config->get('hide_out_of_stock');
		$s_sep = $this->config->get('multi_seller') == 2;
		$v_sep = $this->config->get('multi_variant') == 2;

		$items = array();
		$batch = 'P%1$d' . ($v_sep ? 'V%2$d' : '') . ($s_sep ? 'S%3$d' : '');

		foreach ($this->items as $item)
		{
			$item->set('is_visible', 0);
			$item->set('batch_id', sprintf($batch, $item->get('product_id'), $item->get('variant_id'), $item->get('seller_uid')));

			$valid = $item->get('product_active') &&
				($item->get('variant_active') || !$item->get('variant_id')) &&
				$item->get('listing_active') && $item->get('seller_active') && $item->get('is_selling') &&
				(!$hzp || $item->get('price_valid')) &&
				(!$hns || $item->get('stock_capacity') > 0);

			if ($valid)
			{
				$items[] = $item;
			}
		}

		usort($items, array($this, 'sortItems'));

		foreach ($items as $index => $item)
		{
			$item->set('is_visible', $index + 1);
		}
	}

	/**
	 * Method to find the best seller/variant item among given items
	 *
	 * @param   Registry  $a
	 * @param   Registry  $b
	 *
	 * @return  int
	 *
	 * @since   2.0.0
	 */
	protected function sortItems(Registry $a, Registry $b)
	{
		$aWin = 1;
		$bWin = -1;
		$eTie = 0;

		$aInStock = $a->get('stock_capacity') > 0;
		$bInStock = $b->get('stock_capacity') > 0;
		$aPriceOk = $a->get('price_valid');
		$bPriceOk = $b->get('price_valid');
		$aSplOk   = $a->get('spl_category');
		$bSplOk   = $b->get('spl_category');

		// We first prioritise the in-stock item
		$byStock = $aInStock ? ($bInStock ? $eTie : $aWin) : ($bInStock ? $bWin : false);

		if ($byStock === $aWin)
		{
			return $aPriceOk ? $aWin : $eTie;
		}
		elseif ($byStock === $bWin)
		{
			return $bPriceOk ? $bWin : $eTie;
		}
		elseif ($byStock === false)
		{
			return $eTie;
		}

		// Next by valid-price if both are in-stock
		$byPriceOk = $aPriceOk ? ($bPriceOk ? $eTie : $aWin) : ($bPriceOk ? $bWin : false);

		if ($byPriceOk === $aWin || $byPriceOk === $bWin)
		{
			return (int) $byPriceOk;
		}
		elseif ($byPriceOk === false)
		{
			return $eTie;
		}

		// Next by special category (better among special categories is not considered yet) if both price are valid
		$bySpl = $aSplOk ? ($bSplOk ? $eTie : $aWin) : ($bSplOk ? $bWin : $eTie);

		if ($bySpl === $aWin)
		{
			return $aPriceOk ? $aWin : $eTie;
		}
		elseif ($bySpl === $bWin)
		{
			return $bPriceOk ? $bWin : $eTie;
		}

		// The types free/basic/flat/dynamic are better than call/email/queryform/hidden
		$mType = array('free', 'basic', 'flat', 'dynamic');
		$aM    = in_array($a->get('pricing_type'), $mType);
		$bM    = in_array($b->get('pricing_type'), $mType);

		if ($aM && !$bM)
		{
			return $aWin;
		}

		if (!$aM && $bM)
		{
			return $bWin;
		}

		if ($aM && $bM)
		{
			// Next by cheapest price if both has or has not special category, 'free' is also the best
			$delta = round($a->get('sales_price_fx') - $b->get('sales_price_fx'), 2);

			if (abs($delta) >= 0.01)
			{
				return ($delta < 0) ? $aWin : $bWin;
			}
		}

		// Now seller rating should be considered
		$delta = round($a->get('seller_rating.rating') - $b->get('seller_rating.rating'), 2);

		if (abs($delta) >= 0.01)
		{
			return ($delta > 0) ? $aWin : $bWin;
		}

		return $eTie;
	}

	/**
	 * Add a record to the result list
	 *
	 * @param   Registry  $registry
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function append(Registry $registry)
	{
		foreach (static::getColumns() as $column => $type)
		{
			$registry->def($column, '');
		}

		$this->items[] = $registry;
	}
}
