<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Storage;

use Exception;
use Joomla\Registry\Registry;
use Sellacious\Database\Table\Filter\AbstractFilter;

defined('_JEXEC') or die;

/**
 * Class AbstractCacheStorageBuild
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
abstract class AbstractCacheStorageBuild
{
	const REBUILD_FULL = 1;

	const REBUILD_PARTIAL = 2;

	const REBUILD_PATCH = 3;

	/**
	 * Mark the beginning of a cache rebuild and let the storage handler prepare itself
	 *
	 * @param   string  $name     The table name
	 * @param   array   $columns  The table columns
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function initialise($name, $columns);

	/**
	 * Send the cache records for write in the cache database during full rebuild
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function write($name, $records, $length);

	/**
	 * Send the cache records for write in the cache database for partial update in live cache
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function append($name, $records, $length);

	/**
	 * Send the cache records for write in the cache and also remove some records
	 * from the cache database for partial update in live cache
	 *
	 * @param   string          $name
	 * @param   Registry[]      $records
	 * @param   int             $length
	 * @param   AbstractFilter  $removeFilter
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function replace($name, $records, $length, $removeFilter = null);

	/**
	 * Mark the end of cache rebuild and let the storage handler perform any post-write cleanup etc
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function finalise($name);

	/**
	 * Method to delete a list of records with matching filters
	 *
	 * @param   string          $name
	 * @param   AbstractFilter  $filters
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function remove($name, AbstractFilter $filters);

	/**
	 * Update some of the cache records in the cache database
	 *
	 * @param   string  $name   The table name
	 * @param   array   $batch  A list of updates each containing [values, conditions]
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	abstract public function update($name, array $batch);
}
