<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Storage\Remote;

use Exception;
use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;
use Joomla\Registry\Registry;
use Sellacious\Cache\Record\ProductCacheRecord;
use Sellacious\Cache\Storage\AbstractCacheStorageBuild;
use Sellacious\Database\Table\Filter\AbstractFilter;

defined('_JEXEC') or die;

/**
 * Class RemoteCacheStorageBuild
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
class RemoteCacheStorageBuild extends AbstractCacheStorageBuild
{
	const CMD_FULL_START = 'rebuild/initiate';

	const CMD_FULL_WRITE = 'rebuild/write';

	const CMD_FULL_ENDED = 'rebuild/finalise';

	const CMD_FULL_ABORT = 'force_unlock';

	const CMD_FULL_CLEAR = 'clear';

	const CMD_PATCH_DELETE = 'delete';

	const CMD_PATCH_APPEND = 'append';

	const CMD_PATCH_UPDATE = 'update';

	const CMD_PATCH_REPLACE = 'replace';

	/**
	 * Mark the beginning of a cache rebuild and let the storage handler prepare itself
	 *
	 * @param   string  $name     The collection name
	 * @param   array   $columns  The collection fields
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function initialise($name, $columns)
	{
		$indexes   = array();
		$indexCols = ProductCacheRecord::getIndexes();

		foreach ($indexCols as $i => $index)
		{
			foreach ($index as $col)
			{
				$indexes[$i][$col] = 1;
			}
		}

		$args = array(
			'collection' => $name,
			'indexes'    => $indexes,
		);

		$this->send(self::CMD_FULL_START, $args);
	}

	/**
	 * Send the cache records for write in the cache database during full rebuild
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function write($name, $records, $length)
	{
		$args   = array(
			'collection' => $name,
			'records'    => $records,
			'length'     => $length,
		);

		$this->send(self::CMD_FULL_WRITE, $args);
	}

	/**
	 * Send the cache records for write in the cache database for partial update in live cache
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function append($name, $records, $length)
	{
		$args   = array(
			'collection' => $name,
			'records'    => $records,
			'length'     => $length,
		);

		$this->send(self::CMD_PATCH_APPEND, $args);
	}

	/**
	 * Send the cache records for write in the cache and also remove some records
	 * from the cache database for partial update in live cache
	 *
	 * @param   string          $name
	 * @param   Registry[]      $records
	 * @param   int             $length
	 * @param   AbstractFilter  $removeFilter
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function replace($name, $records, $length, $removeFilter = null)
	{
		$args   = array(
			'collection' => $name,
			'remove'     => $removeFilter,
			'records'    => $records,
			'length'     => $length,
		);

		$this->send(self::CMD_PATCH_REPLACE, $args);
	}

	/**
	 * Mark the end of cache rebuild and let the storage handler perform any post-write cleanup etc
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function finalise($name)
	{
		$args = array('collection' => $name);

		$this->send(self::CMD_FULL_ENDED, $args);
	}

	/**
	 * Method to delete a list of records with matching filters
	 *
	 * @param   string          $name
	 * @param   AbstractFilter  $filters
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function remove($name, AbstractFilter $filters)
	{
		$args = array(
			'collection' => $name,
			'filters'    => $filters,
		);

		$this->send(self::CMD_PATCH_DELETE, $args);
	}

	/**
	 * Update some of the cache records in the cache database
	 *
	 * @param   string  $name   The table name
	 * @param   array   $batch  A list of updates each containing [values, conditions]
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function update($name, array $batch)
	{
		$args = array(
			'collection' => $name,
			'batch'      => $batch,
		);

		$this->send(self::CMD_PATCH_UPDATE, $args);
	}

	/**
	 * Send the request to the remote api and handle the response received
	 *
	 * @param   string  $command  The command for the API
	 * @param   array   $args     The post data for the API request
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function send($command, array $args = array())
	{
		$config = Factory::getConfig();
		$apiUrl = $config->get('sellacious_cache_api');

		if (!preg_match('/https?:\/\/.+/i', $apiUrl))
		{
			throw new Exception('Invalid remote cache api endpoint.');
		}

		$url    = $apiUrl . '/' . $command;
		$secret = $config->get('secret');
		$header = array(
			'Content-Type'       => 'application/json',
			'X-Validation-Token' => md5($secret),
		);
		$data   = json_encode($args);
		$http   = HttpFactory::getHttp();
		$resp   = $http->post($url, $data, $header, 30);

		if ($resp->code != 200)
		{
			throw new Exception($resp->body);
		}
	}
}
