<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Storage\Remote;

use ArrayIterator;
use Exception;
use Iterator;
use JLog;
use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Http\Response;
use Sellacious\Cache\Storage\AbstractCacheStorageRead;
use Sellacious\Database\Table\Field\SelectField;
use stdClass;

defined('_JEXEC') or die;

/**
 * Class RemoteCacheStorageRead
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
class RemoteCacheStorageRead extends AbstractCacheStorageRead
{
	const CMD_GET_LIST = 'get_list';

	const CMD_GET_ITEM = 'get_item';

	const CMD_GET_TOTAL = 'get_total';

	protected $totals = array();

	/**
	 * Method to check the schema version for the application compatibility
	 *
	 * @param   int  $version
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function matchVersion($version)
	{
		return true;
	}

	/**
	 * Send the request to the remote api and handle the response received
	 *
	 * @param   string  $command  The command for the API
	 * @param   array   $args     The post data for the API request
	 *
	 * @return  Response
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function send($command, array $args = array())
	{
		$config = Factory::getConfig();
		$apiUrl = $config->get('sellacious_cache_api');

		if (!preg_match('/https?:\/\/.+/i', $apiUrl))
		{
			throw new Exception('Invalid remote cache api endpoint.');
		}

		$url    = $apiUrl . '/' . $command;
		$secret = $config->get('secret');
		$header = array(
			'Content-Type'       => 'application/json',
			'X-Validation-Token' => md5($secret),
		);
		$data   = json_encode($args);
		$http   = HttpFactory::getHttp();

		return $http->post($url, $data, $header, 30);
	}

	/**
	 * Method to load a single record found with matching filters
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItem()
	{
		$args = array(
			'collection' => $this->reader->getName(),
			'select'     => $this->reader->getSelects(),
			'filters'    => $this->reader->getFilters(),
			'ordering'   => $this->reader->getSortOrders(),
		);

		$resp = $this->send(self::CMD_GET_ITEM, $args);

		$result = @json_decode($resp->body);

		if (is_object($result))
		{
			if ($resp->code == 200)
			{
				if (isset($result->item))
				{
					return $result->item;
				}
			}
			else
			{
				if (isset($result->error))
				{
					throw new Exception($result->error);
				}
			}
		}

		JLog::add('Invalid api response: ' . json_encode($resp), JLog::WARNING, 'jerror');

		return null;
	}

	/**
	 * Method to load a list of records found with matching filters
	 *
	 * @param   int  $offset  The called method
	 * @param   int  $limit   The array of arguments passed to the method
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItems($offset = 0, $limit = 0)
	{
		$name = $this->reader->getName();
		$args = array(
			'collection' => $name,
			'select'     => $this->reader->getSelects() ?: array(new SelectField('*')),
			'filters'    => $this->reader->getFilters(),
			'ordering'   => $this->reader->getSortOrders(),
			'offset'     => $offset,
			'limit'      => $limit,
		);

		$resp = $this->send(self::CMD_GET_LIST, $args);

		$result = @json_decode($resp->body);

		if (is_object($result))
		{
			if ($resp->code == 200)
			{
				if (isset($result->{$name}))
				{
					if (isset($result->total))
					{
						$cid = $name . md5(json_encode($this->reader->getFilters()));

						$this->totals[$cid] = $result->total;
					}

					return $result->{$name};
				}
			}
			else
			{
				if (isset($result->error))
				{
					throw new Exception($result->error);
				}
			}
		}

		JLog::add('Invalid api response: ' . json_encode($resp), JLog::WARNING, 'jerror');

		return array();
	}

	/**
	 * Method to load an iterator for records found with matching filters
	 *
	 * @return  Iterator
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getIterator()
	{
		$items = $this->getItems();

		return new ArrayIterator($items);
	}

	/**
	 * Method to find number of matching records
	 *
	 * @param   bool  $cached
	 *
	 * @return  int
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getTotal($cached = true)
	{
		$name = $this->reader->getName();

		if ($cached)
		{
			$cid = $name . md5(json_encode($this->reader->getFilters()));

			if (isset($this->totals[$cid]))
			{
				return $this->totals[$cid];
			}
		}

		$args = array(
			'collection' => $name,
			'filters'    => $this->reader->getFilters(),
		);

		$resp   = $this->send(self::CMD_GET_TOTAL, $args);
		$result = @json_decode($resp->body);

		if (is_object($result))
		{
			if ($resp->code == 200)
			{
				if (isset($result->{$name}))
				{
					return $result->{$name};
				}
			}
			else
			{
				if (isset($result->error))
				{
					throw new Exception($result->error);
				}
			}
		}

		JLog::add('Invalid api response: ' . json_encode($resp), JLog::WARNING, 'jerror');

		return 0;
	}
}
