<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Storage\Sqlite;

use Exception;
use JDatabaseQuerySqlite;
use JFile;
use JFolder;
use Joomla\Registry\Registry;
use PDO;
use PDOStatement;
use Sellacious\Cache\CacheHelper;
use Sellacious\Cache\Record\ProductCacheRecord;
use Sellacious\Cache\Storage\AbstractCacheStorageBuild;
use Sellacious\Database\SQLite\SQLiteQueryParser;
use Sellacious\Database\Table\Filter\AbstractFilter;

defined('_JEXEC') or die;

/**
 * Class SqliteCacheStorageBuild
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
class SqliteCacheStorageBuild extends AbstractCacheStorageBuild
{
	/**
	 * PDO connection object to the storage database
	 *
	 * @var   PDO
	 *
	 * @since   2.0.0
	 */
	protected $dbo;

	/**
	 * PDO prepared statement for record insertion
	 *
	 * @var   PDOStatement
	 *
	 * @since   2.0.0
	 */
	protected $statement;

	/**
	 * Table column names
	 *
	 * @var   string[]
	 *
	 * @since   2.0.0
	 */
	protected $columns;

	/**
	 * Mark the beginning of a cache rebuild and let the storage handler prepare itself
	 *
	 * @param   string  $name     The table name
	 * @param   array   $columns  The table columns
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function initialise($name, $columns)
	{
		$this->createTemporaryStorage($name);

		$this->setupTable($name, $columns);

		$this->columns   = $columns;
		$this->statement = $this->createStatement($name);
	}

	/**
	 * Send the cache records for write in the cache database during full rebuild
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function write($name, $records, $length)
	{
		try
		{
			if (!isset($this->statement))
			{
				throw new Exception('The cache build insert batch was not initialised.');
			}

			$this->dbo->beginTransaction();

			foreach ($records as $record)
			{
				$parameters = $record->toArray();

				foreach ($parameters as $key => $value)
				{
					if (!is_scalar($value))
					{
						$parameters[$key] = json_encode($value);
					}
				}

				$this->statement->execute($parameters);
			}

			$this->dbo->commit();
		}
		catch (Exception $e)
		{
			$this->dbo->rollBack();

			throw $e;
		}
	}

	/**
	 * Send the cache records for write in the cache database for partial update in live cache
	 *
	 * @param   string      $name
	 * @param   Registry[]  $records
	 * @param   int         $length
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function append($name, $records, $length)
	{
		try
		{
			$this->loadStorage($name);

			$statement = $this->createStatement($name, true);

			$this->dbo->beginTransaction();

			foreach ($records as $record)
			{
				$parameters = $record->toArray();

				foreach ($parameters as $key => $value)
				{
					if (!is_scalar($value))
					{
						$parameters[$key] = json_encode($value);
					}
				}

				$statement->execute($parameters);
			}

			$this->dbo->commit();
		}
		catch (Exception $e)
		{
			$this->dbo->rollBack();

			throw $e;
		}
	}

	/**
	 * Send the cache records for write in the cache and also remove some records
	 * from the cache database for partial update in live cache
	 *
	 * @param   string          $name
	 * @param   Registry[]      $records
	 * @param   int             $length
	 * @param   AbstractFilter  $removeFilter
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function replace($name, $records, $length, $removeFilter = null)
	{
		$this->loadStorage($name);

		$this->remove($name, $removeFilter);

		$this->append($name, $records, $length);
	}

	/**
	 * Mark the end of cache rebuild and let the storage handler perform any post-write cleanup etc
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function finalise($name)
	{
		$this->statement = null;

		$indexes = ProductCacheRecord::getIndexes();

		foreach ($indexes as $index)
		{
			if (reset($index) === true)
			{
				array_shift($index);

				$str = 'CREATE UNIQUE INDEX IF NOT EXISTS idx_%s ON %s (%s)';
			}
			else
			{
				$str = 'CREATE INDEX IF NOT EXISTS idx_%s ON %s (%s)';
			}

			$this->dbo->exec(sprintf($str, implode('_', $index), $name, implode(', ', $index)));
		}

		$this->setLive($name);
	}

	/**
	 * Method to delete a list of records with matching filters
	 *
	 * @param   string          $name
	 * @param   AbstractFilter  $filters
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function remove($name, AbstractFilter $filters)
	{
		$this->loadStorage($name);

		$query      = new JDatabaseQuerySqlite;
		$conditions = SQLiteQueryParser::filter($filters);

		$query->delete($name);

		if ($conditions)
		{
			$query->where($conditions);
		}

		$this->dbo->exec((string) $query);
	}

	/**
	 * Update some of the cache records in the cache database
	 *
	 * @param   string  $name   The table name
	 * @param   array   $batch  A list of updates each containing [values, conditions]
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function update($name, array $batch)
	{
		$this->loadStorage($name);

		foreach ($batch as $command)
		{
			$values     = $command['values'];
			$conditions = $command['conditions'];

			$query = new JDatabaseQuerySqlite;

			$query->update($name);

			foreach ($values as $key => $value)
			{
				$query->set($key . ' = ?');
			}

			if ($conditions instanceof AbstractFilter)
			{
				$where = SQLiteQueryParser::filter($conditions);

				if ($where)
				{
					$query->where($where);
				}
			}

			$ps = $this->dbo->prepare($query);

			$ps->execute(array_values($values));
		}
	}

	/**
	 * Load the cache storage database if it exists
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function createTemporaryStorage($name)
	{
		$filename = CacheHelper::getFilename($name);
		$filename = $filename . '.tmp';

		if (file_exists($filename))
		{
			JFile::delete($filename);
		}

		$this->openSqliteDatabase($filename);

		$version = CacheHelper::getSchemaVersion();

		$this->dbo->exec('PRAGMA user_version = ' . (int) $version);
	}

	/**
	 * Open a connection to the sqlite database file if it exists or create one if it doesn't exist
	 *
	 * @param   string  $path  Absolute path the the database file
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function openSqliteDatabase($path)
	{
		if (!JFolder::exists(dirname($path)))
		{
			JFolder::create(dirname($path));
		}

		$dbo = new PDO('sqlite:' . $path);

		$dbo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$dbo->setAttribute(PDO::ATTR_CASE, PDO::CASE_LOWER);

		$this->dbo = $dbo;
	}

	/**
	 * Create the storage table
	 *
	 * @param   string  $tableName  The storage table name
	 * @param   array   $columns    The table columns
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function setupTable($tableName, array $columns)
	{
		$cols = array('id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT');

		foreach ($columns as $col => $type)
		{
			switch ($type)
			{
				case 'int':
					$cols[] = $col . ' INT';
					break;
				case 'float':
					$cols[] = $col . ' FLOAT';
					break;
				default:
					$cols[] = $col . ' VARCHAR';
					break;
			}
		}

		$this->dbo->exec('DROP TABLE IF EXISTS ' . $tableName);

		$this->dbo->exec('VACUUM');

		$this->dbo->query('CREATE TABLE IF NOT EXISTS ' . $tableName . ' (' . implode(', ', $cols) . ')');
	}

	/**
	 * Retrieves field list for a given table
	 *
	 * @param   string  $tableName  The name of the database table
	 *
	 * @return  string[]  An array of fields for the database table
	 *
	 * @since   2.0.0
	 */
	protected function getTableColumns($tableName)
	{
		$columns = array();
		$result  = $this->dbo->query('pragma table_info(' . $tableName . ')');

		while ($field = $result->fetchObject())
		{
			$columns[] = $field->name;
		}

		return $columns;
	}

	/**
	 * Create a prepared statement from PDO instance for record inserts
	 *
	 * @param   string  $tableName  The storage table name to insert into, defaults to current db name
	 * @param   bool    $replace    Whether to create a REPLACE INTO statement instead of INSERT INTO.
	 *
	 * @return  PDOStatement
	 *
	 * @since   2.0.0
	 */
	protected function createStatement($tableName, $replace = false)
	{
		$columns = $this->getTableColumns($tableName);
		$values  = array_map(function ($s) { return ':' . $s; }, $columns);

		if ($replace)
		{
			$query = sprintf('REPLACE INTO %s (%s) VALUES (%s);', $tableName, implode(', ', $columns), implode(', ', $values));
		}
		else
		{
			$query = sprintf('INSERT INTO %s (%s) VALUES (%s);', $tableName, implode(', ', $columns), implode(', ', $values));
		}

		return $this->dbo->prepare($query);
	}

	/**
	 * Replace the actual cache database with the newly created temporary cache storage
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function setLive($name)
	{
		$filename = CacheHelper::getFilename($name);

		JFile::delete($filename);

		if (IS_WIN)
		{
			// File lock is not released at this point, so just copy! Fixes for later.
			JFile::copy($filename . '.tmp', $filename);
		}
		else
		{
			JFile::move($filename . '.tmp', $filename);
		}
	}

	/**
	 * Load the cache storage database if it exists
	 *
	 * @param   string  $name  The collection name
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function loadStorage($name)
	{
		if (!$this->dbo)
		{
			$filename = CacheHelper::getFilename($name);

			$this->openSqliteDatabase($filename);
		}
	}
}
