<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Cache\Storage\Sqlite;

use Exception;
use JDatabaseIterator;
use JDatabaseQuerySqlite;
use JText;
use PDO;
use Sellacious\Cache\CacheHelper;
use Sellacious\Cache\Exception\CacheMissingException;
use Sellacious\Cache\Reader\AbstractCacheReader;
use Sellacious\Cache\Storage\AbstractCacheStorageRead;
use Sellacious\Database\SQLite\SQLiteDatabaseDriver;
use Sellacious\Database\SQLite\SQLiteFunctions;
use Sellacious\Database\SQLite\SQLiteQueryParser;
use stdClass;

defined('_JEXEC') or die;

/**
 * Class SqliteCacheStorageRead
 *
 * @package  Sellacious\Cache
 *
 * @since   2.0.0
 */
class SqliteCacheStorageRead extends AbstractCacheStorageRead
{
	/**
	 * Database connection object to the storage database
	 *
	 * @var   SQLiteDatabaseDriver
	 *
	 * @since   2.0.0
	 */
	protected $db;

	/**
	 * The sql statement for reading cache database
	 *
	 * @var  JDatabaseQuerySqlite
	 *
	 * @since   2.0.0
	 */
	protected $query;

	/**
	 * AbstractCacheStorage constructor
	 *
	 * @param   AbstractCacheReader  $reader
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct(AbstractCacheReader $reader)
	{
		parent::__construct($reader);

		$filename = CacheHelper::getFilename($this->reader->getName());

		$this->openSqliteDatabase($filename);
	}

	/**
	 * Method to check the schema version for the application compatibility
	 *
	 * @param   int  $version
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function matchVersion($version)
	{
		$value = $this->db->setQuery('PRAGMA user_version')->loadResult();

		return version_compare($value, $version, '==');
	}

	/**
	 * Get the sql query bound to this object, new query if requested
	 *
	 * @param   bool  $new
	 *
	 * @return  JDatabaseQuerySqlite
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getQuery($new = false)
	{
		if ($new || !$this->query)
		{
			$this->query = $this->db->getQuery(true);

			$tableName = $this->reader->getName();
			$selects   = $this->reader->getSelects();
			$filters   = $this->reader->getFilters();
			$groupBy   = $this->reader->getGroupBy();
			$sortBy    = $this->reader->getSortOrders();
			$columns   = SQLiteQueryParser::select($selects);
			$filter    = SQLiteQueryParser::filter($filters);
			$ordering  = SQLiteQueryParser::ordering($sortBy);

			$this->query->select($columns ?: '*');
			$this->query->from($tableName);

			if ($filter)
			{
				$this->query->where($filter);
			}

			if ($groupBy)
			{
				$this->query->group($groupBy);
			}

			if ($ordering)
			{
				$this->query->order($ordering);
			}
		}

		return $this->query;
	}

	/**
	 * Method to load a single record found with matching filters
	 *
	 * @return  stdClass
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItem()
	{
		$query = clone $this->getQuery();

		$this->db->setQuery($query, 0, 1);

		return $this->db->loadObject();
	}

	/**
	 * Method to load a list of records found with matching filters
	 *
	 * @param   int  $offset  The called method
	 * @param   int  $limit   The array of arguments passed to the method
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getItems($offset = 0, $limit = 0)
	{
		$query = clone $this->getQuery();

		$this->db->setQuery($query, $offset, $limit);

		return $this->db->loadObjectList();
	}

	/**
	 * Method to load an iterator for records found with matching filters
	 *
	 * @return  JDatabaseIterator
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getIterator()
	{
		$query = clone $this->getQuery();

		$this->db->setQuery($query);

		return $this->db->getIterator();
	}

	/**
	 * Method to find number of matching records
	 *
	 * @return  int
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function getTotal()
	{
		$query = clone $this->getQuery();

		$query->clear('select')->select('SUM(1) OVER ()');

		return $this->db->setQuery($query)->loadResult();
	}

	/**
	 * Open a connection to the sqlite database file if it exists or create one if it doesn't exist
	 *
	 * @param   string  $path  Absolute path the the database file
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function openSqliteDatabase($path)
	{
		if (!file_exists($path))
		{
			throw new CacheMissingException(JText::sprintf('COM_SELLACIOUS_ERROR_CACHE_MISSING_DATABASE', $this->reader->getName()));
		}

		$db = SQLiteDatabaseDriver::getInstance(array('database' => $path));

		$db->setOption(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$db->setOption(PDO::ATTR_CASE, PDO::CASE_LOWER);

		$db->addFunction('json_val', array(SQLiteFunctions::class, 'jsonField'));
		$db->addFunction('json_in_array', array(SQLiteFunctions::class, 'jsonInArray'));
		$db->addFunction('json_in_key', array(SQLiteFunctions::class, 'jsonInKey'));
		$db->addFunction('json_intersect_array', array(SQLiteFunctions::class, 'jsonIntersectArray'));
		$db->addFunction('json_intersect_key', array(SQLiteFunctions::class, 'jsonIntersectKey'));

		$this->db = $db;
	}
}
