<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Database\Query;

use Sellacious\Database\Table\Field\SelectField;
use Sellacious\Database\Table\Filter\AbstractFilter;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;
use Sellacious\Database\Table\Sort\TableSort;

defined('_JEXEC') or die;

trait QueryBuilderTrait
{
	/**
	 * Flag to indicate whether some filter is known to cause zero results.
	 * This is to improve performance by skipping query.
	 *
	 * @var  bool
	 *
	 * @since   2.0.0
	 */
	protected $fallacy = false;

	/**
	 * Filters for the query loader
	 *
	 * @var  SelectField[]
	 *
	 * @since   2.0.0
	 */
	protected $selects = array();

	/**
	 * Filters for the query loader
	 *
	 * @var  FilterSet
	 *
	 * @since   2.0.0
	 */
	protected $filters;

	/**
	 * Sort orders for the query loader
	 *
	 * @var  TableSort[]
	 *
	 * @since   2.0.0
	 */
	protected $sortOrders = array();

	/**
	 * Group by for the query loader
	 *
	 * @var  string[]
	 *
	 * @since   2.0.0
	 */
	protected $groupBy = array();

	/**
	 * Append a filter to the filters instance
	 *
	 * @param   SelectField  $field
	 * @param   bool         $clear
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function addSelect(SelectField $field, $clear = false)
	{
		if ($clear)
		{
			$this->selects = array();
		}

		$this->selects[] = $field;
	}

	/**
	 * Append a filter to the filters instance
	 *
	 * @param   AbstractFilter  $filter
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function addFilter(AbstractFilter $filter)
	{
		$this->filters->addElement($filter);
	}

	/**
	 * Append a group to the query instance
	 *
	 * @param   string  $col
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function addGroupBy($col)
	{
		$this->groupBy[] = $col;
	}

	/**
	 * Append a sort order field to the instance
	 *
	 * @param   TableSort $sort
	 * @param   bool      $clear
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function addSort(TableSort $sort, $clear = false)
	{
		if ($clear)
		{
			$this->sortOrders = array();
		}

		$this->sortOrders[] = $sort;
	}

	/**
	 * Set fallacy flag ON, this disabled all filters and forces any query to have no match
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function fallacy()
	{
		$this->fallacy = true;
	}

	/**
	 * Filter on given value using said operator for comparison
	 *
	 * @param   string        $col    Database column name
	 * @param   string|mixed  $value  Value to match
	 * @param   string        $op     Comparison operator
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterValue($col, $value, $op = '=')
	{
		if ($op === 'IN' || is_array($value))
		{
			$this->addFilter(new FilterElement('in', $col, (array) $value));
		}
		elseif ($op === 'LIKE')
		{
			$this->addFilter(new FilterElement('str_contains', $col, $value));
		}
		else
		{
			$this->addFilter(new FilterElement(strtolower($op), $col, $value));
		}
	}

	/**
	 * Filter on given value in a json column using said operator for comparison
	 *
	 * @param   string  $col    Database column name
	 * @param   string  $key    JSON field to lookup for
	 * @param   string  $value  Value to match
	 * @param   string  $op     Comparison operator
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterJsonValue($col, $key, $value, $op = '=')
	{
		if ($op === 'IN' || is_array($value))
		{
			$this->addFilter(new FilterElement('in', $this->getKey($col, $key), (array) $value));
		}
		elseif ($op === 'LIKE')
		{
			$this->addFilter(new FilterElement('str_contains', $this->getKey($col, $key), $value));
		}
		else
		{
			$this->addFilter(new FilterElement(strtolower($op), $this->getKey($col, $key), $value));
		}
	}

	/**
	 * Filter matching the given value is a value or a key (as indicated with {$matchKey}) in a json array
	 *
	 * @param   string  $col    Database column name
	 * @param   string  $key    JSON field to lookup for
	 * @param   string  $value  Value to match
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterInJsonArray($col, $key, $value)
	{
		$this->addFilter(new FilterElement('set_contains', $this->getKey($col, $key), $value));
	}

	/**
	 * Filter matching the given values is a value or a key (as indicated with {$matchKey}) in a json array
	 *
	 * @param   string    $col     Database column name
	 * @param   string    $key     JSON field to lookup for
	 * @param   string[]  $values  Value to match
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterIntersectJsonArray($col, $key, $values)
	{
		$count = count($values);

		if ($count == 0)
		{
			$this->fallacy();
		}
		elseif ($count == 1)
		{
			$this->addFilter(new FilterElement('set_contains', $this->getKey($col, $key), reset($values)));
		}
		else
		{
			$or = new FilterSet('or');

			foreach ($values as $value)
			{
				$or->addElement(new FilterElement('set_contains', $this->getKey($col, $key), $value));
			}

			$this->addFilter($or);
		}
	}

	/**
	 * Filter matching the given values is a value or a key (as indicated with {$matchKey}) in a json array
	 *
	 * @param   string    $col     Database column name
	 * @param   string    $key     JSON field to lookup for
	 * @param   string[]  $values  Value to match
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterIntersectJsonKey($col, $key, $values)
	{
		$count = count($values);

		if ($count === 0)
		{
			$this->fallacy();
		}
		elseif ($count === 1)
		{
			$this->addFilter(new FilterElement('prop_exists', $this->getKey($col, $key), reset($values)));
		}
		else
		{
			$or = new FilterSet('or');

			foreach ($values as $value)
			{
				$or->addElement(new FilterElement('prop_exists', $this->getKey($col, $key), $value));
			}

			$this->addFilter($or);
		}
	}

	/**
	 * Filter matching the given value is a value or a key (as indicated with {$matchKey}) in a json array
	 *
	 * @param   string  $col    Database column name
	 * @param   string  $key    JSON field to lookup for
	 * @param   string  $value  Value to match
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	public function filterInJsonKey($col, $key, $value)
	{
		$this->addFilter(new FilterElement('prop_exists', $this->getKey($col, $key), $value));
	}

	/**
	 * Get the column index to lookup for
	 *
	 * @param   $col
	 * @param   $key
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected function getKey($col, $key)
	{
		return $key ? $col . '.' . $key : $col;
	}

	/**
	 * Get the list of select columns
	 *
	 * @return  SelectField[]
	 *
	 * @since   2.0.0
	 */
	public function getSelects()
	{
		return $this->selects;
	}

	/**
	 * Get list of filters
	 *
	 * @return  FilterSet
	 *
	 * @since   2.0.0
	 */
	public function getFilters()
	{
		return $this->filters;
	}

	/**
	 * Get list of sorting options
	 *
	 * @return  TableSort[]
	 *
	 * @since   2.0.0
	 */
	public function getSortOrders()
	{
		return $this->sortOrders;
	}

	/**
	 * Get group by options
	 *
	 * @return  string[]
	 *
	 * @since   2.0.0
	 */
	public function getGroupBy()
	{
		return $this->groupBy;
	}
}
