<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\Database\SQLite;

use Exception;
use Joomla\CMS\Factory;
use Sellacious\Database\Table\Field\SelectField;
use Sellacious\Database\Table\Filter\AbstractFilter;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;
use Sellacious\Database\Table\Sort\TableSort;

defined('_JEXEC') or die;

/**
 * @package  Sellacious\Database\SQLite
 *
 * @since    2.0.0
 */
class SQLiteQueryParser
{
	/**
	 * Parse the filter elements into where condition string
	 *
	 * @param   AbstractFilter  $filter  The filter element
	 *
	 * @return  string
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function filter(AbstractFilter $filter)
	{
		if ($filter instanceof FilterSet)
		{
			switch ($operator = $filter->getType())
			{
				case 'and':
				case 'or':
					return static::conjunct($filter->getElements(), $operator);
				case 'not':
					return static::not($filter->getElements());
			}
		}
		elseif ($filter instanceof FilterElement)
		{
			switch ($operator = $filter->getType())
			{
				case '=':
				case '!=':
				case '<':
				case '>':
				case '<=':
				case '>=':
					return static::compare($filter->getCol(), $filter->getValues(), $operator);

				case 'in':
					return static::in($filter->getCol(), $filter->getValues());

				case 'except':
					return static::except($filter->getCol(), $filter->getValues());

				case 'between':
					return static::between($filter->getCol(), $filter->getValues());

				case 'str_contains':
					return static::strContains($filter->getCol(), $filter->getValues());

				case 'set_contains':
					return static::setContains($filter->getCol(), $filter->getValues());

				default:
			}
		}

		return null;
	}

	/**
	 * Parse the select elements
	 *
	 * @param   SelectField[]  $selects  The select elements
	 *
	 * @return  string[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public static function select(array $selects)
	{
		$cols = array();

		foreach ($selects as $select)
		{
			$col = $select->col;

			if ($select->function)
			{
				$col = sprintf('%s(%s)', $select->function, $col);
			}

			if ($select->alias)
			{
				$col = sprintf('%s AS %s', $col, $select->alias);
			}

			$cols[] = $col;
		}

		return $cols;
	}

	/**
	 * Generate ordering clause
	 *
	 * @param   TableSort[]  $sortBy
	 *
	 * @return  string[]
	 *
	 * @since   2.0.0
	 */
	public static function ordering(array $sortBy)
	{
		$orders = array();

		foreach ($sortBy as $sort)
		{
			$orders[] = $sort->getCol() . ($sort->getDir() === 1 ? ' ASC' : ' DESC');
		}

		return $orders;
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 * @param   string           $operator
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function compare($column, $values, $operator)
	{
		$db = Factory::getDbo();

		return $column . ' ' . $operator . ' ' . $db->q($values);
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function in($column, $values)
	{
		$db = Factory::getDbo();

		return $column . ' in (' . implode(',', $db->q((array) $values)) . ')';
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function except($column, $values)
	{
		$db = Factory::getDbo();

		return $column . ' not in (' . implode(',', $db->q((array) $values)) . ')';
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function between($column, $values)
	{
		$db = Factory::getDbo();

		$values = (array) $values;

		if (count($values) !== 2)
		{
			return null;
		}

		list($min, $max) = $values;

		return '(' . $column . ' >= ' . $db->q($min) . ' and ' . $column . ' <= ' . $db->q($max) . ')';
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function strContains($column, $values)
	{
		$db = Factory::getDbo();

		if (!is_array($values))
		{
			return $column . ' like ' . $db->q('%' . $db->escape($values, true) . '%', false);
		}
		elseif (count($values) === 1)
		{
			return $column . ' like ' . $db->q('%' . $db->escape(reset($values), true) . '%', false);
		}
		else
		{
			$and = array();

			foreach ($values as $value)
			{
				$and[] = $column . ' like ' . $db->q('%' . $db->escape($value, true) . '%', false);
			}

			return '(' . implode(' and ', $and) . ')';
		}
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   string           $column
	 * @param   string|string[]  $values
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	protected static function setContains($column, $values)
	{
		$db = Factory::getDbo();

		if (strpos($column, '.'))
		{
			list($col, $key) = explode('.', $column, 2);
		}
		else
		{
			$col = $column;
			$key = 'null';
		}

		return sprintf('json_in_array(%s, %s, %s)', $col, $key, $db->q($values));
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   AbstractFilter[]  $elements
	 * @param   string            $operator
	 *
	 * @return  string
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function conjunct(array $elements, $operator)
	{
		$conditions = array_map(array(static::class, 'filter'), $elements);
		$pieces     = array_filter($conditions, 'strlen');

		if (count($pieces) === 0)
		{
			return null;
		}

		if (count($pieces) === 1)
		{
			reset($pieces);
		}

		return '(' . implode(' ' . $operator . ' ', $pieces) . ')';
	}

	/**
	 * Generate filter condition fragment
	 *
	 * @param   AbstractFilter[]  $elements
	 *
	 * @return  string
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected static function not(array $elements)
	{
		if (count($elements) !== 1)
		{
			return null;
		}

		$cond = static::filter(reset($elements));

		if ($cond === null)
		{
			return null;
		}

		return 'not ' . $cond;
	}
}
