<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
namespace Sellacious\Search\Adapter;

defined('_JEXEC') or die;

use Exception;
use JDatabaseIterator;
use JFactory;
use JRoute;
use LimitIterator;
use Sellacious\Cache\Reader\ProductsCacheReader;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Sort\TableSort;
use Sellacious\Search\SearchHelper;
use stdClass;

/**
 * Adapter class for products search
 *
 * @since   1.7.0
 */
class ProductsSearchAdapter extends AbstractSearchAdapter
{
	/**
	 * Generate search index for TNT Search API based searches
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   1.7.0
	 */
	public function buildIndex()
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select('a.id, a.title, a.local_sku, a.metakey, a.tags')
		      ->from($db->qn('#__sellacious_products', 'a'))
		      ->where('a.state = 1');

		SearchHelper::createIndex($this->getName(), (string) $query, 'id');
	}

	/**
	 * Perform a search for the relevant entity
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   1.7.0
	 */
	public function load()
	{
		$results = array();

		if ($this->start > 0 || $this->limit > 0)
		{
			$iterator = new LimitIterator($this->getIterator(), $this->start, $this->limit);
		}
		else
		{
			$iterator = $this->getIterator();
		}

		foreach ($iterator as $p)
		{
			$sCurrency  = $p->seller_currency;
			$dPrice     = $this->helper->currency->display($p->sales_price, $sCurrency, '');
			$categories = json_decode($p->category_ext, true);

			// Exclude ROOT category
			unset($categories[1]);

			$language = JFactory::getLanguage()->getTag();

			// Translate categories
			if ($categories)
			{
				foreach ($categories as $id => &$category)
				{
					$this->helper->translation->translateValue($id, 'sellacious_categories', 'title', $category, $language);
				}
			}

			$item = new stdClass;

			$item->type       = 'product';
			$item->value      = $p->product_title;
			$item->price      = $dPrice;
			$item->code       = $p->code;
			$item->image      = $this->helper->product->getImage($p->product_id);
			$item->link       = JRoute::_('index.php?option=com_sellacious&view=product&p=' . $item->code, false);
			$item->categories = implode(', ', $categories);

			$results[] = $item;
		}

		return $results;
	}

	/**
	 * Execute the search query to get an iterator instance
	 *
	 * @return  JDatabaseIterator
	 *
	 * @throws  Exception
	 *
	 * @since   1.7.0
	 */
	protected function execute()
	{
		$cache = new ProductsCacheReader;

		$cache->filterValue('is_selling', 1);
		$cache->filterValue('listing_active', 1);
		$cache->filterValue('product_active', 1);
		$cache->filterValue('seller_active', 1);

		$catId  = (int) $this->filters->get('category');
		$catIdP = (int) $this->filters->get('parent_category');
		$sid    = (int) $this->filters->get('seller');

		if ($this->keys)
		{
			$cache->filterValue('product_id', array_map('intval', $this->keys), 'IN');
			$cache->filterValue('variant_id', 0);
		}
		else
		{
			$cache->fallacy();
		}

		if ($catId)
		{
			$cache->addFilter(new FilterElement('set_contains', 'category_id_ext', $catId));
		}

		if ($catIdP)
		{
			$cache->addFilter(new FilterElement('set_contains', 'category_id_ext', $catIdP));
		}

		if ($sid)
		{
			$cache->filterValue('seller_uid', $sid);
		}

		if ($this->helper->config->get('hide_out_of_stock'))
		{
			$cache->filterValue('stock_capacity', 0, '>');
		}

		if ($this->helper->config->get('hide_zero_priced'))
		{
			// $cache->getQuery()->where('(sales_price > 0 OR price_display > 0)');
		}

		$cache->addFilter(new FilterElement('>', 'is_visible', 0));

		// The no-stock items are always at end
		$cache->addSort(new TableSort('no_stock', TableSort::SORT_ASC));
		$cache->addSort(new TableSort('is_visible', TableSort::SORT_DESC));

		return $cache->getIterator();
	}
}
