<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
namespace Sellacious\StoreTimings;

// no direct access
defined('_JEXEC') or die;

use DateTime;
use DateTimeZone;
use Exception;

/**
 * Class Slot
 *
 * @package Sellacious\StoreTimings
 *
 * @since   2.0.0
 */
class Slot
{
	/**
	 * The start time date-time object
	 *
	 * @var  DateTime
	 *
	 * @since   2.0.0
	 */
	protected $sdate;

	/**
	 * The end time date-time object
	 *
	 * @var  DateTime
	 *
	 * @since   2.0.0
	 */
	protected $edate;

	/**
	 * The timezone object for current timezone
	 *
	 * @var  DateTimeZone
	 *
	 * @since   2.0.0
	 */
	protected $tzo;

	/**
	 * The slot day name
	 *
	 * @var  string
	 *
	 * @since   2.0.0
	 */
	protected $day;

	/**
	 * The slot start time as HH:MM:SS
	 *
	 * @var  string
	 *
	 * @since   2.0.0
	 */
	protected $start;

	/**
	 * The slot end time as HH:MM:SS
	 *
	 * @var  string
	 *
	 * @since   2.0.0
	 */
	protected $end;

	/**
	 * Slot constructor
	 *
	 * @param   string  $day
	 * @param   string  $start
	 * @param   string  $end
	 * @param   string  $tz
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct($day, $start, $end, $tz = 'UTC')
	{
		$tzo  = new DateTimeZone($tz);
		$day  = ucfirst(strtolower($day));
		$days = array('Mon' => 2, 'Tue' => 3, 'Wed' => 4, 'Thu' => 5, 'Fri' => 6, 'Sat' => 7, 'Sun' => 8);

		$this->day   = $day;
		$this->start = $start;
		$this->end   = $end;
		$this->tzo   = $tzo;

		// Note: This does not account for DST for America timings
		$this->sdate = new DateTime(sprintf('1987-03-%d %s', $days[$this->day], $this->start), $tzo);
		$this->edate = new DateTime(sprintf('1987-03-%d %s', $days[$this->day], $this->end), $tzo);
	}

	/**
	 * Convert the slot to another timezone taking into account the day change
	 *
	 * @param   string  $tz
	 *
	 * @return  Slot[]
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function convert($tz)
	{
		$slots = array();
		$tzn   = new DateTimeZone($tz);
		$sdate = clone $this->sdate;
		$edate = clone $this->edate;

		$sdate->setTimezone($tzn);
		$edate->setTimezone($tzn);

		if ($sdate->format('d') < $edate->format('d'))
		{
			$slots[] = new Slot($sdate->format('D'), $sdate->format('H:i:s'), '23:59:59', $tz);
			$slots[] = new Slot($edate->format('D'), '00:00:00', $edate->format('H:i:s'), $tz);
		}
		else
		{
			$slots[] = new Slot($sdate->format('D'), $sdate->format('H:i:s'), $edate->format('H:i:s'), $tz);
		}

		return $slots;
	}

	/**
	 * Return an array representation of the slot
	 *
	 * @return  array
	 *
	 * @since   2.0.0
	 */
	public function toArray()
	{
		return array('day' => $this->day, 'start' => $this->start, 'end' => $this->end, 'tz' => $this->tzo->getName());
	}

	/**
	 * Timezone
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getTz()
	{
		return $this->tzo->getName();
	}

	/**
	 * Day
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getDay()
	{
		return $this->day;
	}

	/**
	 * Start time
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getStart()
	{
		return $this->start;
	}

	/**
	 * End time
	 *
	 * @return  string
	 *
	 * @since   2.0.0
	 */
	public function getEnd()
	{
		return $this->end;
	}
}
