<?php
/**
 * @version     2.0.0
 * @package     Sellacious.Cli
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */

use Joomla\Utilities\ArrayHelper;
use Sellacious\Cache\Builder\ProductsCacheBuilder;
use Sellacious\Import\AbstractImporter;
use Sellacious\Import\ImportHelper;
use Sellacious\Media\MediaCleanup;
use Sellacious\Utilities\Timer;

/**
 * Sellacious Importer CLI.
 *
 * This is a command-line script to help with sellacious database import, such as from CSV etc.
 */
define('_JEXEC', 1);

define('JPATH_BASE', dirname(__DIR__));

require_once __DIR__ . '/cli.php';
require_once __DIR__ . '/application.php';

ini_set('memory_limit', '5G');

/**
 * A command line cron job to run the Sellacious importer.
 *
 * @since   1.6.1
 */
class SellaciousImporterCli extends SellaciousCliApplication
{
	/**
	 * The timer/logger instance
	 *
	 * @var   Timer
	 *
	 * @since   1.6.1
	 */
	protected $timer;

	/**
	 * Entry point for Smart Search CLI script
	 *
	 * @return  void
	 *
	 * @since   1.6.1
	 */
	public function doExecute()
	{
		ob_start();

		// Print a blank line.
		$this->out();
		$this->out(JText::_('Sellacious Importer Cli'));
		$this->out('============================');

		// Remove the script time limit.
		@set_time_limit(0);

		try
		{
			file_put_contents(__DIR__ . '/.import-lock', getmypid());

			$this->loadLanguage();
			JLoader::import('sellacious_importer.loader');
			JPluginHelper::importPlugin('system', 'sellaciousimporter');

			$this->out('Import starting...');

			// Do the business stuff
			$this->doImport();
		}
		catch (Exception $e)
		{
			$this->out('========= ERROR ===================');

			$this->out($e->getMessage());
			$this->out($e->getFile() . ' : ' . $e->getLine());
			$this->out($e->getTraceAsString());

			$this->out('========= ERROR ==================');
		}

		@unlink(__DIR__ . '/.import-lock');

		$this->out('Import finished.');
		$this->out('EOF');

		$this->out();
	}

	/**
	 * Start the queued import process
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   1.6.1
	 */
	protected function doImport()
	{
		$userId   = $this->input->getInt('user');
		$reset    = $this->input->getBool('reset');
		$importId = $this->input->getInt('job');
		$import   = ImportHelper::getImport($importId);

		if (!$import->id)
		{
			return;
		}

		// Force importer log file
		$this->timer = Timer::getInstance('Import.' . $import->handler, $import->log_path);

		/** @var  AbstractImporter  $importer */
		$importer = ImportHelper::getImporter($import->handler);

		$importer->timer->log('Initializing import...');

		// Remove current progress if reset is requested, will be saved only if executed
		if ($reset)
		{
			$import->progress = null;

			@unlink($import->log_path);
			@unlink($import->output_path);
		}

		$importer->setup($import);

		// Override active user if set
		if ($userId)
		{
			$importer->setOption('session.user', $userId);
		}

		$importer->timer->log('Starting import process...');

		$import->setState(2);

		$importer->import();

		$import->setState(3);

		$pks = $importer->getIndex('Product');

		if ($pks)
		{
			$this->rebuildCache(array_values($pks));
		}
	}

	/**
	 * Start the cache process
	 *
	 * @param   int[]  An array of imported (created/updated) product ids
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   1.6.1
	 */
	protected function rebuildCache($pks)
	{
		$this->out('Initializing cache rebuild. This may take very long to update. Please be patient...');

		if (!$pks)
		{
			return;
		}

		try
		{
			$n      = 0;
			$size   = 50;
			$c      = count($pks);
			$pks    = ArrayHelper::getColumn($pks, 0);
			$chunks = array_chunk($pks, $size, false);

			foreach ($chunks as $i => $chunk)
			{
				$cache = new ProductsCacheBuilder;

				$cache->rebuild($chunk);

				$this->timer->log(sprintf('Rebuilding cache: %d of %d products processed.', $n + count($chunk), $c), true);
			}

			$this->timer->log(sprintf('Rebuilding cache: %d of %d', $c, $c), true);

			// Sync media as well
			$this->out('Discovering media files from filesystem.');

			$cleanup = new MediaCleanup;

			$cleanup->execute();

			$this->out('Discovery of media files from filesystem finished.');

			$jCache = JCache::getInstance('output', array('caching' => true, 'cachebase' => JPATH_SITE . '/cache'));

			if (@$jCache->clean('product_cache', 'group'))
			{
				$this->out('Product output cache deleted.');
			}
		}
		catch (Exception $e)
		{
			$this->out($e->getMessage());
			$this->out($e->getTraceAsString());
		}
	}

	protected function loadLanguage()
	{
		$language = JFactory::getLanguage();
		$current  = $language->getTag();

		$language->load('lib_importer', JPATH_LIBRARIES . '/sellacious_importer');
		$language->load('lib_importer', JPATH_BASE);

		$language->load('com_importer', JPATH_BASE . '/components/com_importer', 'en-GB');
		$language->load('com_importer', JPATH_BASE, 'en-GB');
		$language->load('com_importer', JPATH_BASE . '/components/com_importer', $current);
		$language->load('com_importer', JPATH_BASE, $current);
	}
}

$application = JApplicationCli::getInstance('SellaciousImporterCli');

// Set application instance to not cause loading default application (site) in Cli
JFactory::$application = $application;

$application->execute();
