<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Aditya Chakraborty <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
defined('_JEXEC') or die;

/**
 * Methods supporting a list of categories with commissions
 *
 * @since  2.0.0
 */
class SellaciousModelCommissions extends SellaciousModelList
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see    JController
	 * @since  2.0.0
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'title', 'a.title',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * @param   string  $ordering   An optional ordering field.
	 * @param   string  $direction  An optional direction (asc|desc).
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		parent::populateState($ordering, $direction);

		$id = $this->app->input->getInt('id', 0);
		$this->state->set('category_id', $id);

		$currency = $this->app->input->getString('currency');
		$this->state->set('currency', $currency);
	}

	/**
	 * Method to get basic category commissions query
	 *
	 * @param   int     $id       Category id for which commission are being fetched
	 * @param   string  $context  Category context
	 * @param   string  $type     Additional context/type
	 *
	 * @return  JDatabaseQuery
	 *
	 * @since   2.0.0
	 */
	protected function getBasicQuery($id, $context, $type)
	{
		// Create a new query object.
		$db      = $this->getDbo();
		$query   = $db->getQuery(true);

		$query->select('a.id, a.title, a.level, b.commission')
			->from($db->qn('#__sellacious_categories') . ' AS a');

		if ($type == 'com_sellacious.user' && $context == 'product')
		{
			$query->join('left', $db->qn('#__sellacious_seller_commissions', 'b') . ' ON b.' . $context . '_catid = a.id AND b.seller_uid = ' . (int) $id);
			$query->where('(b.seller_uid IS NULL OR b.seller_uid = ' . (int) $id . ')');
		}
		else
		{
			$commContext = $context == 'seller' ? 'product' : 'seller';

			$query->join('left', $db->qn('#__sellacious_category_commissions', 'b') . ' ON b.' . $context . '_catid = a.id AND b.' . $commContext . '_catid = ' . (int) $id);
			$query->where('(b.' . $commContext . '_catid IS NULL OR b.' . $commContext . '_catid = ' . (int) $id . ')');
		}

		if ($context == 'product')
		{
			$query->where('a.type LIKE ' . $db->q('product/%', false));
		}
		elseif ($context == 'seller')
		{
			$query->where('a.type = ' . $db->q('seller'));
		}
		else
		{
			// Fallacy
			return $query->where('0 = 1');
		}

		$query->where('a.state = 1');

		return $query;
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 *
	 * @since   2.0.0
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db      = $this->getDbo();
		$id      = $this->state->get('category_id');
		$context = $this->state->get('filter.context');
		$type    = $this->state->get('filter.form_type');
		$query   = $this->getBasicQuery($id, $context, $type);

		// Add the level in the tree.
		$query->select('MAX(c2.level) AS max_gc_level')
			->join('LEFT OUTER', $db->qn('#__sellacious_categories') . ' AS c2 ON a.lft < c2.lft AND a.rgt > c2.rgt AND a.level = 2');

		// Filter the comments over the search string if set.
		$search = $this->getState('filter.search');

		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('a.id = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->q('%' . $db->escape($search, true) . '%');
				$query->where('a.title LIKE ' . $search);
			}
		}

		// Only show till 1st child
		$query->where('a.level < 3');

		$query->group('a.id, a.lft, a.rgt');

		$ordering = $this->state->get('list.fullordering', 'a.lft ASC');
		$ordering = $ordering ?: 'a.lft ASC';

		if (trim($ordering))
		{
			$query->order($db->escape($ordering));
		}

		return $query;
	}

	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 *
	 * @since   2.0.0
	 */
	public function getItems()
	{
		$items   = parent::getItems();
		$context = $this->state->get('filter.context');
		$id      = $this->state->get('category_id');

		if ($context == 'seller')
		{
			$default               = new stdClass;
			$default->id           = 1;
			$default->title        = JText::_('JDEFAULT');
			$default->level        = 0;
			$default->max_gc_level = 0;

			$rates               = $this->helper->category->getSellerCommissionsBySellerCategory(1);
			$default->commission = isset($rates[$id]) ? $rates[$id] : 0.00;

			array_unshift($items, $default);
		}

		return $items;
	}

	/**
	 * Pre-process loaded list before returning if needed
	 *
	 * @param   stdClass[]  $items  The items loaded from the database using the list query
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public function processList($items)
	{
		foreach ($items as $item)
		{
			$this->helper->translation->translateRecord($item, 'sellacious_categories');
		}

		return parent::processList($items);
	}

	/**
	 * Method to get child categories commission for a category
	 *
	 * @param   int     $baseCatId   Base category id for which children are fetched
	 * @param   int     $catId       Category id for which commission are being fetched
	 * @param   string  $context     Category context
	 * @param   string  $type        Additional context/type
	 *
	 * @return  stdClass[]
	 *
	 * @since   2.0.0
	 */
	public function getChildCommissions($baseCatId, $catId, $context, $type)
	{
		if (!$baseCatId)
		{
			return array();
		}

		$category = $this->helper->category->getItem($baseCatId);

		// Create a new query object.
		$db    = $this->getDbo();
		$query = $this->getBasicQuery($catId, $context, $type);

		$query->where('a.lft > ' . (int) $category->lft);
		$query->where('a.rgt < ' . (int) $category->rgt);

		$query->order('a.lft ASC');

		$db->setQuery($query);

		$items = $db->loadObjectList();

		if ($items)
		{
			foreach ($items as $item)
			{
				$item->max_gc_level = 0;
			}
		}

		return $items;
	}
}
