<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     SPL Sellacious Private License; see http://www.sellacious.com/spl.html
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
defined('_JEXEC') or die;

use Joomla\String\StringHelper;
use Joomla\Utilities\ArrayHelper;

/**
 * Sellacious model.
 */
class SellaciousModelPaymentMethod extends SellaciousModelAdmin
{
	/**
	 * Method to test whether a record can be deleted.
	 *
	 * @param   object  $record  A record object.
	 *
	 * @return  boolean  True if allowed to delete the record. Defaults to the permission for the component.
	 *
	 * @since   12.2
	 */
	protected function canDelete($record)
	{
		$me      = JFactory::getUser();
		$allowed = $this->helper->access->check('paymentmethod.delete') ||
			($me->id == $record->seller_uid && $this->helper->access->check('paymentmethod.delete.own'));

		return $allowed;
	}

	/**
	 * Method to test whether a record can be deleted.
	 *
	 * @param   object  $record  A record object.
	 *
	 * @return  boolean  True if allowed to change the state of the record. Defaults to the permission for the component.
	 *
	 * @since   12.2
	 */
	protected function canEditState($record)
	{
		$me      = JFactory::getUser();
		$allowed = $this->helper->access->check('paymentmethod.edit.state') ||
			($me->id == $record->seller_uid && $this->helper->access->check('paymentmethod.edit.state.own'));
		
		return $allowed;
	}

	/**
	 * Method to save the form data.
	 *
	 * @param   array  $data  The form data.
	 *
	 * @return  boolean  True on success.
	 *
	 * @throws  Exception
	 *
	 * @since   1.2.0
	 */
	public function save($data)
	{
		// Initialise variables;
		$dispatcher = JEventDispatcher::getInstance();

		$table = $this->getTable();
		$pk    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState($this->getName() . '.id');
		$isNew = true;

		$translations = ArrayHelper::getValue($data, 'translations', array());

		// Include current language translation, if not already included
		$lang = JFactory::getLanguage()->getTag();

		if (!isset($translations[$lang]))
		{
			$translations[$lang] = array('title' => $data['title'], 'description' => $data['description']);
		}

		// Include the content plugins for the on save events.
		JPluginHelper::importPlugin('sellacious');

		// Load the row if saving an existing category.
		if ($pk > 0)
		{
			$table->load($pk);
			$isNew = false;

			$existing_translations = $this->helper->translation->getTranslations($pk, 'sellacious_paymentmethod');

			// Main table should always have English translations
			if (isset($existing_translations['en-GB']) && JLanguageMultilang::isEnabled() && $lang != 'en-GB')
			{
				$data = array_merge($data, $existing_translations['en-GB']);
			}
		}

		// Alter the title for save as copy
		if ($this->app->input->get('task') == 'save2copy')
		{
			$data['title'] = StringHelper::increment($data['title']);
		}

		if (!$this->helper->access->check('paymentmethod.edit'))
		{
			$data['seller_uid'] = JFactory::getUser()->id;
		}

		// For any seller payment method, Shopping Cart option will be default
		if ($data['seller_uid'] != 0)
		{
			$data['contexts'] = array('cart');
		}

		if (!$table->bind($data))
		{
			$this->setError($table->getError());

			return false;
		}

		// Check the data.
		if (!$table->check())
		{
			$this->setError($table->getError());

			return false;
		}

		// Trigger the onBeforeSave event.
		$result = $dispatcher->trigger($this->event_before_save, array($this->option . '.' . $this->name, &$table, $isNew));

		if (in_array(false, $result, true))
		{
			$this->setError($table->getError());

			return false;
		}

		// Store the data.
		if (!$table->store())
		{
			$this->setError($table->getError());

			return false;
		}

		try
		{
			$_control    = 'jform.logo';
			$_tableName  = 'paymentmethod';
			$_context    = 'logo';
			$_recordId   = $table->id;
			$_extensions = array('jpg', 'png', 'jpeg', 'gif');
			$_options    = ArrayHelper::getValue($data, 'logo', array(), 'array');

			$this->helper->media->handleUploader($_control, $_tableName, $_context, $_recordId, $_extensions, $_options);
		}
		catch (Exception $e)
		{
			$this->app->enqueueMessage($e->getMessage(), 'warning');
		}

		// Save Translations if any
		$this->helper->translation->saveTranslations($translations, $table->get('id'), 'sellacious_paymentmethod');

		// Trigger the onAfterSave event.
		$dispatcher->trigger($this->event_after_save, array($this->option . '.' . $this->name, &$table, $isNew));

		$this->setState($this->getName() . '.id', $table->get('id'));

		return true;
	}

	/**
	 * Method to allow derived classes to preprocess the data.
	 *
	 * @param   string  $context  The context identifier.
	 * @param   mixed   &$data    The data to be processed. It gets altered directly.
	 * @param   string  $group    The name of the plugin group to import (defaults to "content").
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 */
	protected function preprocessData($context, &$data, $group = 'content')
	{
		if (is_object($data))
		{
			// Load Translations to form
			if ($data->id)
			{
				$data->translations = $this->helper->translation->getTranslations($data->id, 'sellacious_paymentmethod');

				$this->helper->translation->translateRecord($data, 'sellacious_paymentmethod');
			}
		}

		parent::preprocessData($context, $data, $group);
	}

	/**
	 * Override preprocessForm to load the sellacious plugin group instead of content.
	 *
	 * @param   JForm   $form   A form object.
	 * @param   mixed   $data   The data expected for the form.
	 * @param   string  $group  Plugin Group
	 *
	 * @return  void
	 * @throws  Exception  If there is an error in the form event.
	 */
	protected function preprocessForm(JForm $form, $data, $group = 'sellacious')
	{
		$obj = is_array($data) ? ArrayHelper::toObject($data) : $data;

		$form->setFieldAttribute('logo', 'recordId', $obj->id);

		if (!$this->helper->access->isSubscribed())
		{
			$form->setFieldAttribute('use_credit', 'readonly', 'true', 'params');
			$form->setFieldAttribute('credit_days', 'readonly', 'true', 'params');
			$form->setFieldAttribute('credit_everyone', 'readonly', 'true', 'params');
			$form->setFieldAttribute('credit_default', 'readonly', 'true', 'params');
		}

		$canEdit    = $this->helper->access->check('paymentmethod.edit');
		$canEditOwn = $this->helper->access->check('paymentmethod.edit.own');

		if (!$canEdit && $canEditOwn)
		{
			$form->removeField('seller_uid');
			$form->removeField('contexts');
		}

		// When form is loaded or saved
		if (is_object($data))
		{
			if ($data->seller_uid !== NULL && $data->seller_uid != 0)
			{
				$form->removeField('contexts');
			}
		}
		// Remove Add Fund and Shopping Cart checkboxes if any seller is selected
		elseif (is_array($data) && isset($data['seller_uid']))
		{
			if ($data['seller_uid'] != 0)
			{
				$form->removeField('contexts');
			}
		}

		// Show Translation fields
		$this->setTranslationFields($form);

		parent::preprocessForm($form, $data, $group);
	}

	/**
	 * Method to translations fields to form
	 *
	 * @param   \JForm  $form  The form
	 *
	 * @return  void
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function setTranslationFields(&$form)
	{
		$defLanguage = JFactory::getLanguage();
		$tag         = $defLanguage->getTag();
		$languages   = JLanguageHelper::getContentLanguages();

		$languages = array_filter($languages, function ($item) use ($tag){
			return ($item->lang_code != $tag);
		});

		if (!empty($languages))
		{
			// Language Tabs
			$spacer = htmlentities('<div class="container">');

			foreach ($languages as $language)
			{
				$spacer .= htmlentities('<a class="btn btn-primary margin-right-5" href="#jform_translations_' . str_replace('-', '_', $language->lang_code) . '_language_title-lbl">' . '<img src="' . JUri::root() . 'media/mod_languages/images/'. $language->image . '.gif" alt="'. $language->image . '"> ') . $language->title . htmlentities('</a>');
			}

			$spacer .= htmlentities('</div>');

			$spacerElement = new SimpleXMLElement('
				<field type="spacer" name="language_tab" label="' . $spacer . '" />
			');

			$form->setField($spacerElement, 'translations', true, 'translations');

			// Language Translation fields
			foreach ($languages as $language)
			{
				$spacer = htmlentities('<b>') . $language->title . htmlentities('</b>');

				$element = new SimpleXMLElement('
				<fields name="' . $language->lang_code . '">
					<field type="spacer" name="language_title" label="' . $spacer . '" />
					<field
						name="title"
						type="text"
						label="COM_SELLACIOUS_PAYMENTMETHOD_FIELD_TITLE_LABEL"
						description="COM_SELLACIOUS_PAYMENTMETHOD_FIELD_TITLE_DESC"
						class="inputbox"
						size="40"
					/>
					<field
						name="description"
						type="textarea"
						label="COM_SELLACIOUS_PAYMENTMETHOD_FIELD_DESCRIPTION_LABEL"
						description="COM_SELLACIOUS_PAYMENTMETHOD_FIELD_DESCRIPTION_DESC"
						class="inputbox textarea"
						size="20"
					/>
				</fields>');

				$form->setField($element, 'translations', true, 'translations');
			}
		}
		else
		{
			$form->removeGroup('translations');
		}
	}
}
