<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\Utilities\ArrayHelper;
use Sellacious\Cache\Builder\Adapter\Product\ManufacturerAdapter;
use Sellacious\Cache\Builder\Adapter\Product\SellerAdapter;
use Sellacious\Cache\Builder\ProductsCacheBuilder;
use Sellacious\Cart;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;

JLoader::import('sellacious.loader');

if (class_exists('SellaciousHelper')):

/**
 * The Cache management plugin for products and prices
 *
 * @since   1.4.0
 */
class PlgSystemSellaciousCache extends SellaciousPlugin
{
	/**
	 * @var    bool
	 *
	 * @since  1.4.0
	 */
	protected $hasConfig = true;

	/**
	 * The cache updater instance
	 *
	 * @var   ProductsCacheBuilder
	 *
	 * @since  2.0.0
	 */
	protected $updater;

	/**
	 * Constructor
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An optional associative array of configuration settings
	 *
	 * @throws  Exception
	 *
	 * @since   1.5.0
	 */
	public function __construct(&$subject, $config = array())
	{
		parent::__construct($subject, $config);

		$this->updater = new ProductsCacheBuilder;
	}

	/**
	 * Method is called right after an item is saved
	 *
	 * @param   string  $context  The calling context
	 * @param   object  $table    A JTable object
	 * @param   bool    $isNew    If the content is just about to be created
	 *
	 * @return  bool
	 *
	 * @since   1.5.0
	 */
	public function onContentAfterSave($context, $table, $isNew)
	{
		try
		{
			if ($context == 'com_sellacious.product')
			{
				$this->updater->rebuild($table->id);
			}
			elseif ($context == 'com_sellacious.user' && !$isNew)
			{
				$batch  = array();
				$record = ManufacturerAdapter::getRecord($table->id);

				if ($record)
				{
					$filter  = new FilterElement('=', 'manufacturer_id', (int) $table->id);
					$batch[] = array('values' => (array) $record, 'conditions' => $filter);
				}

				$record = SellerAdapter::getRecord($table->id);

				if ($record)
				{
					$filter  = new FilterElement('=', 'seller_uid', (int) $table->id);
					$batch[] = array('values' => (array) $record, 'conditions' => $filter);
				}

				if ($batch)
				{
					$this->updater->update($batch);
				}
			}
			elseif (
				$context == 'com_sellacious.variant' ||
				$context == 'com_sellacious.product.price' ||
				$context == 'com_sellacious.product.psx'
			)
			{
				$this->updater->rebuild($table->product_id);
			}
			elseif ($context == 'com_sellacious.productlisting')
			{
				if ($table->product_ids)
				{
					$this->updater->rebuild($table->product_ids);
				}
			}
			elseif ($context == 'com_sellacious.rating' && $table->get('type') == 'product')
			{
				if ($this->helper->config->get('product_rating'))
				{
					$multiVariant    = $this->helper->config->get('multi_variant');
					$variantSeparate = $multiVariant == 2;

					if ($variantSeparate)
					{
						$filter = new FilterSet('and');

						$filter->addElement(new FilterElement('=', 'product_id', (int) $table->product_id));
						$filter->addElement(new FilterElement('=', 'variant_id', (int) $table->variant_id));
					}
					else
					{
						$filter = new FilterElement('=', 'product_id', (int) $table->product_id);
					}

					$rating = $this->helper->rating->getProductRating($table->product_id, $table->variant_id);
					$batch  = array(
						array('values' => array('product_rating' => json_encode($rating)), 'conditions' => $filter)
					);

					$this->updater->update($batch);
				}
			}
			elseif ($context == 'com_sellacious.rating' && $table->get('type') == 'seller')
			{
				if ($this->helper->config->get('seller_rating'))
				{
					$filter = new FilterElement('=', 'seller_id', (int) $table->seller_id);
					$rating = $this->helper->rating->getSellerRating($table->seller_id);
					$batch  = array(
						array('values' => array('seller_rating' => json_encode($rating)), 'conditions' => $filter)
					);

					$this->updater->update($batch);
				}
			}
			elseif ($context == 'com_sellacious.setup')
			{
				// Todo: cache rebuild on sellacious install/upgrade
			}
		}
		catch (Exception $e)
		{
			JLog::add($e->getMessage(), JLog::CRITICAL);
		}

		return true;
	}

	/**
	 * Method is called right after an item state is changed
	 *
	 * @param   string  $context  The calling context
	 * @param   int[]   $pks      Record ids which are affected
	 * @param   bool    $value    The new state value
	 *
	 * @return  bool
	 *
	 * @since   1.5.0
	 */
	public function onContentChangeState($context, $pks, $value)
	{
		if (count($pks) == 0)
		{
			return true;
		}

		/**
		 * If a product or a variant is updated, we update everything for the affected product/variant.
		 * Else, update the modified entities in all relevant product/variants
		 */
		try
		{
			$state = $value == 1 ? 1 : 0;
			$pks   = array_map('intval', $pks);

			if ($context == 'com_sellacious.product')
			{
				$this->updater->rebuild($pks);
			}
			elseif ($context == 'com_sellacious.variant')
			{
				$ids = $this->helper->variant->loadColumn(array('list.select' => 'a.product_id', 'id' => $pks));

				$this->updater->rebuild($ids);
			}
			elseif ($context == 'com_sellacious.product.selling')
			{
				$batch = array(
					array(
						'values'     => array('is_selling' => (int) $state),
						'conditions' => new FilterElement('in', 'psx_id', $pks),
					),
				);

				$this->updater->update($batch);
			}
			elseif ($context == 'com_sellacious.user')
			{
				$batch = array(
					array(
						'values'     => array('seller_active' => (int) $state),
						'conditions' => new FilterElement('in', 'seller_uid', $pks),
					),
				);

				$this->updater->update($batch);
			}
			else
			{
				// Todo: any other updates
			}
		}
		catch (Exception $e)
		{
			JLog::add($e->getMessage(), JLog::CRITICAL);
		}

		return true;
	}

	/**
	 * This method sends a registration email when a order payment finishes on both failure or success.
	 *
	 * @param   string  $context  The calling context
	 * @param   object  $payment  Holds the payment object from the payments table for the target order
	 *
	 * @return  bool
	 *
	 * @since   1.5.0
	 */
	public function onAfterOrderPayment($context, $payment)
	{
		if ($context == 'com_sellacious.order')
		{
			try
			{
				// Should we update here directly and specifically?
				$items = $this->helper->order->getOrderItems($payment->order_id, 'a.product_id');
				$pks   = ArrayHelper::getColumn($items, 'product_id');

				if ($pks)
				{
					$this->updater->rebuild($pks);
				}
			}
			catch (Exception $e)
			{
				JLog::add($e->getMessage(), JLog::WARNING);
			}
		}

		return true;
	}

	/**
	 * This method sends a registration email when a order payment finishes on both failure or success.
	 *
	 * @param   string  $context  The calling context
	 * @param   int     $orderId  The concerned order id
	 *
	 * @return  bool
	 *
	 * @since   1.5.0
	 */
	public function onAfterOrderChange($context, $orderId)
	{
		if ($context == 'com_sellacious.order')
		{
			try
			{
				// Should we update here directly and specifically?
				$items = $this->helper->order->getOrderItems($orderId, 'a.product_id');
				$pks   = ArrayHelper::getColumn($items, 'product_id');

				if ($pks)
				{
					$this->updater->rebuild($pks);
				}
			}
			catch (Exception $e)
			{
				JLog::add($e->getMessage(), JLog::WARNING);
			}
		}

		return true;
	}

	/**
	 * This method sends a registration email when a order placed.
	 *
	 * @param   string  $context
	 * @param   object  $order
	 * @param   array   $products
	 * @param   Cart    $cart
	 *
	 * @return  bool
	 *
	 * @since   1.5.0
	 */
	public function onAfterPlaceOrder($context, $order, $products, $cart)
	{
		if ($context == 'com_sellacious.order')
		{
			try
			{
				// Should we update here directly and specifically?
				$pks = ArrayHelper::getColumn($products, 'product_id');

				if ($pks)
				{
					$this->updater->rebuild($pks);
				}
			}
			catch (Exception $e)
			{
				JLog::add($e->getMessage(), JLog::WARNING);
			}
		}

		return true;
	}
}

endif;
