<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Aditya Chakraborty <info@bhartiy.com> - http://www.bhartiy.com
 */

// No direct access
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Sellacious\Access\AccessHelper;
use Sellacious\Cart;
use Sellacious\Form\CheckoutQuestionsFormHelper;
use Sellacious\Form\Handler\DeliveryHandler;
use Sellacious\Shipping\ShippingHandler;
use Sellacious\StoreTimings\SlotRate;
use Sellacious\StoreTimings\Slots;
use Sellacious\StoreTimings\StoreTimingsHelper;
use Sellacious\Template\AbstractTemplate;
use Sellacious\Template\TemplateVariable;

defined('_JEXEC') or die('Restricted access');

// Include dependencies
jimport('sellacious.loader');

/**
 * Seller Timings Plugin
 *
 * @since  2.0.0
 */
class plgSystemSellaciousdelivery extends SellaciousPluginShipment
{
	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  2.0.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * Whether this class has a configuration to inject into sellacious configurations
	 *
	 * @var    bool
	 *
	 * @since  2.0.0
	 */
	protected $hasConfig = true;

	/**
	 * Constructor
	 *
	 * @param   object &$subject  The object to observe
	 * @param   array   $config   An optional associative array of configuration settings.
	 *                            Recognized key values include 'name', 'group', 'params', 'language'
	 *                            (this list is not meant to be comprehensive).
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function __construct($subject, array $config)
	{
		parent::__construct($subject, $config);

		JLoader::registerNamespace('Sellacious', __DIR__ . '/libraries');

		JTable::addIncludePath(JPATH_SITE . '/components/com_sellacious/tables');
		JTable::addIncludePath(JPATH_SELLACIOUS . '/components/com_sellaciousdelivery/tables');
		JTable::addIncludePath(__DIR__ . '/tables');

		JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_sellacious/models/');

		JForm::addFieldPath(__DIR__ . '/forms/fields');
	}

	/**
	 * Method to add all handlers
	 *
	 * @since   2.0.0
	 */
	public function onAfterInitialise()
	{
		CheckoutQuestionsFormHelper::addHandler('delivery', DeliveryHandler::class);
	}

	/**
	 * Calls on form preparation
	 *
	 * @param   JForm  $form  The form to be altered.
	 * @param   array  $data  The associated data for the form.
	 *
	 * @return  boolean
	 *
	 * @since   2.0.0
	 */
	public function onContentPrepareForm($form, $data)
	{
		if ($form instanceof JForm)
		{
			$me   = JFactory::getUser();
			$name = $form->getName();
			$obj  = is_array($data) ? ArrayHelper::toObject($data) : $data;

			$timeSelection   = $this->helper->config->get('delivery_time_selection', 0);
			$onlyOwnShipRule = !AccessHelper::allow('shippingrule.edit', 'com_sellacious')
			                   && AccessHelper::allow('shippingrule.edit.own', 'com_sellacious');

			if ($name != 'com_sellacious.shippingrule')
			{
				parent::onContentPrepareForm($form, $data);
			}

			// Include config
			if (($name == 'com_sellacious.user' || $name == 'com_sellacious.profile') && isset($obj->seller) && $obj->seller->category_id)
			{
				JForm::addFieldPath(JPATH_SITE . '/sellacious/components/com_sellacious/models/fields');

				$deliveryTimeSelection = $this->helper->config->get('delivery_time_selection', 0);
				$scope                 = $this->helper->config->get('delivery_time_scope', 'product');

				if (empty($obj->seller->category_id))
				{
					$sParams = new Registry;
				}
				else
				{
					$filter  = array('list.select' => 'a.params', 'id' => $obj->seller->category_id);
					$sParams = $this->helper->category->loadResult($filter);
				}

				$sParams = new Registry($sParams);
				$sParams = $sParams->extract('profile_fields');

				if ($this->app->isClient('site'))
				{
					$visible = $sParams ? $sParams->get('profile') : null;
				}
				else
				{
					$visible = $sParams ? $sParams->get('admin') : null;
				}

				$visible = new JObject($visible);

				if ($visible->get('seller.delivery_timings'))
				{
					$formPath = $this->pluginPath . '/forms/seller/delivery.xml';
					$form->loadFile($formPath, false);
				}

				if ($visible->get('seller.pickup_timings'))
				{
					$formPath = $this->pluginPath . '/forms/seller/pickup.xml';
					$form->loadFile($formPath, false);
				}

				// Front end seller profile page
				if ($this->app->isClient('site'))
				{
					JHtml::_('stylesheet', 'plg_system_sellaciousdelivery/layout.profile.css', array('version' => S_VERSION_CORE, 'relative' => true));
				}
				else
				{
					$form->setFieldAttribute('show_delivery_availability', 'type', 'radioswitch', 'seller.params');
					$form->setFieldAttribute('show_pickup_availability', 'type', 'radioswitch', 'seller.params');
					$form->setFieldAttribute('show_delivery_timings', 'type', 'radioswitch', 'seller.params');
					$form->setFieldAttribute('show_pickup_timings', 'type', 'radioswitch', 'seller.params');
				}

				switch ($deliveryTimeSelection)
				{
					case 1:
						$form->removeField('slot_window', 'seller.params');
						$form->removeField('slot_pad', 'seller.params');
						$form->removeField('delivery_prep_time', 'seller.params');
						break;
					case 2:
						if ($visible->get('seller.pickup_timings') && $scope == 'shipping')
						{
							$form->loadFile($this->pluginPath . '/forms/seller/pickup_slots.xml', false);
						}
						break;
					case 3:
						$form->removeField('slot_window', 'seller.params');
						$form->removeField('slot_limit', 'seller.params');
						$form->removeField('slot_pad', 'seller.params');
						break;
					case 0:
					default:
						$form->removeField('slot_window', 'seller.params');
						$form->removeField('slot_pad', 'seller.params');
						$form->removeField('today_availability', 'seller.params');
						$form->removeField('slot_limit', 'seller.params');
						$form->removeField('delivery_prep_time', 'seller.params');
						$form->removeField('delivery_slot_settings', 'seller.params');
						break;
				}

				$enable_delivery_availability = $this->helper->config->get('enable_delivery_availability', 1);

				if (!$enable_delivery_availability)
				{
					$form->removeField('show_delivery_availability', 'seller.params');
				}

				$enable_delivery_timings = $this->helper->config->get('enable_delivery_timings', 1);

				if (!$enable_delivery_timings)
				{
					$form->removeField('show_delivery_timings', 'seller.params');
					$form->removeField('delivery_hours', 'seller.seller_timings');
				}

				$enable_pickup_availability = $this->helper->config->get('enable_pickup_availability', 1);

				if (!$enable_pickup_availability)
				{
					$form->removeField('show_pickup_availability', 'seller.params');
				}

				$enable_pickup_timings = $this->helper->config->get('enable_pickup_timings', 1);

				if (!$enable_pickup_timings)
				{
					$form->removeField('show_pickup_timings', 'seller.params');
					$form->removeField('pickup_hours', 'seller.seller_timings');
				}
			}
			elseif ($name == 'com_sellacious.category' && isset($obj->type) && $obj->type == 'seller')
			{
				/** @var JFormFieldCheckMatrix $profileFields */
				$profileFields = $form->getField('profile_fields', 'params');

				// Add delivery and pickup timings profile fields selection
				$profileFields->addRow(
					'seller.delivery_timings',
					'PLG_SYSTEM_SELLACIOUSDELIVERY_CONFIG_PRODUCT_FIELDS_OPTION_DELIVERY_TIMINGS',
					array('profile','admin')
				);

				$profileFields->addRow(
					'seller.pickup_timings',
					'PLG_SYSTEM_SELLACIOUSDELIVERY_CONFIG_PRODUCT_FIELDS_OPTION_PICKUP_TIMINGS',
					array('profile','admin')
				);
			}
			elseif ($name == 'com_sellacioustemplates.template')
			{
				$context = isset($obj->context) ? explode('.', $obj->context) : array();

				if (isset($context[0]) && in_array($context[0], array('view_order', 'backoffice_order')))
				{
					$form->loadFile(__DIR__ . '/forms/view_template.xml', false);
				}
			}
			elseif ($name == 'com_sellacious.shippingrule')
			{
				JHtml::_('jquery.framework');
				JHtml::_('script', 'plg_system_sellaciousdelivery/field.timecombo.js', array('version' => S_VERSION_CORE, 'relative' => true));
				JHtml::_('script', 'plg_system_sellaciousdelivery/view.shiprule.js', array('version' => S_VERSION_CORE, 'relative' => true));

				$form->loadFile($this->pluginPath . '/config.xml', false, '//config');

				$my_handlers = array();
				$this->onCollectHandlers('com_sellacious.shipment', $my_handlers);

				$array      = is_object($data) ? ArrayHelper::fromObject($data) : (array) $data;
				$methodName = ArrayHelper::getValue($array, 'method_name', '');

				if (array_key_exists($methodName, $my_handlers))
				{
					$shippingRuleId = ArrayHelper::getValue($array, 'id', 0);
					$ownedBy        = ArrayHelper::getValue($array, 'owned_by', 0);

					if ($onlyOwnShipRule)
					{
						$ownedBy = $me->id;
					}

					if ($ownedBy)
					{
						$sellerShippingId = $this->getSellerShipRuleId($ownedBy, $shippingRuleId, $methodName);

						if ($sellerShippingId)
						{
							$this->app->enqueueMessage(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPPING_RULE_EXISTS'), 'warning');
						}
					}
					else
					{
						$this->app->enqueueMessage(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPPING_RULE_SELECT_SELLER'), 'warning');
						$form->removeGroup('params.plg_system_sellaciousdelivery');
						$form->removeGroup('plg_system_sellaciousdelivery');
					}

					JHtml::_('jquery.framework');
					JHtml::_('script', 'plg_system_sellaciousdelivery/shipment.js', array('version' => S_VERSION_CORE, 'relative' => true));

					if ($timeSelection != 2)
					{
						$form->removeField('slot_rates', 'params.plg_system_sellaciousdelivery');
					}

					$form->setFieldAttribute('slot_rates', 'seller_uid', $ownedBy, 'params.plg_system_sellaciousdelivery');
					$form->setFieldAttribute('slot_rates', 'shipping_rule_id', $shippingRuleId, 'params.plg_system_sellaciousdelivery');
				}
			}
			elseif ($name == 'com_sellacious.config')
			{
				JHtml::_('jquery.framework');
				JHtml::_('script', 'plg_system_sellaciousdelivery/view.config.js', array('version' => S_VERSION_CORE, 'relative' => true));

				$shippedBy    = $this->helper->config->get('shipped_by');
				$flatShipping = $this->helper->config->get('flat_shipping');
				$scope        = $this->helper->config->get('delivery_time_scope', 'product');

				if ($shippedBy == 'shop' && $flatShipping == 1 && $scope == 'shipping')
				{
					$label = JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_FIELD_DELIVERY_TIME_SCOPE_NOTE_LABEL');
					$form->setFieldAttribute('delivery_time_scope_note', 'label', $label, 'com_sellacious');
				}
			}
			elseif (strpos($name, 'subform.plg_system_sellaciousdelivery.next_hour_rates') !== -1)
			{
				if ($timeSelection != 2)
				{
					$form->removeField('add_to_slot_rates');
				}
			}
		}

		return true;
	}

	/**
	 * Runs on content preparation
	 *
	 * @param   string $context The context for the data
	 * @param   mixed  $data    An object containing the data for the form.
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 *
	 * @throws  Exception
	 */
	public function onContentPrepareData($context, $data)
	{
		parent::onContentPrepareData($context, $data);

		if (is_object($data))
		{
			if ($context == 'com_sellacious.user' || $context == 'com_sellacious.profile')
			{
				$registry = new Registry($data);
				$seller   = $registry->get('seller');

				if (!isset($seller->id))
				{
					return true;
				}

				$sellerUser = JTable::getInstance('Seller', 'SellaciousTable');
				$sellerUser->load($seller->id);

				$sellerUid = $sellerUser->get('user_id');

				$delivery = $this->helper->seller->getTimings($sellerUid, 'delivery_hours');
				$pickup   = $this->helper->seller->getTimings($sellerUid, 'pickup_hours');

				if ($data->seller && isset($data->seller->seller_timings))
				{
					if (is_array($data->seller->seller_timings))
					{
						$data->seller->seller_timings['delivery_hours'] = $delivery;
						$data->seller->seller_timings['pickup_hours']   = $pickup;
					}
					else
					{
						$data->seller->seller_timings->delivery_hours = $delivery;
						$data->seller->seller_timings->pickup_hours   = $pickup;
					}
				}
			}
			elseif ($context == 'com_sellacious.shippingrule')
			{
				$my_handlers = array();
				$this->onCollectHandlers('com_sellacious.shipment', $my_handlers);

				if ($data->id && array_key_exists($data->method_name, $my_handlers))
				{
					$config                    = array();
					$config['next_hour_rates'] = StoreTimingsHelper::getNextTimeRates($data->id);

					$data->plg_system_sellaciousdelivery = $config;
				}
			}
		}

		return true;
	}

	/**
	 * The before save event.
	 *
	 * @param   string   $context  The context
	 * @param   mixed    $data     The data
	 * @param   boolean  $isNew    Is new item
	 *
	 * @return  boolean
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onContentBeforeSave($context, $data, $isNew)
	{
		if ($context == 'com_sellacious.user' || $context == 'com_sellacious.profile')
		{
			// Validating timings parameters
			$params = new Registry(ArrayHelper::getValue($data, 'params', array()));

			if (isset($params->get('slot_window')->m) && $params->get('slot_window')->m < 0)
			{
				throw new Exception(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOT_WINDOW_INVALID'));
			}

			if ($params->get('slot_limit') < 0)
			{
				throw new Exception(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SLOT_LIMIT_INVALID'));
			}

			if (isset($params->get('delivery_prep_time')->m) && $params->get('delivery_prep_time')->m < 0)
			{
				throw new Exception(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_PREP_TIME_INVALID'));
			}
		}
		elseif ($context == 'com_sellacious.shippingrule')
		{
			$my_handlers = array();
			$this->onCollectHandlers('com_sellacious.shipment', $my_handlers);

			$post    = $this->app->input->get('jform', array(), 'Array');
			$method  = $data->get('method_name');
			$ownedBy = $data->get('owned_by');

			if (array_key_exists($method, $my_handlers))
			{
				if (!$ownedBy)
				{
					throw new Exception(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPPING_RULE_SELECT_SELLER'));
				}

				$sellerShippingId = $this->getSellerShipRuleId($ownedBy, $data->get('id'), $method);

				if ($sellerShippingId)
				{
					throw new Exception(JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPPING_RULE_EXISTS'));
				}

				$config    = ArrayHelper::getValue($post, 'plg_system_sellaciousdelivery');
				$nextRates = ArrayHelper::getValue($config, 'next_hour_rates');

				if ($nextRates)
				{
					StoreTimingsHelper::validateNextTimeRates($nextRates);
				}
			}
		}

		return true;
	}

	/**
	 * Method is called right after an item is saved
	 *
	 * @param   string  $context  The calling context
	 * @param   object  $table    A JTable object
	 * @param   bool    $isNew    If the content is just about to be created
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 *
	 * @throws  Exception
	 */
	public function onContentAfterSave($context, $table, $isNew)
	{
		$session = JFactory::getSession();
		$data    = $this->app->input->get('jform', array(), 'array');

		if ($context == 'com_sellacious.user' || $context == 'com_sellacious.profile')
		{
			// Save Seller Timings
			$sellerTimings = ArrayHelper::getValue($data, 'seller.seller_timings', array());

			if (!empty($sellerTimings))
			{
				$deliveryTimings = ArrayHelper::getValue($sellerTimings, 'delivery_hours', array());
				$pickupTimings   = ArrayHelper::getValue($sellerTimings, 'pickup_hours', array());
				$sellerUid       = $table->get('id');

				$this->helper->seller->saveTimings($deliveryTimings, 'delivery_hours', $sellerUid);
				$this->helper->seller->saveTimings($pickupTimings, 'pickup_hours', $sellerUid);

				StoreTimingsHelper::saveSlots($deliveryTimings, $sellerUid, 'delivery');
				StoreTimingsHelper::saveSlots($pickupTimings, $sellerUid, 'pickup');
			}
		}
		elseif ($context == 'com_sellacious.shippingrule')
		{
			$my_handlers = array();
			$this->onCollectHandlers('com_sellacious.shipment', $my_handlers);

			if (isset($data['method_name']) && array_key_exists($data['method_name'], $my_handlers))
			{
				$shippingRuleId = $table->get('id');

				if (!$shippingRuleId)
				{
					return true;
				}

				if ($data['method_name'] != 'delivery_shipping')
				{
					return true;
				}

				if ($slotRates = ArrayHelper::getValue($data, 'params.plg_system_sellaciousdelivery.slot_rates'))
				{
					$flatRate = ArrayHelper::getValue($slotRates, 'flat_rate', 0);
					$variable = ArrayHelper::getValue($slotRates, 'variable_rates', 0);

					if (!$variable)
					{
						// Save same rate for all slots
						$owned_by = isset($data['owned_by']) ? $data['owned_by'] : $table->get('owned_by');
						StoreTimingsHelper::saveSlotsFlatRate($shippingRuleId, $owned_by, $flatRate);
					}
					else
					{
						// Save variable slots
						$ratesInSession = $session->get('slot_rates', array(), 'shippingrule');
						$slotsInSession = ArrayHelper::getValue($ratesInSession, 'slots', array());

						foreach ($slotsInSession as $slotId => $slotData)
						{
							$amount = ArrayHelper::getValue($slotData, 'amount', 0.00);
							StoreTimingsHelper::saveSlotRate($shippingRuleId, $slotId, $amount);
						}
					}
				}

				$config    = ArrayHelper::getValue($data, 'plg_system_sellaciousdelivery');
				$nextRates = ArrayHelper::getValue($config, 'next_hour_rates');

				if ($nextRates)
				{
					StoreTimingsHelper::saveNextTimeRates($nextRates, $shippingRuleId);
				}

				$nextRatesDeleteIds = ArrayHelper::getValue($config, 'next_hour_rates_delete_ids');

				if ($nextRatesDeleteIds)
				{
					StoreTimingsHelper::deleteNextTimeRates(explode(',', $nextRatesDeleteIds));
				}
			}

			// Reset in session
			$session->set('slot_rates', null, 'shippingrule');
		}

		return true;
	}

	/**
	 * Method to save delivery date and slots on order
	 *
	 * @param   string    $context
	 * @param   stdClass  $order
	 * @param   array     $products
	 * @param   Cart      $cart
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onAfterPlaceOrder($context, $order, $products, $cart)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($context == 'com_sellacious.cart' && $scope == 'shipping' && $timeSelection > 0)
		{
			$itemisedShip  = $this->helper->config->get('itemised_shipping', SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT);
			$shipInProduct = $this->helper->config->get('product_select_shipping', true);

			if ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT)
			{
				foreach ($products as $product)
				{
					$orderItemId  = $product->id;
					$uid          = $product->item_uid;
					$shippingData = (array) $cart->getItemParam($uid, 'shippingform');

					if (!$shippingData)
					{
						continue;
					}

					$this->saveOrderDeliverySlot($shippingData, $orderItemId, null);
				}
			}
			elseif ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_SELLER)
			{
				$sellers      = $cart->getSellers();
				$shippingData = $cart->getParam('shippingform');

				foreach ($sellers as $seller)
				{
					$sellerUid = $seller->seller_uid;

					if (!isset($shippingData->$sellerUid))
					{
						continue;
					}

					$shipData = json_decode(json_encode($shippingData->$sellerUid), true);

					if ($shipInProduct)
					{
						foreach ($products as $product)
						{
							$orderItemId = $product->id;

							if ($product->seller_uid != $sellerUid || !isset($shipData[$product->item_uid]))
							{
								continue;
							}

							$sData = $shipData[$product->item_uid];

							$this->saveOrderDeliverySlot($sData, $orderItemId, $sellerUid);
						}
					}
					else
					{
						$this->saveOrderDeliverySlot($shipData, null, $sellerUid, $products);
					}
				}
			}
		}
	}

	/**
	 * Method to update order delivery date status on payment
	 *
	 * @param   string    $context The context
	 * @param   \stdClass $payment The payment object
	 *
	 * @return  void
	 *
	 * @throws  \Exception
	 *
	 * @since   2.0.0
	 */
	public function onAfterOrderPayment($context, $payment)
	{
		if ($context == 'com_sellacious.order' && $payment->state >= 1)
		{
			$orderId    = $payment->order_id;
			$orderItems = $this->helper->order->getOrderItems($orderId);

			foreach ($orderItems as $orderItem)
			{
				$table = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');
				$table->load(array('order_item_id' => $orderItem->id));

				if ($table->get('id'))
				{
					$table->set('status', 1);
					$table->check();
					$table->store();
				}

				$this->handleStock($orderId, $orderItem->id, $orderItem->item_uid);
			}
		}
	}

	/**
	 * Handle stock after order status change
	 *
	 * @param   string  $context   The calling context
	 * @param   int     $order_id  The order id
	 *
	 * @throws  \Exception
	 *
	 * @since 2.0.0
	 */
	public function onAfterOrderChange($context, $order_id)
	{
		if ($context == 'com_sellacious.order')
		{
			$orderItems = $this->helper->order->getOrderItems($order_id, 'a.id, a.item_uid');
			$lastItemUid = $this->helper->order->loadResult(array(
				'list.select' => 'a.item_uid',
				'list.from'   => '#__sellacious_order_status',
				'list.where'  => array(
					'a.order_id = ' . $order_id,
					'a.item_uid <> ""',
				),
				'list.order'  => 'a.id DESC',
			));

			foreach ($orderItems as $orderItem)
			{
				$item_uid = $orderItem->item_uid;

				if ($item_uid == $lastItemUid)
				{
					$this->handleStock($order_id, $orderItem->id, $item_uid);
				}
			}
		}
	}

	/**
	 * Method to call after delete
	 *
	 * @param   string  $context The context
	 * @param   JTable  $table   The table object
	 *
	 * @since   2.0.0
	 */
	public function onContentAfterDelete($context, $table)
	{
		if ($context == 'com_sellacious.order')
		{
			$pk = $table->get('id');

			$query = 'DELETE a FROM #__sellacious_order_delivery_slot AS a ';
			$query .= 'INNER JOIN #__sellacious_order_items AS b ON b.id = a.order_item_id ';
			$query .= 'WHERE b.order_id = ' . $pk;

			$this->db->setQuery($query)->execute();
		}
		elseif ($context == 'com_sellacious.shippingrule')
		{
			$pk = $table->get('id');

			StoreTimingsHelper::removeShippingRuleRates($pk);
			StoreTimingsHelper::deleteShipRuleNextRates($pk);
		}
	}

	/**
	 * Method to add replacements related to delivery to view template
	 *
	 * @param   string   $context      The context for the data
	 * @param   Registry $data         The data object
	 * @param   array    $replacements Array of replacements by code
	 *
	 * @since   2.0.0
	 */
	public function onParseProductRow($context, $data, &$replacements)
	{
		if ($context == 'com_sellacious.templates.product')
		{
			if (!$data instanceof Registry)
			{
				$data = new Registry($data);
			}

			$orderslot = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');
			$orderslot->load(array('order_item_id' => $data->get('id')));

			if ($orderslot->get('id'))
			{
				$fromTime = JFactory::getDate($orderslot->get('slot_from_time'));
				$toTime   = JFactory::getDate($orderslot->get('slot_to_time'));
				$values   = array(
					'product_delivery_date' => $fromTime->format('d M, Y'),
					'product_delivery_slot' => ($fromTime == $toTime) ? $fromTime->format('g:i A') : $fromTime->format('g:i A') . ' - ' . $toTime->format('g:i A'),
				);
			}
			else
			{
				$values = array(
					'product_delivery_date' => '',
					'product_delivery_slot' => '',
				);
			}

			$replacements = array_merge($replacements, $values);
		}
	}

	/**
	 * Method to add store availability filter
	 *
	 * @param   string  $context  The calling context
	 * @param   array   $filters  List of filters
	 *
	 * @since   2.0.0
	 */
	public function onLoadProductFilters($context, &$filters)
	{
		if ($context == 'com_sellacious.products')
		{
			$filters['delivery_pickup_availability'] = 'Sellacious\ProductsFilter\DeliveryPickupAvailability';
		}
	}

	/**
	 * Method to add more options to sortoptions field
	 *
	 * @param   string                          $context     Calling context
	 * @param   stdClass[]                      $options     Options to load
	 * @param   string                          $columnName  Name of column
	 * @param   SellaciousFormFieldSortOptions  $field       The field
	 *
	 * @since   2.0.0
	 */
	public function onLoadFieldOptions($context, &$options, $columnName, $field)
	{
		if ($context == 'com_sellacious.field.sortoptions' && $field->id == 'jform_com_sellacious_sellacious_filters_sections')
		{
			$tmp = (object) array(
				'text'   => 'PLG_SYSTEM_SELLACIOUSDELIVERY_FIELD_FILTER_SECTIONS_DELIVERY_PICKUP_AVAILABILITY_LABEL',
				'value'  => 'delivery_pickup_availability',
				'column' => $columnName,
			);

			$options[$tmp->value] = $tmp;
		}
	}

	/**
	 * Method to alter the list query
	 *
	 * @param   string          $context  The calling context
	 * @param   JDatabaseQuery  $query    The database query to process
	 * @param   JObject         $state    The model state object
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onProcessListQuery($context, $query, $state)
	{
		if ($context == 'com_sellacious.stores')
		{
			$date    = JFactory::getDate();
			$now     = $date->format('H:i:s');
			$weekDay = $date->format('N') - 1;

			// Filter by store delivery availability
			$delivery = $state->get('filter.delivery_available', 0);

			if ($delivery)
			{
				$query->join('left', $this->db->qn('#__sellacious_seller_timings', 'td') . ' ON td.seller_uid = a.user_id AND td.week_day = ' . $weekDay . ' AND td.state = 1 AND td.type = ' . $this->db->quote('delivery_hours'));
				$query->where('td.from_time <= ' . $this->db->quote($now) . ' AND td.to_time >= ' . $this->db->quote($now));
			}

			// Filter by store pickup availability
			$pickup = $state->get('filter.pickup_available', 0);

			if ($pickup)
			{
				$query->join('left', $this->db->qn('#__sellacious_seller_timings', 'tp') . ' ON tp.seller_uid = a.user_id AND tp.week_day = ' . $weekDay . ' AND tp.state = 1 AND tp.type = ' . $this->db->quote('pickup_hours'));
				$query->where('tp.from_time <= ' . $this->db->quote($now) . ' AND tp.to_time >= ' . $this->db->quote($now));
			}

			$query->group('a.user_id');
		}
	}

	/**
	 * Ajax function to Purge cache
	 *
	 * @return  void
	 *
	 * @since   2.0.0
	 *
	 * @throws  \Exception
	 */
	public function onAjaxSellaciousdelivery()
	{
		$method = $this->app->input->getString('method', '');

		try
		{
			if (!empty($method) && method_exists(__CLASS__, $method))
			{
				$data = $this->$method();

				echo new JResponseJson($data, '');
			}
		}
		catch (Exception $e)
		{
			echo new JResponseJson($e);
		}

		JFactory::getApplication()->close();
	}

	/**
	 * Ajax function to get slots for timing
	 *
	 * @return array
	 *
	 * @since   2.0.0
	 *
	 * @throws  \Exception
	 */
	public function onAjaxGetSlots()
	{
		$app           = JFactory::getApplication();
		$slotDate      = $app->input->get('slot_date', '');
		$weekDay       = $app->input->get('week_day', '');
		$code          = $app->input->getString('code', '');
		$sellerUid     = $app->input->getInt('seller_uid');
		$shipRuleId    = $app->input->getInt('shipping_rule_id', 0);
		$shipRates     = $app->input->get('ship_rates', '');
		$fieldName     = $app->input->getString('field_name', '');
		$slotType      = $app->input->getString('slot_type', 'delivery');
		$fieldId       = $app->input->get('field_id', '', 'RAW');
		$getAllSlots   = $app->input->getString('get_all_slots') == 'true' ? true : false;
		$html          = '';
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);

		if ($weekDay != '')
		{
			$weekDay = (int) $weekDay;
			$sDate   = JFactory::getDate($slotDate);
			$sDay    = $this->helper->core->getWeekDay($sDate);

			if ($sDay != $weekDay)
			{
				$diff = $weekDay - $sDay;
				$sDate->modify($diff . ' day');

				$slotDate = $sDate->format('Y-m-d');
			}
		}

		$this->helper->product->parseCode($code, $productId, $variantId, $sUid);

		$sellerUid = $sellerUid ?: $sUid;

		if ($timeSelection == 2)
		{
			$shipRuleId  = $shipRates != 'false' ? $shipRuleId : null;
			$slotOptions = StoreTimingsHelper::getAvailableSlots($productId, $sellerUid, $slotDate, '', $getAllSlots, $slotType, $shipRuleId);

			$fieldName = $fieldName ?: (!empty($code) ? 'delivery_slot_' . strtolower($code) : 'delivery_slot');
			$fieldId   = $fieldId ?: 'delivery_slot';

			if (empty($slotOptions))
			{
				$slotOptions = array_merge(array('' => JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SELECT_' . strtoupper($slotType) . '_SLOT')), $slotOptions);
			}

			$html = JHtml::_('select.genericlist', $slotOptions, $fieldName, 'class="delivery_slot"', 'id', 'title', null, $fieldId);
		}

		if ($timeSelection == 3)
		{
			$slotDate = JFactory::getDate(str_replace(array('AM', 'PM'), '', $slotDate));
		}
		else
		{
			$slotDate = JFactory::getDate($slotDate);
		}

		$disabledIntervals = array();

		if ($timeSelection == 3 && $sellerUid && $slotDate)
		{
			$weekDay = $this->helper->core->getWeekDay($slotDate);
			$timing  = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');

			if ($timing)
			{
				$fromTime = JFactory::getDate($timing->from_time);
				$toTime   = JFactory::getDate($timing->to_time);
				$minTime   = JFactory::getDate('02:00');
				$maxTime   = JFactory::getDate('22:00');

				if ($fromTime >= $minTime)
				{
					$fromTime->modify('-1 hour');
				}

				if ($toTime <= $maxTime)
				{
					$toTime->modify('+1 hour');
				}

				$slotDay = $slotDate->format('Y-m-d');

				$disabledInterval        = array();
				$disabledInterval['min'] = $slotDay . ' 00:00';
				$disabledInterval['max'] = $slotDay . ' ' . $fromTime->format('H:i');
				$disabledIntervals[]     = $disabledInterval;

				$disabledInterval        = array();
				$disabledInterval['min'] = $slotDay . ' ' . $toTime->format('H:i');
				$disabledInterval['max'] = $slotDay . ' 23:59';
				$disabledIntervals[]     = $disabledInterval;
			}
		}

		$date = array('weekday' => $slotDate->format('D'), 'day' => $slotDate->format('F d'));
		$data = array('date' => $date, 'formatted_date' => $slotDate->format('Y-m-d'), 'html' => $html, 'disabled_intervals' => $disabledIntervals);

		return $data;
	}

	/**
	 * Method to call when getting time slots
	 *
	 * @param   string  $context     Calling context
	 * @param   Slots   $slots       Timing slots object
	 * @param   int     $sellerUid   Seller user id
	 * @param   int     $weekDay     Day no. of the week
	 * @param   string  $type        Type of time slots
	 * @param   int     $shipRuleId  Shipping rule id
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onGetTimeSlots($context, &$slots, $sellerUid, $weekDay = 0, $type = 'delivery', $shipRuleId = 0)
	{
		if ($context == 'com_sellaciousdelivery.slots' && $type == 'delivery')
		{
			$timing = $this->helper->seller->getTiming($sellerUid, $weekDay, $type . '_hours');

			if (!$timing)
			{
				return;
			}

			$timeSlots = StoreTimingsHelper::getSlots($timing->id);

			if (!$timeSlots)
			{
				return;
			}

			$session        = JFactory::getSession();
			$ratesInSession = $session->get('slot_rates', array(), 'shippingrule');
			$slotsInSession = ArrayHelper::getValue($ratesInSession, 'slots', array());

			foreach ($timeSlots as $timeSlot)
			{
				$slot = new SlotRate(SellaciousHelperSeller::DAYS[$weekDay], JFactory::getDate($timeSlot->from_time)->format('H:i'), JFactory::getDate($timeSlot->to_time)->format('H:i'));
				$slot->setId($timeSlot->id);

				if ($shipRuleId)
				{
					$filters = array('shippingrule_id' => $shipRuleId, 'slot_id' => $timeSlot->id);
					$rate    = StoreTimingsHelper::getTimeSlotRate($filters);

					$slot->setAmount($rate->amount);
				}

				$amnt = ArrayHelper::getValue($slotsInSession, $timeSlot->id . '.amount');

				if ($amnt != '')
				{
					$slot->setAmount($amnt);
				}

				$slots->add($slot);
			}
		}
	}

	/**
	 * Method to call on saving slot rates
	 *
	 * @param   string  $context  Calling context
	 * @param   array   $data     Slot data to save
	 *
	 * @since   2.0.0
	 */
	public function onSaveSlotRates($context, $data)
	{
		if ($context == 'com_sellaciousdelivery.slots')
		{
			// Set to session (temporarily)
			$session = JFactory::getSession();
			$session->set('slot_rates', $data, 'shippingrule');
		}
	}

	/**
	 * Method to add timings information for seller
	 *
	 * @param   string    $context  The context
	 * @param   Registry  $seller   The seller object
	 * @param   array     $availability     Seller information
	 * @param   array     $sellerTimings     Seller information
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onLoadSellerTimings($context, $seller, &$availability, &$sellerTimings)
	{
		if ($context == 'com_sellacious.store')
		{
			$sellerUid          = $seller->get('user_id');
			$sellerParams       = new Registry($seller->get('params'));
			$sellerTimeZone     = StoreTimingsHelper::getUserTimezone($sellerUid);
			$now                = JFactory::getDate('now', $sellerTimeZone);
			$weekDay            = $this->helper->core->getWeekDay($now);
			$today_availability = $sellerParams->get('today_availability', '');
			$today_availability = $today_availability ? JFactory::getDate($now->format('Y-m-d', true) . ' ' . $today_availability) : null;

			$enable_d_timings      = $this->helper->config->get('enable_delivery_timings', 1);
			$show_delivery_timings = $this->helper->config->get('show_delivery_timings', 1);
			$show_delivery_timings = $enable_d_timings && $sellerParams->get('show_delivery_timings', $show_delivery_timings);

			$enable_p_timings    = $this->helper->config->get('enable_pickup_timings', 1);
			$show_pickup_timings = $this->helper->config->get('show_pickup_timings', 1);
			$show_pickup_timings = $enable_p_timings && $sellerParams->get('show_pickup_timings', $show_pickup_timings);

			$enable_d_availability      = $this->helper->config->get('enable_delivery_availability', 1);
			$show_delivery_availability = $this->helper->config->get('show_delivery_availability', 1);
			$show_delivery_availability = $enable_d_availability && $sellerParams->get('show_delivery_availability', $show_delivery_availability);

			$enable_p_availability    = $this->helper->config->get('enable_pickup_availability', 1);
			$show_pickup_availability = $this->helper->config->get('show_pickup_availability', 1);
			$show_pickup_availability = $enable_p_availability && $sellerParams->get('show_pickup_availability', $show_pickup_availability);

			$deliveryTimings = $this->helper->seller->getTimings($sellerUid, 'delivery_hours', 1);

			foreach ($deliveryTimings as &$deliveryTiming)
			{
				$slotInfo = array();
				$fromTime = JFactory::getDate($deliveryTiming['from_time']);
				$toTime   = JFactory::getDate($deliveryTiming['to_time']);
				$slots    = StoreTimingsHelper::getSlotsForTime($deliveryTiming['id'], $sellerUid, $fromTime, $toTime);

				$slotLimit = $sellerParams->get('slot_limit', 0);
				$slotLimit = $slotLimit == 0 ? JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SELLER_TIMINGS_TOTAL_SLOTS_UNLIMITED') : $slotLimit;

				$slotInfo[] = array(
					'label' => JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SELLER_TIMINGS_TOTAL_SLOTS'),
					'value' => count($slots)
				);
				$slotInfo[] = array(
					'label' => JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SELLER_TIMINGS_SLOT_LIMIT'),
					'value' => $slotLimit
				);

				$deliveryTiming['slot_info'] = $slotInfo;
			}

			$sellerTimings['delivery_hours'] = array(
				'title' => JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_HOURS'),
				'list'  => $deliveryTimings,
				'show'  => $show_delivery_timings
			);

			$sellerTimings['pickup_hours'] = array(
				'title' => JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_PICKUP_HOURS'),
				'list'  => $this->helper->seller->getTimings($sellerUid, 'pickup_hours', 1),
				'show'  => $show_pickup_timings
			);

			if ($show_delivery_availability)
			{
				$literals = array(
					'not_available' => 'PLG_SYSTEM_SELLACIOUSDELIVERY_NOT_AVAILABLE_DELIVERY_HOURS',
					'available'     => 'PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_HOURS_AVAILABLE',
					'available_in'  => 'PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_HOURS_AVAILABLE_IN',
				);

				$delAvail            = $this->helper->seller->getStoreAvailability($sellerTimings['delivery_hours']['list'], $literals);
				$todayDeliveryTiming = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');

				if ($todayDeliveryTiming)
				{
					if (strpos($delAvail, 'timing_available') !== -1 && $today_availability)
					{
						$delAvail .= '<span class=\'timing_available\'> ' . JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_HOURS_AVAILABLE_TODAY') . $today_availability->format('g:i A') . '</span>';
					}

					$delAvail .= ' <a class="hasTooltip" href="javascript:void;" title="' . JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_HOURS_AVAILABLE_INFO') . '"><i class="fa fa-info-circle"></i></a>';
				}

				$availability['delivery_hours_availability'] = $delAvail;
			}

			if ($show_pickup_availability)
			{
				$literals = array(
					'not_available' => 'PLG_SYSTEM_SELLACIOUSDELIVERY_NOT_AVAILABLE_PICKUP_HOURS',
					'available'     => 'PLG_SYSTEM_SELLACIOUSDELIVERY_PICKUP_HOURS_AVAILABLE',
					'available_in'  => 'PLG_SYSTEM_SELLACIOUSDELIVERY_PICKUP_HOURS_AVAILABLE_IN',
				);

				$availability['pickup_hours_availability'] = $this->helper->seller->getStoreAvailability($sellerTimings['pickup_hours']['list'], $literals);
			}
		}
	}

	/**
	 * Method to show additional order item info
	 *
	 * @param   string    $context The context for the data
	 * @param   stdClass  $item    The order item
	 * @param   array     $info    Array of rendered order item information
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function onRenderOrderItem($context, $item, &$info)
	{
		if ($context == 'com_sellacious.order')
		{
			$oslot = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');
			$oslot->load(array('order_item_id' => $item->id));

			if ($oslot->get('id'))
			{
				$info[] = $this->renderLayout('default_orderitem', $oslot->getProperties(1));
			}
		}
	}

	/**
	 * Method to load template variables
	 *
	 * @param  AbstractTemplate  $template
	 * @param  string            $context
	 *
	 * @since  2.0.0
	 */
	public function onLoadTemplateVariables($template, $context)
	{
		$name = $template->getName();
		$parts = explode('_', $name);

		if ($context == 'com_sellacious.template.grid' && isset($parts[0]) && $parts[0] == 'order')
		{
			$template->addVariable(new TemplateVariable('product_delivery_date', JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHORTCODE_ORDER_PRODUCTS_PRODUCT_DELIVERY_DATE'), ''));
			$template->addVariable(new TemplateVariable('product_delivery_slot', JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHORTCODE_ORDER_PRODUCTS_PRODUCT_DELIVERY_SLOT'), ''));
		}
	}

	/**
	 * Method to load template values for variables
	 *
	 * @param   AbstractTemplate  $template
	 * @param   stdClass          $data
	 * @param   array             $replacements
	 *
	 * @since   2.0.0
	 */
	public function onLoadTemplateValues($template, $data, &$replacements)
	{
		$name = $template->getName();
		$parts = explode('_', $name);

		if (isset($parts[0]) && $parts[0] == 'order')
		{
			$slot = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');
			$slot->load(array('order_item_id' => $data->id));

			if ($slot->get('id'))
			{
				$fromTime = JFactory::getDate($slot->get('slot_from_time'));
				$toTime   = JFactory::getDate($slot->get('slot_to_time'));

				$replacements['product_delivery_date'] = $fromTime->format('d M, Y');
				$replacements['product_delivery_slot'] = ($fromTime == $toTime) ? $fromTime->format('g:i A') : $fromTime->format('g:i A') . ' - ' . $toTime->format('g:i A');
			}
		}
	}

	/**
	 * Stock handling for the items in the order when a status change is triggered.
	 *
	 * @param   int  $order_id       The concerned order id
	 * @param   int  $order_item_id  The concerned order item id
	 * @param   int  $item_uid       The uid of the order item (PnVnSnBnDn)
	 *
	 * @return  bool
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	public function handleStock($order_id, $order_item_id, $item_uid)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$history       = $this->helper->order->getStatusLog($order_id, $item_uid);

		if (empty($history))
		{
			return true;
		}

		if ($timeSelection == 3)
		{
			return true;
		}

		$recent = array_shift($history);

		// If this status has no effect on stock, exit.
		if ($recent->s_stock == '')
		{
			return true;
		}

		while ($previous = array_shift($history))
		{
			if (in_array($previous->s_stock, array('A', 'R', 'O')))
			{
				break;
			}
		}

		$c_handle = $recent->s_stock;
		$o_handle = $previous ? $previous->s_stock : '';
		$o_handle = $o_handle ? $o_handle : '-';
		$h_key    = $o_handle . $c_handle;

		// Key => [A=Stock, R=Reserved, O=Sold]
		$handle_map = array(
			//  '-A' => array( '0',  '0',  '0'),
			'-R' => array('-1', '+1', '0'),
			'-O' => array('-1', '0', '+1'),
			'AR' => array('-1', '+1', '0'),
			'AO' => array('-1', '0', '+1'),
			'RA' => array('+1', '-1', '0'),
			'RO' => array('0', '-1', '+1'),
			'OA' => array('+1', '0', '-1'),
			'OR' => array('0', '+1', '-1'),
		);

		$change = ArrayHelper::getValue($handle_map, $h_key);

		if (empty($change))
		{
			return true;
		}

		$table = JTable::getInstance('OrderDeliverySlot', 'SellaciousTable');
		$table->load(array('order_item_id' => $order_item_id));

		$from_time = $table->get('slot_from_time');
		$to_time   = $table->get('slot_to_time');
		$full_day  = $table->get('full_day', 0);

		$this->helper->product->parseCode($item_uid, $product_id, $variant_id, $seller_uid);

		$slotData = array(
			'seller_uid'     => $seller_uid,
			'product_id'     => $product_id,
			'full_day'       => $full_day,
			'slot_from_time' => $from_time,
			'slot_to_time'   => $to_time
		);

		$slotLimit = JTable::getInstance('ProductSellerSlotLimit', 'SellaciousTable');
		$slotLimit->load($slotData);

		$c_stock = $slotLimit->get('slot_count', 0);

		if (!$slotLimit->get('id'))
		{
			$slotLimit->bind($slotData);

			$seller       = $this->helper->seller->getItem(array('user_id' => $seller_uid));
			$sellerParams = new Registry($seller->params);
			$c_stock      = $sellerParams->get('slot_limit', 0);

			$slotLimit->set('slot_limit', $c_stock);
		}

		// Slot limit will change with only one unit per order, so quantity not being used
		$c_stock += $change[0];

		$slotLimit->set('slot_count', $c_stock);
		$slotLimit->check();
		$slotLimit->store();

		return true;
	}

	/**
	 * Returns handlers to the shipment methods that will be managed by this plugin
	 *
	 * @param   string  $context    The calling context, must be 'com_sellacious.shipment' to effect
	 * @param   array   &$handlers  ByRef, associative array of handlers
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	public function onCollectHandlers($context, array &$handlers)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($context == 'com_sellacious.shipment' && $scope == 'shipping' && $timeSelection > 0)
		{
			$handlers['delivery_shipping'] = new ShippingHandler('delivery_shipping', JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPMENT_DELIVERY'), true, false, false);

			if ($timeSelection == 2)
			{
				$handlers['pickup_shipping'] = new ShippingHandler('pickup_shipping', JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_SHIPMENT_PICKUP'), true, false, false);
			}
		}
	}

	/**
	 * Triggers rate request for the given items
	 *
	 * @param   string    $context    The calling context, must be 'com_sellacious.shipment' to effect
	 * @param   object    $rule       The shipment rule object loaded from the database
	 * @param   array     $items      The products for which the rate is to be fetched. Product must include:
	 *                                Product id, weight+unit and dimensions+unit. May include any additional info.
	 * @param   Registry  $ship_from  The shipper contact and address:
	 *                                Name, Company, Phone, City, State, Country, ZIP, Residential
	 * @param   Registry  $ship_to    The recipient contact and address:
	 *                                Name, Company, Phone, City, State, Country, ZIP, Residential
	 * @param   array     $quotes     The quotes for the item with given shipment parameters. (ByRef) Will be updated.
	 *
	 * @return  bool
	 * @throws  Exception
	 *
	 *  @since  2.0.0
	 */
	public function onRequestFreightQuote($context, $rule, $items, $ship_from, $ship_to, &$quotes)
	{
		$cart     = $this->helper->cart->getCart();
		$handlers = array();
		$this->onCollectHandlers($context, $handlers);

		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($scope != 'shipping' || $timeSelection < 1)
		{
			return true;
		}

		$itemisedShip  = $this->helper->config->get('itemised_shipping', SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT);
		$shipInProduct = $this->helper->config->get('product_select_shipping', true);

		if (!array_key_exists($rule->method_name, $handlers) || $itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_CART)
		{
			return true;
		}

		$quoteIds = array_filter($this->app->input->get('seller_shipment', array(), 'array'));
		$shipForm = array_filter($this->app->input->get('cart', array(), 'array'));

		try
		{
			// Must set active rule first
			$this->setActiveRule($rule);

			foreach ($items as $item)
			{
				if ($timeSelection > 0)
				{
					if ($rule->method_name == 'pickup_shipping')
					{
						if ($timeSelection != 2)
						{
							continue;
						}

						$rate = new \stdClass();

						$rate->service      = 'pickup_' . $rule->id;
						$rate->serviceName  = null;
						$rate->amount       = 0;
						$rate->deliveryDate = null;
						$rate->TransitTime  = null;

						$quotes[] = $rate;

						continue;
					}

					$sellerUid = $item->seller_uid;
					$itemUid   = $this->helper->product->getCode($item->product_id, $item->variant_id, $item->seller_uid);

					if ($this->rule->owned_by != $sellerUid)
					{
						return true;
					}

					$amount = null;

					if ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT)
					{
						$shippingData = (array) $cart->getItemParam($itemUid, 'shippingform');

						$cartItem    = $cart->getItem($itemUid);
						$shipQuoteId = $cartItem->getShipQuoteId();

						if ($shipQuoteId && $shippingData)
						{
							$parts  = explode('___', $shipQuoteId);
							$ruleId = $parts[1];

							$deliveryDate   = ArrayHelper::getValue($shippingData, 'delivery.delivery_date.date');
							$deliveryAmount = ArrayHelper::getValue($shippingData, 'delivery.delivery_date.amount');
							$deliverySlot   = ArrayHelper::getValue($shippingData, 'delivery.delivery_slot');

							$rates  = StoreTimingsHelper::getDaySlotRates($ruleId, $sellerUid, $deliveryDate);
							$amount = $deliveryAmount ?: ArrayHelper::getValue($rates, $deliverySlot, null);
						}
					}
					elseif ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_SELLER)
					{
						$ruleIds      = $cart->getSellerShipping('ruleId');
						$shippingData = $cart->getParam('shippingform');

						$quote    = ArrayHelper::getValue($quoteIds, $sellerUid . '.' . $itemUid);
						$shipData = $quote ? ArrayHelper::getValue($shipForm, $sellerUid . '.' . $itemUid . '.' . $quote) : null;

						if (!$shipData && $shippingData && isset($shippingData->$sellerUid))
						{
							$shipData = json_decode(json_encode($shippingData->$sellerUid), true);
						}

						if ($ruleIds && isset($ruleIds[$sellerUid]) && $shipData)
						{
							if ($shipInProduct)
							{
								$ruleId = null;

								if ($quote)
								{
									$ruleId = $this->rule->id;
								}
								else
								{
									foreach ($ruleIds[$sellerUid] as $uid => $shipRuleId)
									{
										if ($uid != $itemUid)
										{
											continue;
										}

										$shipData = ArrayHelper::getValue($shipData, $uid);
										$ruleId   = $shipRuleId;
									}
								}
							}
							else
							{
								$ruleId = $ruleIds[$sellerUid];
							}

							if ($ruleId && $shipData)
							{
								$deliveryDate   = ArrayHelper::getValue($shipData, 'delivery.delivery_date.date');
								$deliveryAmount = ArrayHelper::getValue($shipData, 'delivery.delivery_date.amount');
								$deliverySlot   = ArrayHelper::getValue($shipData, 'delivery.delivery_slot');

								$rates  = StoreTimingsHelper::getDaySlotRates($ruleId, $sellerUid, $deliveryDate);
								$amount = $deliveryAmount ?: ArrayHelper::getValue($rates, $deliverySlot, null);
							}
						}
					}

					$rate = new \stdClass();

					$rate->service      = 'deliveryslot_' . $rule->id;
					$rate->serviceName  = null;
					$rate->amount       = $amount;
					$rate->deliveryDate = null;
					$rate->TransitTime  = null;
					$rate->tbd          = !$amount;

					$quotes[] = $rate;
				}
			}
		}
		catch (Exception $e)
		{
			JLog::add($e->getMessage(), JLog::WARNING, 'jerror');
		}

		return true;
	}

	/**
	 * Method to get shipping rule id made by a seller for a method
	 *
	 * @param   int     $sellerUid   Seller user id
	 * @param   int     $excludeId   Id of shipping rule to be excluded
	 * @param   string  $methodName  Name of the method
	 *
	 * @return  int
	 *
	 * @since   2.0.0
	 */
	protected function getSellerShipRuleId($sellerUid, $excludeId, $methodName)
	{
		$filters = array(
			'list.select' => 'a.id',
			'list.where'  => array(
				'owned_by = ' . $sellerUid,
				'method_name = ' . $this->db->q($methodName),
				'id != ' . (int) $excludeId
			)
		);

		return $this->helper->shippingRule->loadResult($filters);
	}

	/**
	 * Get the form for selected handler. Some plugins may set up different forms for different context/handlers.
	 *
	 * @param   stdClass  $method   payment method object
	 * @param   string    $context  The Payment Context, i.e. for what the payment is to be made (unused)
	 *
	 * @return  SimpleXMLElement|false
	 *
	 * @since   2.0.0
	 */
	protected function getFormXml($method, $context = null)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($scope != 'shipping' || $timeSelection < 1)
		{
			return false;
		}

		$xml   = false;
		$parts = explode('_', $context, 3);

		if ((is_numeric(strpos($context, 'deliveryslot')) || is_numeric(strpos($context, 'pickup'))) && count($parts) == 2)
		{
			JForm::addFieldPath(__DIR__ . '/forms/fields');
			$xml = parent::getFormXml($method, $context);
		}

		return $xml;
	}

	/**
	 * Method to render shipping form data for item in cart
	 *
	 * @param   string  $context    The calling context
	 * @param   string  $itemUid    The code of the item in question
	 * @param   array   $builtData  The built shipping form data
	 * @param   array   $rawData    The shipping form data in raw form
	 * @param   string  $html       Html to render
	 *
	 * @since   2.0.0
	 */
	public function onRenderItemShippingData($context, $itemUid, $builtData, $rawData, &$html)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($scope != 'shipping' || $timeSelection != 2)
		{
			return;
		}

		if ($context == 'com_sellacious.cart')
		{
			$rawData      = (array) $rawData;
			$deliveryDate = ArrayHelper::getValue($rawData, 'delivery.delivery_date.date');
			$deliveryId   = ArrayHelper::getValue($rawData, 'delivery.delivery_date.id');
			$deliverySlot = ArrayHelper::getValue($rawData, 'delivery.delivery_slot');

			if (!$deliveryDate)
			{
				return;
			}

			if ($deliveryId)
			{
				$deliveryDate = JFactory::getDate($deliveryDate);
				$date         = $deliveryDate->format('F d, Y g:i A');

				$html .= JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_BY') . $date;
			}
			else
			{
				$deliveryDate = JFactory::getDate($deliveryDate);
				$date         = $deliveryDate->format('F d, Y');
				$slots        = explode('-', $deliverySlot);
				$slots        = array_map('trim', $slots);

				$html .= JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_ON') . $date;

				if (isset($slots[1]))
				{
					$slotStart = JFactory::getDate($slots[0]);
					$slotEnd   = JFactory::getDate($slots[1]);
					$slots[0]  = $slotStart->format('g:i A');
					$slots[1]  = $slotEnd->format('g:i A');

					$html .= ' ' . implode(' - ', $slots);
				}
			}
		}
	}

	/**
	 * Adds plugin form(s) to the sellacious payment params form for user to fil, such as card details etc.
	 *
	 * @param   string    $context      The calling context
	 * @param   JForm     $form         The form to manipulate.
	 * @param   stdClass  $method       Shipment Method record object.
	 * @param   string    $serviceName  Any specific service to load the form for
	 * @param   mixed     $data         Form data
	 * @param   string    $itemUid      Product code
	 *
	 * @return  bool
	 *
	 * @since   2.0.0
	 */
	public function onLoadShippingForm($context, $form, $method, $serviceName = null, $data = null, $itemUid = null)
	{
		if (!parent::onLoadShippingForm($context, $form, $method, $serviceName, $data, $itemUid))
		{
			return false;
		}

		$itemisedShip  = $this->helper->config->get('itemised_shipping', SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT);
		$shipInProduct = $this->helper->config->get('product_select_shipping', true);
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);
		$scope         = $this->helper->config->get('delivery_time_scope', 'product');

		if ($scope != 'shipping' || $timeSelection < 1)
		{
			return true;
		}

		$registry = new Registry($data);
		$now      = JFactory::getDate();
		$today    = $now->format('Y-m-d', true);

		if ($sellerUid = $method->owned_by)
		{
			$seller       = $this->helper->seller->getItem(array('user_id' => $sellerUid));
			$sellerParams = new Registry($seller->params);
			$parts        = explode('_', $serviceName, 3);

			if (isset($parts[0]) && $parts[0] == 'pickup')
			{
				// Check for enabled pickup week days
				$pickupTimings = $this->helper->seller->getTimings($sellerUid, 'pickup_hours', 1);
				$disabledDays  = StoreTimingsHelper::getDisabledDays($pickupTimings);
				$disabledDates = array();
				$nxtPickupDate = StoreTimingsHelper::getNextAvailableDate(  $sellerUid, 'pickup_hours', true);

				if ($nxtPickupDate)
				{
					if (!$data)
					{
						$form->setValue('pickup_date', 'pickup', $nxtPickupDate->format('Y-m-d'));
					}

					$pickupDate      = $nxtPickupDate->modify('-1 day')->format('Y-m-d 00:00:00', true);
					$disabledDates[] = $nxtPickupDate->format('Y-m-d');
					$form->setFieldAttribute('pickup_date', 'selected_date', $pickupDate, 'pickup');
				}

				$form->setFieldAttribute('pickup_date', 'shipping_rule_id', $method->id, 'pickup');
				$form->setFieldAttribute('pickup_date', 'time_selection', $timeSelection, 'pickup');
				$form->setFieldAttribute('pickup_date', 'disabled_dates', json_encode($disabledDates), 'pickup');
				$form->setFieldAttribute('pickup_date', 'disabled_days', implode(',', $disabledDays), 'pickup');
				$form->setFieldAttribute('pickup_date', 'seller_uid', $sellerUid, 'pickup');

				if ($field = $form->getField('pickup_slot', 'pickup'))
				{
					$form->setFieldAttribute('pickup_date', 'slot_picker_id', $field->id, 'pickup');
				}

				if ($timeSelection != 2)
				{
					$form->removeField('pickup_slot', 'delivery');
				}

				if ($data)
				{
					$slotDate = $registry->get('pickup.pickup_date.date');

					$form->setFieldAttribute('pickup_slot', 'seller_uid', $sellerUid, 'pickup');
					$form->setFieldAttribute('pickup_slot', 'slot_date', $slotDate, 'pickup');
				}

				return true;
			}

			// If shipping rule is for next hours/days, nothing to do
			if (count($parts) > 2)
			{
				return true;
			}

			list($nextDate, $disabledDates) = StoreTimingsHelper::processDatesForShipRule($method->id);

			if (!$data)
			{
				$form->setValue('delivery_date', 'delivery', $nextDate->format('Y-m-d'));
			}

			$form->setFieldAttribute('delivery_date', 'selected_date', $nextDate->modify('-1 day')->toSql(), 'delivery');

			// Check for enabled delivery week days
			$deliveryTimings = $this->helper->seller->getTimings($sellerUid, 'delivery_hours', 1);
			$disabledDays    = StoreTimingsHelper::getDisabledDays($deliveryTimings);
			$disabledDates[] = $nextDate->format('Y-m-d');
			$days            = array_keys(SellaciousHelperSeller::DAYS);

			// If all days are disabled, then no point of showing slot selection
			if ($disabledDays === $days)
			{
				$form->removeField('delivery_slot', 'delivery');
			}

			$field = $form->getField('delivery_slot', 'delivery');

			$form->setFieldAttribute('delivery_date', 'product_uid', $itemUid, 'delivery');
			$form->setFieldAttribute('delivery_date', 'shipping_rule_id', $method->id, 'delivery');
			$form->setFieldAttribute('delivery_date', 'seller_uid', $sellerUid, 'delivery');
			$form->setFieldAttribute('delivery_date', 'time_selection', $timeSelection, 'delivery');
			$form->setFieldAttribute('delivery_date', 'disabled_days', implode(',', $disabledDays), 'delivery');
			$form->setFieldAttribute('delivery_date', 'disabled_dates', json_encode($disabledDates), 'delivery');

			if ($field)
			{
				$form->setFieldAttribute('delivery_date', 'slot_picker_id', $field->id, 'delivery');
			}

			$shipForm = array_filter($this->app->input->get('cart', array(), 'array'));

			if ($shipForm)
			{
				$shipFormData = new Registry($shipForm);

				if ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_SELLER)
				{
					$quoteIds     = array_filter($this->app->input->get('seller_shipment', array(), 'array'));
					$quoteIdsData = new Registry($quoteIds);

					if ($shipInProduct)
					{
						$quote  = $quoteIdsData->get($sellerUid . '.' . $itemUid);
						$keys   = array_filter(array($sellerUid, $itemUid, $quote, 'delivery', 'delivery_date'));
						$key    = implode('.', $keys);
						$amount = $shipFormData->get($key . '.amount');
						$id     = $shipFormData->get($key . '.id');

						if ($amount && $id)
						{
							$form->removeField('delivery_slot', 'delivery');
						}
					}
					else
					{
						$quote  = $quoteIdsData->get($sellerUid);
						$keys   = array_filter(array($sellerUid, $quote, 'delivery', 'delivery_date'));
						$key    = implode('.', $keys);
						$amount = $shipFormData->get($key . '.amount');
						$id     = $shipFormData->get($key . '.id');

						if ($amount && $id)
						{
							$form->removeField('delivery_slot', 'delivery');
						}
					}
				}
				elseif ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_PRODUCT)
				{
					$quoteIds     = array_filter($this->app->input->get('shipment', array(), 'array'));
					$quoteIdsData = new Registry($quoteIds);
					$quote        = $quoteIdsData->get($itemUid);
					$keys         = array_filter(array($itemUid, $quote, 'delivery', 'delivery_date', 'amount'));
					$key          = implode('.', $keys);
					$amount       = $shipFormData->get($key);

					if ($amount)
					{
						$form->removeField('delivery_slot', 'delivery');
					}
				}
			}

			if ($timeSelection != 2)
			{
				$form->removeField('delivery_slot', 'delivery');
			}
			elseif ($itemisedShip == SellaciousHelperShipping::SHIPPING_SELECTION_SELLER)
			{
				$form->setFieldAttribute('delivery_slot', 'seller_uid', $sellerUid, 'delivery');
				$form->setFieldAttribute('delivery_slot', 'item_uid', $itemUid, 'delivery');
			}

			$weekDay          = $this->helper->core->getWeekDay($now);
			$deliveryHour     = $this->helper->seller->getTiming($sellerUid, $weekDay, 'delivery_hours');
			$nxtAvailableDate = StoreTimingsHelper::getNextAvailableDate($sellerUid);

			if (!$deliveryHour && $nxtAvailableDate)
			{
				$selectedDate = $nxtAvailableDate->format('Y-m-d 00:00:00', true);
				$form->setFieldAttribute('delivery_date', 'selected_date', $selectedDate, 'delivery');
			}
			elseif ($deliveryHour)
			{
				$to_time            = JFactory::getDate($today . ' ' . $deliveryHour->to_time);
				$today_availability = $sellerParams->get('today_availability', '');
				$today_availability = $today_availability ? JFactory::getDate($today . ' ' . $today_availability) : null;
				$todayNow           = JFactory::getDate($now->format('Y-m-d H:i:s', true));

				if ($today_availability && $today_availability <= $to_time && $todayNow > $today_availability)
				{
					$selectedDate = $nxtAvailableDate ? $nxtAvailableDate->format('Y-m-d 00:00:00', true) : '';
					$form->setFieldAttribute('delivery_date', 'selected_date', $selectedDate, 'delivery');
				}
			}

			if ($data)
			{
				$slotDate = $registry->get('delivery.delivery_date.date');
				$slotId   = $registry->get('delivery.delivery_date.id');

				if (!$shipForm && $slotId)
				{
					$form->setFieldAttribute('delivery_slot', 'required', true, 'delivery');
				}

				$form->setFieldAttribute('delivery_slot', 'seller_uid', $sellerUid, 'delivery');
				$form->setFieldAttribute('delivery_slot', 'shipping_rule_id', $method->id, 'delivery');
				$form->setFieldAttribute('delivery_slot', 'slot_date', $slotDate, 'delivery');
			}
		}

		return true;
	}

	/**
	 * Method called on validating shipment form in cart
	 *
	 * @param   string  $context    Calling context
	 * @param   mixed   $data       Shipment form data
	 * @param   array   $errors     Errors
	 * @param   int     $sellerUid  Seller user id
	 *
	 * @since   2.0.0
	 */
	public function onValidateCartShipmentForm($context, $data, &$errors, $sellerUid = null)
	{
		if ($context == 'com_sellacious.cart')
		{
			$registry = new Registry($data);
			$amount   = $registry->get('delivery.delivery_date.amount');
			$date     = $registry->get('delivery.delivery_date.date');
			$slot     = $registry->get('delivery.delivery_slot');

			$pickupDate = $registry->get('pickup.pickup_date.date');
			$pickupSlot = $registry->get('pickup.pickup_slot');
			$date       = $date ?: $pickupDate;
			$slot       = $slot ?: $pickupSlot;

			$sellerTimeZone = null;

			if ($sellerUid)
			{
				$sellerTimeZone = StoreTimingsHelper::getUserTimezone($sellerUid);
			}

			if ($date && $slot)
			{
				$parts = explode('-', $slot);

				if (isset($parts[1]))
				{
					$now  = JFactory::getDate('now', $sellerTimeZone);
					$date = $amount ? $date : $date . ' ' . $parts[1];
					$dt   = JFactory::getDate($date, $sellerTimeZone);

					if ($dt < $now)
					{
						$errors[] = JText::_('PLG_SYSTEM_SELLACIOUSDELIVERY_DELIVERY_DATE_PASSED');
					}
				}
			}
		}
	}


	/**
	 * Method to save order delivery slot
	 *
	 * @param   array  $data         The form data
	 * @param   int    $orderItemId  Order item id
	 * @param   int    $sellerUid    Seller user id
	 * @param   array  $orderItems   List of Order items (optional)
	 *
	 * @since   2.0.0
	 */
	protected function saveOrderDeliverySlot($data, $orderItemId, $sellerUid, $orderItems = null)
	{
		$timeSelection = $this->helper->config->get('delivery_time_selection', 0);

		$date   = ArrayHelper::getValue($data, 'delivery.delivery_date.date');
		$id     = ArrayHelper::getValue($data, 'delivery.delivery_date.id');
		$amount = ArrayHelper::getValue($data, 'delivery.delivery_date.amount');
		$slot   = ArrayHelper::getValue($data, 'delivery.delivery_slot');
		$source = '';

		$pickupDate = ArrayHelper::getValue($data, 'pickup.pickup_date.date');
		$pickupSlot = ArrayHelper::getValue($data, 'pickup.pickup_slot');

		if ($amount)
		{
			$source = 'other_rates';

			$parts = explode('_', $id);

			if (isset($parts[3]))
			{
				$source .= '_' . $parts[3];
			}
		}
		elseif (!empty($pickupDate))
		{
			$amount = null;
			$date   = $pickupDate;
			$slot   = $pickupSlot;
			$source = 'pickup';
		}

		$date = str_replace(array('AM', 'PM'), '', $date);

		if (!$date)
		{
			return;
		}

		if ($timeSelection == 3 || $amount)
		{
			$fromDate = JFactory::getDate($date)->format('Y-m-d H:i:s');
			$toDate   = $fromDate;
		}
		else
		{
			$slotRange = explode('-', $slot);
			$slotRange = array_map('trim', $slotRange);

			$fromDate = $date;
			$toDate   = $date;
			$fromDate .= ' ' . (isset($slotRange[1]) ? JFactory::getDate($slotRange[0])->format('H:i:s') : '00:00:00');
			$toDate   .= ' ' . (isset($slotRange[1]) ? JFactory::getDate($slotRange[1])->format('H:i:s') : '00:00:00');
		}

		if ($orderItems && is_array($orderItems))
		{
			foreach ($orderItems as $orderItem)
			{
				$orderItemId = $orderItem->id;

				if ($orderItem->seller_uid != $sellerUid)
				{
					continue;
				}

				$data = array(
					'order_item_id'  => $orderItemId,
					'slot_from_time' => $fromDate,
					'slot_to_time'   => $toDate,
				);

				$data['source'] = $source;

				StoreTimingsHelper::saveOrderDeliverySlot($data);
			}
		}
		else
		{
			$data = array(
				'order_item_id'  => $orderItemId,
				'slot_from_time' => $fromDate,
				'slot_to_time'   => $toDate,
			);

			$data['source'] = $source;

			StoreTimingsHelper::saveOrderDeliverySlot($data);
		}
	}

	/**
	 * Method to load more scripts in cart
	 *
	 * @param    string           $context  The context
	 * @param    \SellaciousView  $view     The view data
	 *
	 * @return   void
	 *
	 * @throws   \Exception
	 *
	 * @since    2.0.0
	 */
	public function onBeforeDisplayView($context, $view)
	{
		if ($context == 'com_sellacious.cart')
		{
			JHtml::_('jquery.framework');

			JHtml::_('script', 'com_sellacious/plugin/select2-3.5/select2.js', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('stylesheet', 'com_sellacious/plugin/select2-3.5/select2.css', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('stylesheet', 'plg_system_sellaciousdelivery/glyphicons.css', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('stylesheet', 'plg_system_sellaciousdelivery/bootstrap-datetimepicker.css', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('script', 'plg_system_sellaciousdelivery/moment-with-locales.js', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('script', 'plg_system_sellaciousdelivery/bootstrap-datetimepicker.js', array('version' => S_VERSION_CORE, 'relative' => true));
			JHtml::_('script', 'plg_system_sellaciousdelivery/cart.slots.js', array('version' => S_VERSION_CORE, 'relative' => true));

			JText::script('PLG_SYSTEM_SELLACIOUSDELIVERY_CALENDAR');
		}
	}
}
